# -*- coding: utf-8 -*-
###########################################################################
# Eole NG - 2007  
# Copyright Pole de Competence Eole  (Ministere Education - Academie Dijon)
# Licence CeCill  cf /root/LicenceEole.txt
# eole@ac-dijon.fr 
#  
# Utilitaire pour les actions de gestion de navigation web
#  
###########################################################################

""" outil pour l'interface d'observation web """
from twisted.python import log
from ead2.backend.actions.tools import make_js_link, make_long_js_link
from ead2.backend.config.filenames import navigation_access, dg_restart, dg_reload, navigation_filters, ajout_filtres
from os.path import isfile, join
import commands, os

# fonction de redemarrage
def bastion_restart():
    a,b = commands.getstatusoutput('/etc/init.d/bastion restart')
    log.msg('restarting bastion ||||%s,|||| %s'%(a,b))
    if a == 0:
        return True
    else:
        return False

def dansguardian_restart():
    a = os.system(dg_restart)
    log.msg('restarting_dansguardian')
    log.msg('ret : %s'%a)
    if a == 0:
        return True
    else:
        return False

def dansguardian_reload():
    a = os.system(dg_reload)
    log.msg('reloading_dansguardian')
    log.msg('ret : %s'%a)
    if a == 0:
        return True
    else:
        return False


def get_visit_menu(server_nb):
    return {'href':make_js_link(server_nb, 'navigation_visit'),
            'icone':'/image/amon/visit.png',
            'libelle': 'Visites des sites',
            'title': "Consultation des sites visités les 15 derniers jours"}

def get_refused_menu(server_nb):
    return {'href':make_js_link(server_nb, 'navigation_refused'),
            'icone':'/image/amon/refused.png',
            'title':"Consultations des tentatives d'accès sites refusés",
            'libelle':"Sites refusés"}

def get_filters_menu(server_nb):
    return {'href':make_js_link(server_nb, 'navigation_filters'),
            'icone':'/image/amon/filters.png',
            'title':'Gestion des filtres internet',
            'libelle':'Filtrage'}

def get_access_menu(server_nb):
    return {'href':make_js_link(server_nb, 'navigation_access'),
            'icone':'/image/amon/access.png',
            'libelle':'Accessibilité',
            'title':"Gestion des accès internet"}

def get_return_btn(server_nb):
    """ renvoit la description du bouton retour """
    href = make_js_link(server_nb, 'navigation')
    icone = '/image/back.png'
    libelle = 'Retour vers le menu principal'
    title = 'Revenir en arrière'
    return {'href':href, 'icone':icone, 'libelle':libelle, 'title':title}

# fonctions de gestion du fichier de filtrage utilisateur (moderateur, interdit, listeblanche)
filtergroupf = navigation_access['filtergroupslist']
def get_user_list():
    if isfile(filtergroupf): return file(filtergroupf).read().splitlines()
    return []

def get_users():
    """filter1 = utilisateur normaux, grp par défaut, non utilisé
    filter2 = modérateurs
    filter3 = utilisateur interdits
    filter4 = mode liste blanche
    """
    moderateurs = list()
    banned_users = list()
    white_users = list()
    for userline in get_user_list():
        user, group = userline.split('=')
        if group == 'filter2': moderateurs.append(user)
        elif group == 'filter3': banned_users.append(user)
        elif group == 'filter4': white_users.append(user)
    return dict(moderateurs = moderateurs,
                banned_users = banned_users,
                white_users = white_users)

def remove_user(user):
    users = get_user_list()
    for i in users:
        if i.split('=')[0] == user: users.remove(i)
    return users

def autorise_user(user):
    return write_user_file(remove_user(user))

def add_user(user, mod):
    if mod == 'moderateur': mod = 'filter2'
    elif mod == 'banned_user': mod = 'filter3'
    elif mod == 'white_user': mod = 'filter4'
    a = remove_user(user)
    a.append('%s=%s'%(user, mod))
    return write_user_file(a)

def write_user_file(user_list):
    try:
        fh = file(filtergroupf, 'w')
        for i in user_list:
            fh.write('%s\n'%i)
        fh.close()
        return True
    except:
        log.err("une erreur s'est produite a l'ecriture des statuts utilisateurs")
        users = get_user_list()
        fh = file(filtergroupf, 'w')
        for i in users:
            fh.write('%s\n'%i)
        fh.close()
        return False

# fonctions de gestion du fichier d'interdiction de poste pour le web seulement
forbidden_poste_filename = navigation_access['forbidden_poste']

def get_forbidden_postes():
    """ renvoit la liste des postes """
    if isfile(forbidden_poste_filename):return file(forbidden_poste_filename).read().splitlines()
    else:return []

def forbid_poste(ip, hdeb, hfin, jdeb, jfin):
    """ ajoute un poste à interdire """
    postes = get_forbidden_postes()
    texte = "%s#%s#%s#%s#%s"%(ip, hdeb, hfin, jdeb, jfin)
    if texte not in postes:
        postes.append(texte)
        return write_postes(postes)

def autorize_postes(ips):
    """ enleve une liste de postes """
    postes = get_forbidden_postes()
    for ip in ips:
        for poste in postes:
            if ip in poste:
                postes.remove(poste)
                break
    return write_postes(postes)

def write_postes(postes):
    """ ecrit les postes dans le fichier d'interdiction """
    try:
        po = file(forbidden_poste_filename, 'w')
        for p in postes:
            po.write('%s\n'%p)
        po.close()
        return True
    except:
        po = file(forbidden_poste_filename, 'w')
        postes = get_forbidden_postes()
        for p in postes:
            po.write('%s\n'%p)
        po.close()
        return False


# fonctions de gestion du fichier d'interdiction total de poste
all_forbidden_poste_filename = navigation_access['all_forbidden_poste']

def get_all_forbidden_postes():
    """ renvoit la liste des postes """
    if isfile(all_forbidden_poste_filename):return file(all_forbidden_poste_filename).read().splitlines()
    else:return []

def all_forbid_poste(ip, hdeb, hfin, jdeb, jfin):
    """ ajoute un poste à interdire """
    postes = get_all_forbidden_postes()
    texte = "%s"%ip#"%s#%s#%s#%s#%s"%(ip, hdeb, hfin, jdeb, jfin)
    if texte not in postes:
        postes.append(texte)
        return write_all_postes(postes)

def autorize_all_postes(ips):
    """ enleve une liste de postes """
    postes = get_all_forbidden_postes()
    for ip in ips:
        for poste in postes:
            if ip in poste:
                postes.remove(poste)
                break
    return write_all_postes(postes)

def write_all_postes(postes):
    """ ecrit les postes dans le fichier d'interdiction """
    try:
        po = file(all_forbidden_poste_filename, 'w')
        for p in postes:
            po.write('%s\n'%p)
        po.close()
        return True
    except:
        postes = get_all_forbidden_postes()
        po = file(all_forbidden_poste_filename, 'w')
        for p in postes:
            po.write('%s\n'%p)
        po.close()
        return False

# fonctions de gestion du fichier des sites sur listes blanches utilisateur
grey_list_filename = navigation_access['greylist']
def get_grey_list():
    """ renvoit la liste des postes accessibles pour les utilisateurs sur liste blanche """
    if isfile(grey_list_filename): return file(grey_list_filename).read().splitlines()
    else: return []

def add_site(site):
    """ ajoute un site à la liste """
    sites = get_grey_list()
    if site not in sites:
        sites.append(site)
        write_sites(sites)
    return True

def write_sites(sites):
    """ ecrit les sites accessibles pour les utilisateurs de liste blanche """
    try:
        sit = file(grey_list_filename, 'w')
        for s in sites:
            sit.write('%s\n'%s)
        sit.close()
        return True
    except:
        sites = get_grey_list()
        sit = file(grey_list_filename, 'w')
        for s in sites:
            sit.write('%s\n'%s)
        sit.close()
        return False

def del_site(sites):
    """ enleve une liste de postes """
    all_sites = get_grey_list()
    for site in sites:
        for s in all_sites:
            if site == s:
                all_sites.remove(s)
                break
    return write_sites(all_sites)

# fonctions de gestion des filtres optionnnels
filtres_users_filename = navigation_filters['filtres_users']
filtres_opt_filename = navigation_filters['filtres_opt']

def ajout_filtre():
    a = os.system(ajout_filtres)
    log.msg('Application des filtres optionnels')
    log.msg('ret : %s'%a)

def write_filtres(filtres):
    a = file(filtres_users_filename, 'w')
    for filtre in filtres:
        a.write("%s\n"%filtre['name'])
    a.close()
    return True

def get_filtres_options():
    """recupere les filtres optionnels disponible"""
    options = []
    if not isfile(filtres_opt_filename): return options
    opts = file(filtres_opt_filename, 'r').read().splitlines()
    for opt in opts:
        clef = opt.split('#')[0]
        desc = opt.split('#')[1]
        options.append({'name':clef, 'libelle':desc})
    return options

def get_filtres_user_options():
    """recupere les filtres optionnels actives """
    if isfile(filtres_users_filename):
        users_opt = file(filtres_users_filename).read().splitlines()
        return users_opt
    else:
        return []


# fonction de gestion de fichiers texte #

def get_lines(filename):
    """ renvoit le contenu d'un fichier sous forme de tableau """
    if isfile(filename): return file(filename).read().splitlines()
    else: return []

def write_file(filename, lines):
    fp = file(filename, 'w')
    fp.writelines('\n'.join(lines))
    fp.close()

def add_lines(filename, lines):
    """ ajoute une ligne dans un fichier (sans doublon) """
    if type(lines) != list:
        lines = [lines]
    oldies = get_lines(filename)
    for line in lines:
        if line and line not in oldies:
            oldies.append(line)
    try:
        write_file(filename, oldies)
    except:
        return False
    return True
        
def del_lines(filename, lines):
    """ supprime des lignes dans un fichier """
    if type(lines) != list:
        lines = [lines]
    oldies = get_lines(filename)
    for line in lines:
        if line in oldies:
            oldies.remove(line)
    try:
        write_file(filename, oldies)
    except:
        return False
    return True

def add_url(filename, url):
    """ ajout d'une url dans un fichier """
    if url.startswith('http://'):
        url = url.replace('http://', '', 1)
    return add_lines(filename, url)

def add_extension(filename, ext):
    """ ajoute une extension dans un fichier """
    if not ext.startswith('.'):
        ext = ".%s"%ext
    return add_lines(filename, ext)


