# -*- coding: utf-8 -*-
###########################################################################
# Eole NG - 2007  
# Copyright Pole de Competence Eole  (Ministere Education - Academie Dijon)
# Licence CeCill  cf /root/LicenceEole.txt
# eole@ac-dijon.fr 
#  
# Action filtrage_bayes
#  
# Affiche une évaluation d'une page depuis une url saisi
#       
###########################################################################
""" Action d'evaluation d'url """
from twisted.python import log
from ead2.backend.lib.action import * # import des actions et des types
from ead2.backend.actions.tools import get_request, make_form_link, make_key_form_link
from ead2.backend.actions.tools import format_form_result
from ead2.lib.libead import uni
# import pour l'utilisation du filtre
from filtrage.collect import filter_onepage
from os.path import isfile, join
used_templates = ['main', 'form', 'accordion', 'formselect']
import ead2.backend.actions.amon.amon_tools as t
from ead2.backend.config.filenames import navigation_filters

forbid_filename =  navigation_filters['blacklist']
autorize_filename = navigation_filters['whitelist']

class FiltrageBayes(Action):
    """ interface d'évaluation de page web """
    user_description = Dict(default={}, doc="description de l'éxécutant", keys=['ip', 'name'])
    name = 'filtrage_bayes'
    libelle = "Evaluation d'url"
    description = "Teste une url donnée"
    category = "Outils/Filtrage"
    request = Dict(default={}, doc="arguments de la requete en cour cote frontend",
                                keys=['server', 'action'])
    form_result = Dict(default={}, doc="retour formulaire de la requete en cour coté frontend",
                                    keys = ['url', 'forbid', 'autorize'])
    def execute(self):
        """ renvoit les données pour la mise en forme des actions """
        params, self.server_nb = get_request(self.request)
        self.url = ''
        result = {'titre': self.description, 'sstitre':"Outil de filtrage Bayesien"}
        if self.form_result != {}:
            result.update(self._valid_form())
            self.form_result = {}
        else:
            result.update({'note':'', 'object':''})
        result.update(self._get_form())
        result['forbid_btn'] = self._get_forbid_btn()
        result['autorize_btn'] = self._get_autorize_btn()
        result['current_url'] = {'name':'current_url', 'default_value':self.url}
        result['validate'] = {'href':make_form_link(self.server_nb, self.name, True, ['url']), 
                'icone':'/image/amon/ok.gif', 'libelle':'Evaluer', 'title':'Evaluer une page'}
        result['validation'] = make_key_form_link(self.server_nb, self.name, False, 'url')
        d = {'template':'amon_filtrage_bayes', 'data':{'content':result, 'templates':used_templates}}
        return 0, uni(str(d))

    def _get_form(self):
        """ renvoit la description du formulaire """
        url = {'name':'url','libelle':"Url à évaluer", 'id':'url_input'}
        if self.url:
            url['default_value'] = self.url
        return dict(url=url, focus='url_input')

    def _valid_form(self):
        if  self.form_result.has_key('url'):
            return self._valid_url()
        elif self.form_result.has_key('forbid'):
            return self._valid_forbid()
        else:
            return self._valid_autorize()

    def _valid_url(self):
        """ evalue l'url passée en paramètre """
        if not self.form_result.has_key('url'):
            return '', ''
        result = format_form_result(self.form_result['url'])
        if result.has_key('url'):
            self.url = result['url']
            try:
                return self._get_note(self.url)
            except:
                log.msg("Filtrage Bayesien: erreur à la récupération de la note")
                return dict(note = ["Erreur : erreur à la récupération de la note"], object='')
        return dict(note='', object='')

    def _get_note(self, url):
        note = filter_onepage(url)
        if note == []:
            note = ["Aucune note n'a pu être attribuée à cette page."]
        if 'http' not in url:
            url = 'http://%s'%url
        return dict(note=note, object=url)

    def _valid_forbid(self):
        if not self.form_result.has_key('forbid'):
            return '', ''
        result = format_form_result(self.form_result['forbid'])
        self.url = result['current_url'].strip()
        ret = self._get_note(self.url)
        sites = t.get_lines(autorize_filename)
        if True in [self.url in a for a in sites]:
            ret.update({'message':"Veuillez retirer l'url de la liste de sites interdits avant de l'autoriser.\\n      (Outils-Filtrage-Sites-sites interdits)"})
            return ret
        if not self.url or not '.' in self.url:
            ret.update({'message':"L'url n'est pas valide."})
            return ret
        if not t.add_url(forbid_filename, self.url):
            ret.update({'message':"Erreur d'écriture dans le fichier de sites interdits."})
            log.err("Erreur d'ecriture dans %s"%forbid_filename)
            return ret
        t.dansguardian_reload()
        ret.update({'message':"L'url %s a été ajoutée à la liste des sites interdits.\\n      (Outils-Filtrage-Sites-sites interdits)"%self.url})
        return ret

    def _valid_autorize(self):
        if not self.form_result.has_key('autorize'):
            return '', ''
        result = format_form_result(self.form_result['autorize'])
        self.url = result['current_url'].strip()
        ret = self._get_note(self.url)
        if not self.url or not '.' in self.url:
            ret.update({'message':"L'url n'est pas valide."})
            return ret
        sites = t.get_lines(forbid_filename)
        if True in [self.url in a for a in sites]:
            ret.update({'message':"Veuillez retirer l'url de la liste de sites interdits avant de l'autoriser.\\n      (Outils-Filtrage-Sites-sites autorisés)"})
            return ret
        if not t.add_url(autorize_filename, self.url):
            ret.update({'message':"Erreur d'écriture dans le fichier de sites autorisés."})
            log.err("Erreur d'ecriture dans %s"%autorize_filename)
            return ret
        t.dansguardian_reload()
        ret.update({'message':"L'url %s a été ajoutée à la liste des sites autorisés.\\n      (Outils-Filtrage-Sites-sites autorisés)"%self.url})
        return ret

    def _get_forbid_btn(self):
        """ renvoit la description du bouton d'interdiction de la page """
        return {'href':make_form_link(self.server_nb, self.name, True, ['forbid']), 
                'icone':'/image/amon/refused.png', 'libelle':'Interdire', 'title':'Interdire cette page'}

    def _get_autorize_btn(self):
        """ renvoie la description du bouton de ré-autorisation de la page """
        return {'href':make_form_link(self.server_nb, self.name, True, ['autorize']),
                'icone':'/image/redemarrer.gif', 'libelle':'Ré-autoriser', 'title':'Ré-autoriser cette page'}

