# -*- coding: utf-8 -*-
###########################################################################
# Eole NG - 2007  
# Copyright Pole de Competence Eole  (Ministere Education - Academie Dijon)
# Licence CeCill  cf /root/LicenceEole.txt
# eole@ac-dijon.fr 
#  
# Action navigation_access
#  
# Gere les interdiction d'acces de poste ou d'url ainsi que l'autorisation
# de moderateur
#       
###########################################################################

""" Action de gestion des filtres web sur un amon """

#import sys
#sys.path.append('/usr/share')

from twisted.python import log
from ead2.backend.lib.action import * # import des actions et des types
from ead2.backend.actions.tools import get_request, make_form_link, make_confirm_link, make_js_link, make_long_js_link, make_key_form_link
from ead2.backend.actions.tools import format_form_result
import ead2.backend.actions.amon.amon_tools as t #fonctions de gestion des utilisateurs, postes....
from ead2.backend.config.config import SCRIPT_DIR
from ead2.lib.libead import uni
import commands
from os.path import join


page_msgs = {'poste':"Gestion des postes à interdire", 
             'banned_user':"Gestion des utilisateurs à interdire",
             'moderateur':"Gestion des modérateurs"
             }

#nom des formulaires
form_name = {'poste':['list_add'], 'banned_user':['list_add', 'current_page'], 
            'moderateur':['list_add', 'current_page']}

used_templates = ['main','form', 'formselect', 'accordion']


class NavigationAccess(Action):
    """ interface de gestion des acces internet"""
    user_description = Dict(default={}, doc="description de l'éxécutant", keys=['ip', 'name'])
    name = 'navigation_access'
    libelle = 'Accès internet'
    description = 'Gérer les accès internet'
    category = "Outils/Filtrage"
    request = Dict(default={}, doc="arguments de la requete en cour cote frontend", 
                    keys=['server', 'action', 'current_page'])
    form_result = Dict(default={}, doc="retour formulaire de la requete en cour coté frontend",
                        keys = ['list_suppr', 'list_add', 'current_page', 'all_list_suppr'])
    def execute(self):
        """ renvoit les données pour la mise en forme des actions """
        params, self.server_nb = get_request(self.request)
        self.page = 'poste'
        result = {'titre': self.description}
        if self.form_result != {}:
            if self.form_result.has_key('current_page'):
                self.page = self.form_result['current_page'][0]['value']
            self._valid_form()
            self.form_result = {}
        if params.has_key('current_page'):
            self.page = params['current_page'][0]
        result.update(self._get_form())
        result['sstitre'] =  page_msgs[self.page]
        result['menus'] = _get_menus(self.server_nb, self.name, self.page)
        result['page'] = {'name':'current_page', 'default_value':self.page}
        result['validate'] = {'href':make_form_link(self.server_nb, self.name, True, form_name[self.page]),
                                'icone':'/image/amon/ok.gif', 'libelle':'Valider', 'title':'Valider les changements'}
        result['validation'] = make_key_form_link(self.server_nb, self.name, False, form_name[self.page])
        d = {'template':'amon_navigation_access', 'data':{'content':result, 'templates':used_templates}}
        return 0, uni(str(d))

    def _get_form(self):
        if self.page == 'poste':
            return self._get_poste_form()
        elif self.page == 'banned_user':
            return self._get_banned_user_form()
        elif self.page == 'moderateur':
            return self._get_moderateur_form()

    def _get_poste_form(self):
        """ decrit le formulaire a presenter """
        ip = {'name':'ip', 'onblur':"testVoid('ip', 'error_ip')", 
                    'libelle':"adresse / sous réseau à interdire"}
        heure_deb = {'name':'heure_deb', 'options':[{'name':'heure de début', 'options':get_hours()}],
                        'libelle':"Heure de début"}
        heure_fin = {'name':'heure_fin', 'options':[{'name':'heure de fin', 'options':get_hours(True)}], 
                    'libelle':"Heure de fin"}
        jour_deb = {'name':'jour_deb', 'options':[{'name':'jour de début', 'options':get_days()}], 
                        'libelle':" du "}
        jour_fin = {'name':'jour_fin', 'options':[{'name':'jour de fin', 'options':get_days(True)}],
                        'libelle':" au "}
        niveau = {'options':[("Toute activité réseau",{'name':'niveau', 'value':'all', 'inline':'ok'}),
            ('Seulement le web',{'name':'niveau', 'value':'web', 'checked':'true', 'inline':'ok'})], 
                'libelle':'Niveau de restriction'}
        #postes interdits de web: description de la section de reautorisation
        # une entete de tableau, une selectbox, un bouton valider
        postes = []
        for poste in t.get_forbidden_postes():
            poste = poste.split("#")
            libelle = "%s de %s à %s du %s au %s"%(poste[0],poste[1], poste[2], poste[3], poste[4])
            nom = poste[0]
            postes.append({'name':nom, 'libelle':libelle})
        if postes != []:
            forbidden_poste = {'entete':"Postes interdits d'accès web",
                 'validate':{'href':make_form_link(self.server_nb, self.name, True, ['current_page','list_suppr']), 
                                'icone':'/image/amon/supprimer_big.gif',
                 'libelle':'Réautoriser', 'title':'Réautoriser les postes selectionnés'},
                 'select':{'name':'to_del', 'options':[{'name':"ips/sous-réseau interdits", "options":postes}],
                     'libelle':"Veuillez sélectionner le ou les postes à (ré)autoriser",'multi':'ok'}
                                }
        else:
            forbidden_poste = {"msg": "Aucun poste n'est interdit d'accès web", 'entete':"Postes interdits d'accès web"}
        # postes interdits sur tout le reseau: description de la section de reautorisation
        # une entete de tableau, une selectbox, un bouton valider
        postes = []
        for poste in t.get_all_forbidden_postes():
            poste = poste.split("#")
            libelle = "%s est interdit tout le temps"%poste[0]
            nom = poste[0]
            postes.append({'name':nom, 'libelle':libelle})
        if postes != []:
            all_forbidden_poste = {'entete':"Postes interdits sur tout le réseau (ne tient pas compte des horaires)",
                 'validate':{'href':make_form_link(self.server_nb, self.name, True, ['current_page', 'all_list_suppr']), 'icone':'/image/amon/supprimer_big.gif',
                 'libelle':'Réautoriser', 'title':'Réautoriser les postes selectionnés'},
                 'select':{'name':'to_del', 'options':[{'name':"ips/sous-réseau interdits", "options":postes}],
                     'libelle':"Veuillez sélectionner le ou les postes à (ré)autoriser",'multi':'ok'}
                                }
        else:
            all_forbidden_poste = {"msg": "Aucun poste n'est interdit totalement", 
                        'entete':"Postes interdits sur tout le réseau (ne tient pas compte des horaires)"}
        return dict(ip=ip, heure_deb=heure_deb, heure_fin=heure_fin, jour_deb=jour_deb, 
                jour_fin=jour_fin, niveau=niveau, listing=forbidden_poste, 
                all_forbidden_poste=all_forbidden_poste, focus='ip')

    def _get_banned_user_form(self):
        ban_name = {'name':'name', 'onblur':"testVoid('name', 'error_name')",
                'libelle':"Login de l'utilisateur à interdire"}
        users = list()
        for user in t.get_users()['banned_users']:
            users.append({'name':user, 'libelle':user})
        banned_users = {'entete':"Utilisateurs interdits"}
        if users != []:
            banned_users.update({
               'validate':{'href':make_form_link(self.server_nb, self.name, True, ['current_page', 'list_suppr']),
                   'icone':'/image/amon/supprimer_big.gif',
                  'libelle':'Réautoriser', 'title':'Réautoriser les utilisateurs selectionnés'},
              'select':{'name':'to_del', 'options':[{'name':"utilisateurs interdits", "options":users}],
                  'libelle':"Veuillez sélectionner le ou les utilisateurs à (ré)autoriser", 'multi':"ok"}})
        else:
            banned_users.update({"msg": "Aucun utilisateur n'est interdit"})
        return dict(ban_name=ban_name, listing=banned_users, focus='name')

    def _get_moderateur_form(self):
        moderateur_name = {'name':'name', 'onblur':"testVoid('name', 'error_name')", 
                'libelle':"Login du modérateur à ajouter"}
        moderateurs = []
        for moderateur in t.get_users()['moderateurs']:
            moderateurs.append({'name':moderateur, 'libelle':moderateur})
        moderateur_users =  {'entete':"Supprimer un modérateur"}
        if moderateurs != []:
            moderateur_users.update({
                'validate':{'href':make_form_link(self.server_nb, self.name, True, ['current_page', 'list_suppr']), 
                        'icone':'/image/amon/supprimer_big.gif',
                    'libelle':'Supprimer', 'title':'Supprimer les modérateurs selectionnés'},
                'select':{'name':'to_del', 'options':[{'name':"moderateurs", "options":moderateurs}],
                    'libelle':"Veuillez sélectionner le ou les modérateurs à supprimer", 'multi':"ok"}})
        else:
            moderateur_users.update({"msg": "Aucun modérateur n'a été défini"})
        return dict(moderateur_name=moderateur_name, listing=moderateur_users, focus='name')

    def _valid_form(self):
        """ valide les retours de formulaire"""
        if self.page == 'poste':
            return self._valid_poste()
        elif self.page == 'banned_user':
            return self._valid_banned_user()
        elif self.page == 'moderateur':
            return self._valid_moderateur()

    def _valid_poste(self):
        """ valide l'interdiction de poste """
        if not self.form_result.has_key('list_add') and not self.form_result.has_key('list_suppr') and not self.form_result.has_key('all_list_suppr'):
            log.msg('il mnaque un truc')
            return ''
        # on interdit des postes
        if self.form_result.has_key('list_add'):
            result = self.form_result['list_add']
            poste = format_form_result(result)
            if poste.has_key('ip'):ip=poste['ip']
            else:return ''
            if poste.has_key('heure_deb'):deb=poste['heure_deb']
            else:return ''
            if poste.has_key('heure_fin'):fin=poste['heure_fin']
            else:return ''
            if poste.has_key('jour_deb'):jdeb=poste['jour_deb']
            else:return ''
            if poste.has_key('jour_fin'):jfin=poste['jour_fin']
            else:return ''
            if poste.has_key('niveau'):niveau=poste['niveau']
            else:return ''
            if niveau == 'web':
                t.forbid_poste(ip, deb, fin, jdeb, jfin)
                return t.bastion_restart()
            else:
                t.all_forbid_poste(ip, deb, fin, jdeb, jfin)
                return t.bastion_restart()
        # on leve des interdictions de postes
        elif self.form_result.has_key('list_suppr'):
            result = self.form_result['list_suppr']
            postes = format_form_result(result)
            if not postes.has_key('to_del'):
                return ''
            t.autorize_postes(postes['to_del'])
            return t.bastion_restart()
        elif self.form_result.has_key('all_list_suppr'):
            result = self.form_result['all_list_suppr']
            postes = format_form_result(result)
            if not postes.has_key('to_del'):
                return ''
            t.autorize_all_postes(postes['to_del'])
            return t.bastion_restart()
        # on redemarre le filtrage

    def _valid_banned_user(self):
        """ valide l'interdiction d'utilisateur """
        if not self.form_result.has_key('list_add') and not self.form_result.has_key('list_suppr'):
            log.msg('Il manque un truc !')
            return ''
        # on interdit des users
        if self.form_result.has_key('list_add'):
            result = self.form_result['list_add']
            user = format_form_result(result)
            username = user['name']
            t.add_user(username, 'banned_user')
            t.dansguardian_reload()
        # réautorisation d'utilisateur interdits
        elif self.form_result.has_key('list_suppr'):
            result = self.form_result['list_suppr']
            users = format_form_result(result)
            if not users.has_key('to_del'):
                return ''
            for user in users['to_del']:
                t.autorise_user(user)
                t.dansguardian_reload()

    def _valid_moderateur(self):
        """ valide l'ajout du moderateur """
        if not self.form_result.has_key('list_add') and not self.form_result.has_key('list_suppr'):
            log.msg('il manque un truc')
            return ''
        # on ajoute un modérateur
        if self.form_result.has_key('list_add'):
            result = self.form_result['list_add']
            user = format_form_result(result)
            username = user['name']
            t.add_user(username, 'moderateur')
            t.dansguardian_reload()
        # suppression de modérateurs
        elif self.form_result.has_key('list_suppr'):
            result = self.form_result['list_suppr']
            users = format_form_result(result)
            if not users.has_key('to_del'):
                return ''
            for user in users['to_del']:
                t.autorise_user(user)
                t.dansguardian_reload()

def _get_menus(server_nb, name, page):
    """renvoit la description du menu de la section filtrage """
    item1 = {'href':make_js_link(server_nb, name, current_page='poste'),
             'libelle':"Postes", 'icone':'/image/amon/banned_poste.png', 
             'title':page_msgs['poste']}
    item2 = {'href':make_js_link(server_nb, name, current_page='banned_user'),
             'libelle':"Utilisateurs", 'icone':'/image/amon/banned_user.png',
             'title':page_msgs['banned_user']}
    item3 = {'href':make_js_link(server_nb, name, current_page='moderateur'),
             'libelle':'Modérateurs', 'icone':'/image/amon/moderateur.png',
             'title':page_msgs['moderateur']}
    if page == 'poste':
        item1['href'] = ''
    elif page == 'banned_user':
        item2['href'] =''
    elif page == 'moderateur':
        item3['href'] = ''
    return [item1, item2, item3]


def get_days(fin=False):
    default = 'lundi'
    if fin:
        default = 'dimanche'
    days = ['lundi', 'mardi', 'mercredi', 'jeudi', 'vendredi', 'samedi', 'dimanche']
    #days.remove(default)
    ret_days = []
    for day in days:
        if day == default: ret_days.append({'name':day, 'libelle':day, 'default':'ok'})
        else:ret_days.append({'name':day, 'libelle':day})
    return ret_days

def get_hours(fin=False):
    default = {'libelle':'7:00', 'name':'7:00'}
    if fin:
        default = {'libelle':'19:00','name':'19:00', 'default':'ok'}
    hours = []
    for i in range(24):
        val1  = '%d:00'%i
        val2 = '%d:30'%i
        hour = {'libelle':val1, 'name':val1}
        if  i==7 and not fin:hour['default']='ok'
        elif i==19 and fin: hour['default'] = 'ok'
        halfandhour = {'libelle':val2, 'name':val2}
        hours.append(hour)
        hours.append(halfandhour)       
    #hours.remove(default)
    return hours

