# -*- coding: utf-8 -*-
###########################################################################
# Eole NG - 2007  
# Copyright Pole de Competence Eole  (Ministere Education - Academie Dijon)
# Licence CeCill  cf /root/LicenceEole.txt
# eole@ac-dijon.fr 
#  
# Action navigation_access
#  
# Gere les interdiction d'acces de poste ou d'url ainsi que l'autorisation
# de moderateur
#       
###########################################################################

""" Action de gestion des listes blanches sur amon """
from twisted.python import log
from ead2.backend.lib.action import * # import des actions et des types
from ead2.backend.actions.tools import get_request, make_form_link, make_js_link, make_key_form_link
from ead2.backend.actions.tools import format_form_result
import ead2.backend.actions.amon.amon_tools as t #fonctions de gestion de la liste blanche
from ead2.backend.config.config import SCRIPT_DIR
from ead2.backend.config.filenames import navigation_access, dg_reload
from ead2.lib.libead import uni
import commands
from os.path import isfile, join


page_msgs = { 'white_user':"Gestion des utilisateurs sur liste blanche",
             'grey_list':"Ajout de site pour le mode liste blanche"
             }
#nom des formulaires
form_name = {'white_user':['list_add', 'current_page'],
            'grey_list': ['list_add', 'current_page']}
used_templates = ['main', 'form', 'formselect']


def _get_menus(server_nb, name, page):
    """renvoit la description du menu de la section filtrage """
    item1 = {'href':make_js_link(server_nb, name, current_page='white_user'),
             'libelle':'Liste blanche', 'icone':'/image/amon/white_user.png',
             'title':page_msgs['white_user']}
    item2 = {'href':make_js_link(server_nb, name, current_page='grey_list'),
             'libelle':'Sites de la liste blanche', 'icone':'/image/amon/grey_sites.png',
             'title':page_msgs['grey_list']}
    if page == 'white_user':
        item1['href'] = ''
    else:
        item2['href'] = ''
    return [item1, item2]

class NavigationWhiteList(Action):
    """ interface de gestion des acces interneten mode liste blanche"""
    user_description = Dict(default={}, doc="description de l'éxécutant", keys=['ip', 'name'])
    name = 'navigation_whitelist'
    libelle = 'Mode liste blanche'
    description = 'Mode liste blanche utilisateur'
    category = "Outils/Filtrage"
    request = Dict(default={}, doc="arguments de la requete en cour cote frontend", 
                    keys=['server', 'action', 'current_page'])
    form_result = Dict(default={}, doc="retour formulaire de la requete en cour coté frontend",
                        keys = ['list_suppr', 'list_add', 'current_page'])
    def execute(self):
        """ renvoit les données pour la mise en forme des actions """
        params, self.server_nb = get_request(self.request)
        self.page = 'white_user'
        result = {'titre': self.description}
        if self.form_result != {}:
            if self.form_result.has_key('current_page'):
                self.page = self.form_result['current_page'][0]['value']
            self._valid_form()
            self.form_result = {}
        if params.has_key('current_page'):
            self.page = params['current_page'][0]
        result.update(self._get_form())
        result['sstitre'] =  page_msgs[self.page]
        result['menus'] = _get_menus(self.server_nb, self.name, self.page)
        result['page'] = {'name':'current_page', 'default_value':self.page}
        if form_name.has_key(self.page):
            result['validate'] = {'href':make_form_link(self.server_nb, self.name, True, form_name[self.page]),
                                'icone':'/image/amon/ok.gif', 'libelle':'Valider', 'title':'Valider les changements'}
            result['validation'] = make_key_form_link(self.server_nb, self.name, False, form_name[self.page])
        else:
            result['validate'] = ''
        d = {'template':'amon_navigation_whitelist', 'data':{'content':result, 'templates':used_templates}}
        return 0, uni(str(d))

    def _get_form(self):
        if self.page == 'grey_list':
            return self._get_grey_list_form()
        else:
            return self._get_white_user_form()

    def _get_white_user_form(self):
        whitelist_user = {'name':'name', 'onblur':"testVoid('name', 'error_name')",
                'libelle':"Login de l'utilisateur à ajouter à la liste blanche"}
        whitelist_users = [{'name':user, 'libelle':user}for user in t.get_users()['white_users']]
        whitelist_users_form = {'entete':"Utilisateurs de la liste blanche"}
        if whitelist_users != []:
            whitelist_users_form.update({
                    'validate':{'href':make_form_link(self.server_nb, self.name, True, ['current_page','list_suppr']),
                                'icone':'/image/amon/supprimer_big.gif','libelle':'Retirer de la liste blanche', 
                                'title':'Retirer les utilisateurs selectionnés de la liste blanche'},
                    'select':{'name':'to_del', 'options':[{'name':'utilisateurs de la liste', 'options':whitelist_users}],
                            'libelle':"Veuillez sélectionner le ou les utilisateurs à retirer de la liste blanche",
                            'multi':'ok'}
                            })
        else:
            whitelist_users_form.update({"msg": "Aucun utilisateur n'a été placé sur liste blanche"})
        return dict(whitelist_user=whitelist_user, listing=whitelist_users_form, focus='name')

    def _get_grey_list_form(self):
        whitelist_site = {'name':'name','onblur':"testVoid('name', 'error_name')",
                'libelle':"Site à ajouter à la liste blanche"}
        grey_sites = [{'name':site, 'libelle':site}for site in t.get_grey_list()]
        grey_site_form = {'entete':"Retirer des sites de la liste blanche"}
        if grey_sites != []:
            grey_site_form.update({
                'validate':{'href':make_form_link(self.server_nb, self.name, True, ['current_page','list_suppr']),
                            'icone':'/image/amon/supprimer_big.gif','libelle':'Retirer de la liste blanche',
                            'title':'Retirer les sites selectionnés de la liste blanche'},
                'select':{'name':'to_del', 'options':[{'name':'site en liste blanche', 'options':grey_sites}],
                            'libelle':"Veuillez sélectionner le ou les sites à retirer de la liste blanche",
                            'multi':'ok'}
                        })
        else:
            grey_site_form.update({"msg": "Aucun site n'est placé sur la liste blanche"})
        return dict(whitelist_site=whitelist_site, listing=grey_site_form, focus='name')

    def _valid_form(self):
        """ valide les retours de formulaire"""
        if self.page == 'white_user':
            return self._valid_white_user()
        else:
            return self._valid_grey_list()

    def _valid_white_user(self):
        """ valide l'ajout d'utilisateur à la liste blanche """
        if not self.form_result.has_key('list_add') and not self.form_result.has_key('list_suppr'):
            log.msg('il manque un truc')
            return ''
        # on ajoute un utilisateur à la liste blanche
        if self.form_result.has_key('list_add'):
            result = self.form_result['list_add']
            user = format_form_result(result)
            username = user['name']
            t.add_user(username, 'white_user')
       # suppression d'utilisateur de la liste blanche
        elif self.form_result.has_key('list_suppr'):
            result = self.form_result['list_suppr']
            users = format_form_result(result)
            if not users.has_key('to_del'):
                return ''
            for user in users['to_del']:
                t.autorise_user(user)
        t.dansguardian_reload()

    def _valid_grey_list(self):
        """ valide l'ajout d'un site accessible pour les utilisateurs sur liste blanche"""
        if not self.form_result.has_key('list_add') and not self.form_result.has_key('list_suppr'):
            log.msg('il manque un truc')
            return ''
        # on ajoute un site à la liste blanche
        if self.form_result.has_key('list_add'):
            result = self.form_result['list_add']
            site = format_form_result(result)
            sitename = site['name']
            t.add_site(sitename)
        # suppression d'un site de la liste blanche
        elif self.form_result.has_key('list_suppr'):
            result = self.form_result['list_suppr']
            sites = format_form_result(result)
            if not sites.has_key('to_del'):
                return ''
            t.del_site(sites['to_del'])
        t.dansguardian_reload()
