# -*- coding: utf-8 -*-
###########################################################################
# Eole NG - 2007  
# Copyright Pole de Competence Eole  (Ministere Education - Academie Dijon)
# Licence CeCill  cf /root/LicenceEole.txt
# eole@ac-dijon.fr 
#  
# Action securite
#  
# Edite les regles de pare-feu, interdiction de poste, destinations,
# peer to peer
#       
###########################################################################

""" Actions de gestion des paramètres de sécurité """
from twisted.python import log
from ead2.backend.lib.action import *
from ead2.backend.actions.tools import get_request, make_long_form_link, make_js_link, format_form_result, make_key_form_link
from ead2.backend.config.config import SCRIPT_DIR
import os, commands
from os.path import isfile, join
from ead2.lib.libead import uni
from ead2.backend.config.filenames import securite
from ead2.backend.actions.amon.amon_tools import bastion_restart

page_msgs = {'regles':"Gestion des règles du Pare-Feu",
             'interdits':"Interdire des destinations",
             "p2p":"Interdiction des échanges de fichiers (peer to peer)",
             'horaire':"Horaires d'ouverture du pare-feu"
             }
used_templates = ['main', 'form', 'formselect']

form_name = {'regles':['mod_regles'], 'interdits':['list_add', 'current_page'],
            'horaire':['list_add', 'current_page']}

# nom des fichies a editer
regles_filename = securite['regles']
interdits_filename = securite['interdits']
p2p_filename = securite['p2p']
horaire_filename = securite['horaire']

def get_default_horaires():
    """ renvoit les horaires par defaut """
    return ["1 7 19","2 7 19", "3 7 19", "4 7 19", "5 7 19", "6 7 19","0 - -"]

def get_horaires():
    """ renvoit les horaires du pare-feu 
        sous la forme ["numjour heuredeb heurefin", ...]
    """
    result = []
    if isfile(horaire_filename):
        horaires = file(horaire_filename, 'r').read().splitlines()
        if len(horaires) <7:
            horaires = get_default_horaires()
    else:
        horaires = get_default_horaires()
    return parse_horaires(horaires)

def parse_horaires(horaires):
    """ parse les horaires du pare-feu """
    result = []
    for i in horaires:
        horaire = i.split()
        if len(horaire) == 3:
            result.append((horaire[0], horaire[1], horaire[2],))   
        else:
            # si il y a une erreur dans le fichier
            return parse_horaires(get_default_horaires())
    return result

def write_horaire(jours):
    """ ecrit les horaires du pare-feu """
    a = file(horaire_filename, 'w')
    for jour in jours:
        a.write('%s %s %s \n'%(jour[0], jour[1], jour[2]))
    a.close()
    return True

def get_hours(act_hour='-'):
    """ recupere les heures """
    hours = [{'libelle':"--------", "name":"-"}]
    for i in range(24):
        libelle = '%dh 00'%i
        value = '%d'%i
        hours.append({'libelle':libelle, "name":value})
    for i in hours:
        if i['name'] == act_hour:
            i['default'] = 'ok'
    return hours

class Securite(Action):
    """ Gere les paramètres de sécurité d'un Amon
        :request: -server: numéro du serveur en cour de traitement
                  -action: nom de l'action
                  -current_page: section en cour de traitement (regles,reseau,interdits,p2p)
                  -p2p:interdire/autoriser
        :form_result: -mod_regles: modification des regles de pare feu
                      -current_page:section en cour de traitement (reseau,interdits)
                      -list_add:ajoute une valeur pour les interdiction d'accès réseau ou interdiction poste client
                      -list_suppr:enleve une valeur pour les interdiction d'accès réseau ou interdiction poste client
    """
    user_description = Dict(default={}, doc="description de l'éxécutant", keys=['ip', 'name'])
    name = "securite"
    libelle = 'Règles de sécurité'
    description = 'Sécurité personnalisée des accès internet'
    category = "Outils"
    request = Dict(default={}, doc="arguments de la requete en cour cote frontend",
                    keys=['server', 'action', 'current_page', 'p2p'])
    form_result = Dict(default={}, doc="retour formulaire de la requete en cour coté frontend",
                        keys = ['mod_regles', 'current_page', 'list_add', 'list_suppr', 'p2p'])

    def execute(self):
        """ renvoit les données pour la mise en forme des actions """
        params, self.server_nb = get_request(self.request)
        self.page = 'regles'
        result =  {'titre': self.libelle}
        if params.has_key('current_page'):
            self.page = params['current_page'][0]
        if self.form_result != {}:
             if self.form_result.has_key('current_page'):
                 self.page = self.form_result['current_page'][0]['value']
             self._valid_form()
             self.form_result = {}
        if params.has_key('p2p'):
            self.form_result['p2p'] = params['p2p']
            self._valid_form()
            self.form_result = {}
        result['sstitre'] = page_msgs[self.page]
        result['menus'] = self._get_menu()
        result.update(self._get_form())
        result['page'] = {'name':'current_page', 'default_value':self.page}
        if form_name.has_key(self.page):
            result['validate'] = {'href':make_long_form_link(self.server_nb, self.name, True, form_name[self.page]),
                    'icone':'/image/amon/ok.gif', 'libelle':'Valider', 'title':'Valider les changements'}
            result['validation'] = make_key_form_link(self.server_nb, self.name, False, form_name[self.page])
        else:
            result['validate'] = ''
        result['retour'] = get_return_btn(self.server_nb)
        d = {'template':'amon_securite', 'data':{'content':result, 'templates':used_templates}}
        return 0, uni(str(d))

    def _get_form(self):
        """ renvoit le formulaire lié à la section en cour de traitement """
        if self.page == 'regles':
            return self._get_regles()
        elif self.page == 'interdits':
            return self._get_interdits() 
        elif self.page == 'p2p':
            return self._get_p2p()
        else:
            return self._get_horaire()
    
    def _get_regles(self):
        """ renvoit la decsription du formulaire d'édition des règles de pare-feu """
        lines = []
        if isfile(regles_filename):
            a = file(regles_filename, 'r')
            lines = a.readlines()
            a.close()
        rules = []
        for line in lines:
            desc , val, occ = line.split(',')
            actif = {'name':desc, 'value':'actif'}
            inactif = {'name':desc, 'value':'inactif'}
            if val == '1':
                inactif['checked'] = 'true'
            else:
                actif['checked'] = 'true'
            rules.append({'libelle':desc, 'actif':actif, 'inactif': inactif})
        return dict(rules=rules)

    def _get_interdits(self):
        """ renvoit la description du formulaire pour les interdictions de destination """
        forbid_ip = {'name':'ip', 'onblur':"testVoid('ip', 'error_ip')",
                'libelle':"Ip  ou nom de la machine de destination à interdire"}
        ips = []
        if isfile(interdits_filename):
            for ip in file(interdits_filename).read().splitlines():
                tag = {'name':ip, 'libelle':ip}
                ips.append(tag)
        forbidden_title = "Réautoriser une destination"
        if ips != []:
            forbidden_ips = {'options':[{'name':'destinations interdites', 'options':ips}],
                    'libelle':"Veuillez sélectionner la ou les destinations à réautoriser", 'name':'suppr', 'multi':'ok'}
            forbidden_validate= {'href':make_long_form_link(self.server_nb, self.name, True, ['current_page',
                            'list_suppr']), 'icone':'/image/amon/supprimer_big.gif',
                            'libelle':'Réautoriser', 'title':'Réautoriser les adresses ou postes selectionnés'}
            return dict(forbid_ip=forbid_ip, forbidden_ips=forbidden_ips, forbidden_validate=forbidden_validate,
                        focus='ip', forbidden_title=forbidden_title)
        else:
            forbidden_ips = {"msg": "Aucune interdiction d'accès n'est appliquée pour le moment"}
            return dict(forbid_ip=forbid_ip, forbidden_ips=forbidden_ips, focus='ip', forbidden_title=forbidden_title)

    def _get_p2p(self):
        """ renvoit la description du formulaire de gestion p2p """
        if isfile(p2p_filename):
            msg = "Les échanges de fichiers (peer to peer) sont","interdits"
            color = "#b22"
            validate = {"href":make_js_link(self.server_nb, self.name, current_page="p2p", p2p="autoriser"),
                       'libelle':"Autoriser", 'icone':"/image/ok.gif", 'title':"Autoriser le peer to peer"}
        else:
            msg = "Les échanges de fichiers (peer to peer) sont","autorisés"
            color = "#2b2"
            validate = {"href":make_js_link(self.server_nb, self.name, current_page="p2p", p2p="interdire"),
                        'libelle':"Interdire", 'icone':"/image/amon/refused.png", 'title':"Interdire le peer to peer"}
        return dict(ptpmsg=msg, forbid_p2p=validate, entete="Echange de fichier (peer to peer)", color=color)

    def _get_horaire(self):
        """ renvoit la description du formulaire de gestion des horaires de pare-feu """
        semaine = ['dimanche','lundi', 'mardi', 'mercredi', 'jeudi', 'vendredi', 'samedi']
        horaires = get_horaires()
        days = []
        for horaire in horaires:
            num_jour, houvert, hferme = horaire
            indice = int(num_jour)
            ouvert_select = {'name':'ouvert[%d]'%(indice), 'options':[{'name':'', 'options':get_hours(houvert)}]}
            ferme_select = {'name':'ferme[%d]'%(indice), 'options':[{'name':'', 'options':get_hours(hferme)}]}
            days.append({'libelle':semaine[indice], 'ouvert':ouvert_select, 'ferme':ferme_select})
        return dict(horaire=days)
#            formulaire.append({'libelle':semaine[indice], 'ouvert':get_hours(houvert), 'ferme':get_hours(hferme)})
#        form = {'name':'list_add', 'formulaire':formulaire, 'entete':"Horaires personnalisés"}
#        return form

    def _valid_form(self):
        """ valide les retours de formulaire"""
        if self.page == 'regles':
            return self._valid_regles()
        elif self.page == 'interdits':
            return self._valid_reseau(interdits_filename)
        elif self.page == 'p2p':
            return self._valid_p2p()
        else:
            return self._valid_horaire()

    def _valid_regles(self):
        """ valide les modifications de règles de pare-feu """
        if self.form_result.has_key('mod_regles'):
            regles = self.form_result['mod_regles']
        else:
            return
        options = []
        for regle in regles:
            if regle['checked'] == 'true':
                nom = regle['name']
                valeur = regle['value']
                if valeur == 'actif':
                    val = '3'
                else:
                    val = '1'
                opt = {'nom':nom, 'valeur':val}
                options.append(opt)
        a = file(regles_filename, 'w')
        for opt in options:
            a.write("%s,%s,1\r\n"%(opt['nom'], opt['valeur']))              
        bastion_restart()

    def _valid_reseau(self, filename):
        """ valide l'interdiction de poste ou sous réseau """
        if not self.form_result.has_key('list_add') and not self.form_result.has_key('list_suppr'):
            return ''
        val = None
        # recuperation des postes deja interdits
        if isfile(filename):
            lines = file(filename).read().splitlines()
        else:
            lines = []
        if self.form_result.has_key('list_add'):
            val = format_form_result(self.form_result['list_add'])['ip']
            if val not in lines:
                lines.append(val)
        elif self.form_result.has_key('list_suppr'):
            todel = format_form_result(self.form_result['list_suppr'])['suppr']
            for ip in todel:
                if ip in lines:
                    lines.remove(ip)
        a = file(filename, 'w')
        for line in lines:
            a.write('%s\n'%line)
        a.close()
        bastion_restart()


    def _valid_p2p(self):
        """ valide l'activation/désactivation du peer to peer {'p2p':autoriser/interdire}"""
        if not self.form_result.has_key('p2p'):
            return ''
        p2p_status = self.form_result['p2p'][0]
        if p2p_status == 'autoriser':
            if isfile(p2p_filename):
                os.remove(p2p_filename)
        else:
            a = file(p2p_filename, 'w')
            a.write('   ')
            a.close()
        bastion_restart()

    def _valid_horaire(self):
        """ valide la configuration des horaires du pare-feu """
        if not self.form_result.has_key('list_add'):
            return ''
        config = format_form_result(self.form_result['list_add'])
        jours = []
        for i in range(7)[1:]:
            keyopen = 'ouvert[%d]'%i
            keyclosed = 'ferme[%d]'%i
            indice = i
            jours.append(('%d'%indice, config[keyopen], config[keyclosed]))
        keyopen, keyclosed = 'ouvert[0]', 'ferme[0]'
        jours.append(('0', config[keyopen], config[keyclosed]))
        write_horaire(jours)
        bastion_restart()

    def _get_menu(self):
        """ renvoit la description du menu """
        item1 = {'href':make_js_link(self.server_nb, self.name, current_page='regles'),
                 'libelle':"Regles de Pare-Feu", 'icone':'/image/amon/regles.png',
                 'title':page_msgs['regles']}
        item3 = {'href':make_js_link(self.server_nb, self.name, current_page='interdits'),
                 'libelle':"Destinations interdites", 'icone':'/image/amon/noaccess.png',
                  'title':page_msgs['interdits']}
        item4 = {'href':make_js_link(self.server_nb, self.name, current_page='p2p'),
                 'libelle':"Peer to Peer", 'icone':'/image/amon/p2p.png',
                  'title':page_msgs['p2p']}
        item5 = {'href':make_js_link(self.server_nb, self.name, current_page='horaire'),
                 'libelle':"Horaires du pare-feu", 'icone':'/image/amon/hour.png',
                  'title':page_msgs['horaire']}
        if self.page == 'regles':
            item1['href'] = ''
        elif self.page == 'interdits':
            item3['href'] = ''
        elif self.page == 'p2p':
            item4['href'] = ''
        else:
            item5['href'] = ''
        return [item1,  item3, item4, item5]

def get_return_btn(server_nb):
    """ renvoit la description du bouton retour """
    href = make_js_link(server_nb, 'main_status')
    icone = '/image/back.png'
    libelle = "Retour vers la page d'accueil"
    title = "Revenir vers l'accueil du serveur"
    return {'href':href, 'icone':icone, 'libelle':libelle, 'title':title}

