# -*- coding: UTF-8 -*-
###########################################################################
# Eole NG - 2007  
# Copyright Pole de Competence Eole  (Ministere Education - Academie Dijon)
# Licence CeCill  cf /root/LicenceEole.txt
# eole@ac-dijon.fr 
#  
# Action Bacula et BaculaConfig
#
# Programme et configure des sauvegardes bacula
#       
###########################################################################

""" Actions permettant de gérer les sauvegardes bacula """
from twisted.python import log
from ead2.backend.config.config import BACULA_DIR, BACULA_TEMP_DIR, SCRIPT_DIR
from ead2.backend.config.filenames import bacula as baculafiles
from ead2.backend.lib.action import * # import des actions et des types
from ead2.backend.actions import tools
# ajout du chemin pour bacula dans le pythonpath
import sys
sys.path.append(BACULA_DIR)
import baculaeole as bacula

from nevow.url import URL
from ead2.lib.libead import uni
from ead2.backend.actions.horus.connexion_tools import get_active_workstations
import re
from horus.backend import get_active_users, set_active_users
import commands
from os.path import isfile, join

# def des noms de fichiers de configuration de sauvegarde:
distant_filename = baculafiles['distant']                       # conf de sauvegarde pour les supports distants
usb_filename = baculafiles['usb']                               # pour les supports usb
rapport_filename = baculafiles['rapport']                       # log des sauvegardes effectuees
filenames = {'usb':usb_filename, 'distant':distant_filename}
used_templates = ['main', 'formselect', 'form']

class Bacula(Action):
    """
    interface de planification de sauvegarde
    trois formulaires sont proposes pour les sauvegardes (hebdomadaire,quotidienne,  immediate)
    """
    user_description = Dict(default={}, doc="description de l'éxécutant", keys=['ip', 'name'])
    name = 'bacula'
    libelle = 'Programmation'
    category = 'Sauvegarde'
    description = 'gestionnaire de sauvegarde bacula'
    request = Dict(default={}, doc="arguments de la requete en cour cote frontend",
                            keys=['server', 'action', 'job_suppr', 'page'])
    form_result = Dict(default={}, doc="retour de formulaire en JSON", keys=['sauvegarde', 'page'])

    def execute(self):
        """ renvoit les donnees pour la mise en forme de l'action et gere les retours 
            :titre: titre de la page
            :formulaires: description des 3 formulaires de programmation de 
                        sauvegarde
            :sauvegarde: liste des sauvegardes realisees
            :planifie: liste des sauvegardes planifiees avec la description du 
                       bouton annuler
            :help: bouton d'aide
        """
       # on recup le numero du serveur en cour de traitement et la requete
        params, self.server_nb = tools.get_request(self.request)
        self.page = 'weekly'
        if params.has_key('page'):
            self.page = params['page'][0]
        result = {'titre':self.description}
        code, message = [None, '']
        if self.form_result !={} :
            if self.form_result.has_key('page'):
                self.page = tools.format_form_result(self.form_result['page'])['page']
                self.form_result.pop('page')
            try: 
                code, message = self.save(self.form_result)
                self.form_result = {}
            except:
                code = 1
        if code is not None:
            if code:
                result['retour_msg'] = "Erreur lors de la programmation de la sauvegarde. "
            else:
                result['retour_msg'] = "La sauvegarde a bien été programmée. "
        else:
            result['retour_msg'] = ''
        if message:
            result['retour_msg'] += message
        sup = get_actual_support()
        result['redirect'] = ''
        if sup in filenames.keys():
            if not isfile(filenames[sup]):
                result['retour_msg'] += "Pensez à configurer Bacula avant toutes sauvegardes"
                result['redirect'] = "call_action('%s', 'bacula_config')"%self.server_nb
        if params.has_key('job_suppr'): # annulation d'une sauvegarde programmee
            self.cancel_save(params['job_suppr'][0])
        sstitre_dict = {'weekly':"Programmation de sauvegarde hebdomadaire", 
                        'daily':'Programmation de sauvegarde quotidienne', 
                        'direct':"Programmation de sauvegarde immédiate"}
        result['sstitre'] = sstitre_dict[self.page]
        result['current_page'] = {'default_value':self.page, 'name':'page'}
        result['menus'] = self._get_menu()
        result.update(self._get_form())
        result['valid'] = self._get_valid_btn()
        result['jobs'] = self._get_actual_saves()
        d = {'template':'bacula', 'data':{'content':result, 'templates':used_templates}}
        return 0, uni(str(d))

    def _get_form(self):
        """ renvoit le formulaire adequat """
        if self.page == 'weekly':
            return self._get_form1()
        elif self.page == 'daily':
            return self._get_form2()
        else:
            return self._get_form3()

    def _get_form1(self):
        """premier formulaire pour la programmation de sauvegarde (sauvegarde hebdomadaire)"""
        type_opts = [{'name':'Complete', 'libelle':'Complete', 'default':'ok'},
                        {'name':'Restreinte', 'libelle':'Restreinte'}]
        typesav = {'name':'typesav', 'options':[{'name':'type de sauvegarde', 'options':type_opts}],
                'libelle':"Programmer une sauvegarde", 'inline':'ok'}
        niveau_opts = [{'name':'Full', 'libelle':'Totale'}, {'name':'Incremental', 'libelle':'Incrémentale'}]
        level = {'name':'niveau','options':[{'name':'niveau de sauvegarde', 'options':niveau_opts}], 
                'libelle':'de type', 'inline':'ok'}
        days_opts = self._get_days(1)
        begin = {'name':'jourdebut', 'options':[{'name':'jour de la semaine', 'options':days_opts}], 'libelle':'le',
                'inline':'ok'}
        hour_opts = self._get_hours(0)
        hour = {'name':'heure', 'options':[{'name':'heure de la journée', 'options':hour_opts}],
                'libelle':'à','inline':'ok'}
        return dict(typesav=typesav, level=level, begin=begin, hour=hour)

    def _get_form2(self):
        """ deuxieme formulaire pour la programmation de sauvegarde (sauvegarde quotidienne)"""
        type_opts = [{'name':'Complete', 'libelle':'Complete', 'default':'ok'},
                        {'name':'Restreinte', 'libelle':'Restreinte'}]
        typesav = {'name':'typesav', 'options':[{'name':'type de sauvegarde', 'options':type_opts}],
                'libelle':"Programmer une sauvegarde",'inline':'ok'}
        niveau_opts = [{'name':'Full', 'libelle':'Totale'}, {'name':'Incremental', 'libelle':'Incrémentale'}]
        level = {'name':'niveau','options':[{'name':'niveau de sauvegarde', 'options':niveau_opts}], 
                'libelle':'de type','inline':'ok'}
        begin_opts = self._get_days(1)
        begin  = {'name':'jourdebut', 'options':[{'name':'jour de la semaine', 'options':begin_opts}], 
                'libelle':'du','inline':'ok'}
        end_opts = self._get_days(5)
        end = {'name':'jourfin', 'options':[{'name':'jour de la semaine', 'options':end_opts}], 
                'libelle':'au','inline':'ok'}
        hour_opts = self._get_hours(0)
        hour = {'name':'heure', 'options':[{'name':'heure de la journée', 'options':hour_opts}],
                'libelle':'à','inline':'ok'}
        return dict(typesav=typesav, level=level, begin=begin, end=end, hour=hour)

    def _get_form3(self):
        """ troisieme formulaire de sauvegarde (sauvegarde immediate)"""
        type_opts = [{'name':'Complete', 'libelle':'Complete', 'default':'ok'},
                {'name':'Restreinte', 'libelle':'Restreinte'}]
        typesav = {'name':'typesav', 'options':[{'name':'type de sauvegarde', 'options':type_opts}],
                'libelle':"Programmer une sauvegarde", 'inline':'ok'}
        niveau_opts = [{'name':'Full', 'libelle':'Totale'}, {'name':'Incremental', 'libelle':'Incrémentale'}]
        level = {'name':'niveau','options':[{'name':'niveau de sauvegarde', 'options':niveau_opts}],
                'libelle':'de type','inline':'ok'} 
        return dict(typesav=typesav, level=level)

    def _get_menu(self):
        """ renvoit la description du menu """
        weekly = {'href':tools.make_js_link(self.server_nb, self.name, page='weekly'),
                'libelle':'Hebdomadaire', 'icone':'/image/mail.gif', 'title':'Sauvegarde hebdomadaire'}
        daily = {'href':tools.make_js_link(self.server_nb, self.name, page='daily'),
                'libelle':'Quotidienne', 'icone':'/image/mail.gif', 'title':'Sauvegarde quotidienne'}
        direct = {'href':tools.make_js_link(self.server_nb, self.name, page='direct'),
                'libelle':'Immédiate', 'icone':'/image/mail.gif', 'title':'Sauvegarde immédiate'}
        if self.page == 'weekly':
            weekly['href'] = ''
        elif self.page == 'daily':
            daily['href'] = ''
        else:
            direct['href'] = ''
        return weekly, daily, direct

    def _get_valid_btn(self):
        """ renvoit la description du bouton valider pour le formulaire """
        validate = {}
        ref= tools.make_form_link(self.server_nb, self.name, False, ['sauvegarde', 'page'])
        if self.page == 'direct':
            message = "Etes-vous sûr de vouloir lancer cette sauvegarde ?"
        else:
            message = "Etes-vous sûr de vouloir programmer cette sauvegarde ?"
        validate['href']= tools.make_confirm_link(message, ref)
        validate['icone'] = "/image/ok.gif"
        validate['libelle'] = "OK"
        validate['title'] = "Programmer la sauvegarde"
        return validate

    def _get_actual_saves(self):
        """ renvoit la liste des sauvegardes planifiees """
        saves = bacula.chargedico()
        if saves == {}:
            return [{'sauvegarde':"Aucune sauvegarde n'est programmée"}]
        montemplate1 = """Sauvegarde %(niveau)s, %(typesav)s du %(jourdebut)s au %(jourfin)s à %(heure)s heure"""
        montemplate2 = """Sauvegarde %(niveau)s, %(typesav)s le %(jourdebut)s à %(heure)s heure"""
        dicotraduc = { 'mon':'lundi', 'tue':'mardi', 'wed':'mercredi', 'thu':'jeudi', 'fri':'vendredi', 'sat':'samedi', 'sun':'dimanche', 'Level=Incremental':'Incrémentale', 'Level=Full':'Totale' }
        jobs = []
        for job in saves.keys():
            href = tools.make_js_link(self.server_nb, self.name, False, job_suppr=job)
            saves[job]['jourdebut']=dicotraduc[saves[job]['jourdebut']]
            saves[job]['niveau']=dicotraduc[saves[job]['niveau']]
            if saves[job]['jourfin']:
                saves[job]['jourfin']=dicotraduc[saves[job]['jourfin']]
                sauvegarde = str(job)+" : "+montemplate1 % saves[job]
            else:
                sauvegarde = str(job)+" : "+montemplate2 % saves[job]
            btn = {'href':href, 'title':"Annulation de la sauvegarde", "libelle":"Annuler", 
                    "icone":'/image/supprimer_big.gif'}
            jobs.append({'btn':btn, 'sauvegarde': sauvegarde})
        return jobs

    def _get_sauvegarde(self):
        """ renvoit la liste des sauvegardes realisees"""
        if isfile(rapport_filename):
            a = file(rapport_filename, 'r')
            lines = a.readlines()
            if len(lines) == 0 :
                a.close() 
                return ''
            a.close() 
            return lines
        return ''

    def _get_days(self, default):
        """ renvoit les jours de la semaine pour les formulaires avec default: valeur par defaut"""
        days = [{'name':'mon', 'libelle':'lundi'},
                {'name':'thu', 'libelle':'mardi'},
                {'name':'wed', 'libelle':'mercredi'},
                {'name':'tue', 'libelle':'jeudi'},
                {'name':'fri', 'libelle':'vendredi'},
                {'name':'sat', 'libelle':'samedi'},
                {'name':'sun', 'libelle':'dimanche'}
                ]
        days[default]['default'] = 'ok'
        return days

    def _get_hours(self, default):
        """ renvoit les heures (depuis 0->23) pour les formulaires """
        hours = []
        for i in range(24):
            hour = {'name':'%s'%i, 'libelle':'%s heure'%i}
            hours.append(hour)
        hours[default]['default'] = 'ok'
        return hours

    def save(self, dico):
        """ gère la distribution des sauvegardes """
        if self.page == 'weekly':return self.sauvegarde_hebdo(dico['sauvegarde'])
        elif self.page == 'daily':return self.sauvegarde_quotidienne(dico['sauvegarde'])
        elif self.page == 'direct':return self.sauvegarde_immediate(dico['sauvegarde'])
        else:return 1, ''
    
    def sauvegarde_hebdo(self, options):
        """ programme une sauvegarde hebdomadaire """
        tools.record_action("Programmation d'une sauvegarde hebdomadaire", self.user_description)
        options = tools.format_form_result(options)
        try:
            typesav ,niveau, jourdebut, heure = [options['typesav'],options['niveau'], options['jourdebut'],options['heure']]
        except KeyError:
            return 1, ""
        heure = "%s:00"%heure
        if '' in [typesav ,niveau, jourdebut, heure]:
            return 1, ""
        else:
            code, retour = commands.getstatusoutput('%sbaculaeole.py -t %s -n %s -d %s -h %s'%(BACULA_DIR, typesav ,niveau, jourdebut, heure))
            return code, ''
        
    def sauvegarde_quotidienne(self, options):
        """ programme une sauvegarde quotidienne """
        tools.record_action("Lancement d'une sauvegarde quotidienne", self.user_description)
        typesav ,niveau, jourdebut, jourfin, heure = ['', '', '', '', '']
        options = tools.format_form_result(options)
        try:
            heure = "%s:00"%options['heure']
            values = [options['typesav'], options['niveau'], options['jourdebut'], options['jourfin'], heure]
        except KeyError:
            values = ['']
        if '' in values:
            return 1, ""
        else:
            code, retour = commands.getstatusoutput('%sbaculaeole.py -t %s -n %s -d %s -f %s -h %s'%(BACULA_DIR, values[0],values[1], values[2], values[3], values[4]))
            return code, ''
        
    def sauvegarde_immediate(self, options):
        """ Lance une sauvegarde immédiate """
        tools.record_action("Lancement d'une sauvegarde immédiate", self.user_description)
        for opt in options:
            name = opt['name']
            value = opt['value']
            if name == 'typesav':
                if value == 'restreinte':
                    typesav = '-R'
                else:
                    typesav = ''
            if name == 'niveau':
                if value == 'incrementale':
                    niveau = '-I'
                else:
                    niveau = ''
        commande = ''
        if typesav: commande = typesav
        if niveau: commande += ' %s'%niveau
        code, retour = commands.getstatusoutput('%s/bacula-sauv.sh %s'%(SCRIPT_DIR, commande))
        if code:
            return code, retour
        return code, "La sauvegarde est lancée."

    def cancel_save(self, num):
        """ Annule une sauvegarde programmee """
        code, retour  = commands.getstatusoutput('%sbaculaeole.py -s %s'%(BACULA_DIR, num))

class BaculaConfig(Action):
    """
    interface de configuration de l'outil de sauvegarde
    """
    user_description = Dict(default={}, doc="description de l'éxécutant", keys=['ip', 'name'])
    name = 'bacula_config'
    libelle = 'Configuration'
    category = 'Sauvegarde'
    description = "configuration de l'outil de sauvegarde bacula"
    request = Dict(default={}, doc="arguments de la requete en cour cote frontend", 
                         keys=['server', 'action', 'save_type'])
    form_result = Dict(default={}, doc="retour de formulaire", keys=['config1', 'config2', 'config3'])

    def execute(self):
        """ renvoit les donnees pour la mise en forme de l'action et gere les retours 
            :titre: titre de la page
        """
       # on recup le numero du serveur en cour de traitement et la requete
        params, self.server_nb = tools.get_request(self.request)
        code, message = [None, '']
        result = {'titre':self.description}
        if params.has_key('save_type'):
            self.save_type = params['save_type'][0]
        else:
            self.save_type = get_actual_support()
        # validation des modifications:
        if self.form_result !={} :
            try:
                code, message = self.modify_conf(self.form_result)
                self.form_result = {}
            except:
                code = 1
        if code is not None:
            if code:
                result['retour_msg'] = "Erreur lors de la configuration du système de sauvegarde. "
            else:
                result['retour_msg'] = "Le système de sauvegarde a bien été configuré. "
        else:
            result['retour_msg'] = ""
        if message:
            result['retour_msg'] += message
        if self.save_type == '':
            result['formulaires'] = [ {'titre': 'Support de sauvegarde', 'formulaire':self._get_main_form(),                                       'name':'config1'}]
        else:
            result['formulaires'] = [ {'titre': 'Support de sauvegarde', 'formulaire':self._get_main_form(),
                'name':'config1'},
                {'titre':'Paramètres de sauvegarde pour : %s'%self.save_type, 'formulaire':self._get_form(),
                    'name':'config2'},
                {'titre':"Paramètres d'envoi des logs (Facultatif)", 'formulaire':self._get_log_form(),
                    'name':'config3', 'validate':self._get_validate_btn()}]
            result['help'] = {'href':tools.make_help_link(self.server_nb, 'help', help_page='bacula_conf')}
        d = {'template':'bacula_config', 'data':{'content':result, 'templates':used_templates}}
        return 0, uni(str(d))


    def _get_main_form(self):
        """ renvoit le formulaire pour le choix du type de sauvegarde a configurer """
        possibilities = {'distant':'Distant', 'usb':'Disque usb local','bande':'Bande: Configuration manuelle'}
        values = []
        for name, libelle in possibilities.items():
            # type de support configurer
            if name == self.save_type:
                values.append({'name':name, 'libelle':libelle, 'default':'ok'})
            else:
                values.append({'name':name, 'libelle':libelle})
        tag = {'_type':'choice', 'name':'support', 'options':[{'name':'', 'options':values}],
                'libelle':'Support de sauvegarde', 
                'onchange':tools.make_js_link(self.server_nb, self.name, code=True, save_type='this.options[this.selectedIndex].value'), 
                'inline':'ok'}
        return [tag]

    def _get_form(self):
        """ renvoit le formulaire pour configurer les sauvegardes du type self.save_type """
        # configuration usb
        if self.save_type == 'usb':
            tag = {'_type':'text_input', 'name':'usb_periph', 'libelle':"Chemin d'accès" , 
                    'onblur':"testVoid('usb_periph', 'error_usb_periph');", 'inline':'ok'}
            if len(self._get_usb_conf()) > 0:
                tag['default_value'] = self._get_usb_conf()[0]
            return [tag]
        # configuration distante
        elif self.save_type == 'distant':
            tag1 = {'_type':'text_input', 'name':'machine', 'libelle':"Nom machine distante",
                    'onblur':"testVoid('machine', 'error_machine')", 'inline':'ok'}
            tag2 = {'_type':'text_input', 'name':'ipmachine', 'libelle':"IP machine distante",
                    'onblur':"testIp('ipmachine', 'error_ipmachine')", 'inline':'ok'}
            tag3 = {'_type':'text_input', 'name':'partage', 'libelle':"Partage",
                    'onblur':"testVoid('partage', 'error_partage')", 'inline':'ok'}
            tag4 = {'_type':'text_input', 'name':'login', 'libelle':"Login (facultatif)", 'inline':'ok'}
            tag5 = {'_type':'password', 'name':'password', 'libelle':"Mot de passe (facultatif)", 'inline':'ok'}
            a = self._get_distant_conf()
            if len(a)>0:
                tag1['default_value'] = a[0]
                tag2['default_value'] = a[1]
                tag3['default_value'] = a[2]
                if len(a) == 5:
                    tag4['default_value'] = a[3]
                    tag5['default_value'] = a[4]
            return [tag1, tag2, tag3, tag4, tag5]
        else:
            return []

    def _get_log_form(self):
        """ renvoit le formulairee pour la configuration des logs """
        mail, smtp = self._get_actual_log_conf()
        tag1 = {'_type':'text_input', 'name':'mail', 'libelle':'Mail admin sauvegarde', 'default_value':mail, 'inline':'ok'}
        tag2 = {'_type':'text_input', 'name':'smtp', 'libelle':'Smtp', 'default_value':smtp, 'inline':'ok'}
        return [tag1, tag2]
    
    def _get_distant_conf(self):
        """ renvoit la dernière config enregistree pour les sauvegardes sur les 
            supports distants
        """
        if isfile(distant_filename):
            lines = file(distant_filename, 'r').readlines()
            if len(lines)>0:
                datas = lines[0]
            else:
                return []
            return self.parse_distant_conf(datas)
        return []

    def _get_usb_conf(self):
        """ renvoit la dernière config enregistree pour les sauvegardes sur les 
            supports usb
        """
        if isfile(usb_filename):
            lines = file(usb_filename, 'r').readlines()
            if len(lines)>0:
                datas = lines[0]
            else: 
                return[]
            return self.parse_usb_conf(datas)
        return []

    def _get_actual_log_conf(self):
        """ renvoit  les paramètres de messagerie de bacula """
        code, retour = commands.getstatusoutput('%sbaculamessage.py -l'%BACULA_DIR)
        if ';' in retour:
            retour = retour.strip()
            datas = retour.split(';')
            smtp = datas[0]
            mail = datas[1]
            return mail, smtp
        return '', ''

    def _get_validate_btn(self):
        """ renvoit la description du bouton valider pour le formulaire form_name """
        validate = {}
        validate['href'] = tools.make_form_link(self.server_nb, self.name, True,['config1', 'config2', 'config3'])
        validate['icone'] = "/image/ok.gif"
        validate['libelle'] = "OK"
        validate['title'] = "Valider vos changements"
        return validate

    def parse_distant_conf(self, datas):
        """ parse un fichier de conf distant """
        firstparse = datas.split()
        machine = firstparse[0].split('/')[2]
        partage = firstparse[0].split('/')[3]
        ipmachine = firstparse[1]
        if len(firstparse)==4:
            login = firstparse[2]
            password = firstparse[3]
        else:
            login = ''
            password = ''
        return [machine, ipmachine, partage, login, password]

    def parse_usb_conf(self, datas):
        """ parse un fichier de conf usb """
        path = datas.strip()
        return [path]

    def modify_conf(self, dico):
        """ valide les modifications de configuration de bacula """
        sup = get_actual_support()
        self.save_type = dico['config1'][0]['value'] # recuperation du type de support
        if sup != self.save_type:
            self._set_support(self.save_type)
        if self.save_type == 'distant': # modification apportees pour 'distant'
            datas = dico['config2']
            changes = {}
            for data in datas:
                name = data['name']
                value = data['value']
                if value.strip() != '':
                    changes[name] = value
            self.write_distant_conf(changes)
        elif self.save_type == 'usb': # modification apportees pour 'usb'
            data = dico['config2'][0]
            if data['name'] == 'usb_periph':
                path = data['value']
                self.write_usb_conf(path)
        log_datas = dico['config3'] # modification des parametres de log
        mail, smtp = [None, None]
        for data in log_datas:
            if data['name'] == 'smtp':
                smtp = data['value']
            if data['name'] == 'mail':
                mail = data['value']
        if mail and smtp not in [None, None]:
            code, retour = commands.getstatusoutput('%sbaculamessage.py -s %s -m %s'%(BACULA_DIR, smtp, mail))
            return code, ''
        return 0, ""

    def write_distant_conf(self, changes):
        """ modifie le fichier de configuration de sauvegarde pour les supports distants"""
        if len(changes.keys()) == 3:
            conf = "//%s/%s %s"%(changes['machine'].strip(), changes['partage'].strip(), changes['ipmachine'].strip())
        elif len(changes.keys()) == 5:
            conf = "//%s/%s %s %s %s"%(changes['machine'].strip(), changes['partage'].strip(), 
                                    changes['ipmachine'].strip(), changes['login'].strip(), changes['password'].strip())
        a = file(distant_filename, 'w')
        a.write(conf)
        a.close()

    def write_usb_conf(self, changes):
        """ modifie le fichier de configuration pour les supports usb """
        a = file(usb_filename, 'w')
        a.write(changes)
        a.close()
        
    def _set_support(self, sup):
        """ definit le support de sauvegarde au niveau de bacula """
        if sup in ['distant' , 'usb', 'bande']:
            code, retour = commands.getstatusoutput('%sbaculasupport.py -s %s'%(BACULA_DIR, sup))            

def get_actual_support():
    """ renvoit le type de support actuel """
    code, retour = commands.getstatusoutput('%sbaculasupport.py -l'%BACULA_DIR)
    if retour in ['distant' , 'usb', 'bande']:
        return retour
    else:
        return ''
