# -*- coding: utf-8 -*-
###########################################################################
# Eole NG - 2007  
# Copyright Pole de Competence Eole  (Ministere Education - Academie Dijon)
# Licence CeCill  cf /root/LicenceEole.txt
# eole@ac-dijon.fr 
#  
# Actions isis, isis_disconnect, isis_stop, isis_login, isis_quota, 
# isis_machine, isis_sendmsg
#  
# Gere les connexions au domaine associe au serveur de commande
#       
###########################################################################
""" Actions permettant de gérer les connexions sur un horus (application Isis) """
from twisted.python import log
from ead2.backend.lib.action import * # import des actions et des types
from ead2.backend.actions.tools import get_request, format_form_result
from ead2.backend.actions.tools import make_form_link, make_confirm_link, make_js_link, make_long_js_link, make_key_form_link
from ead2.lib.libead import uni
from horus.backend import get_active_users, set_active_users
import commands
from re import escape
# import des fonctions specifiques a isis
from ead2.backend.actions.horus.connexion_tools import list_valid_workstations, get_back_to_main, parse_smbstatus, get_menu

used_templates = ['main', 'listlist', 'form', 'alphas', 'checklist', 'formselect']

class Isis(Action):
    """ entrée de l'interface isis, presente dans le menu
        :stopping: booleen indiquant qu'on attend une deconnexion de tout le monde
    """
    user_description = Dict(default={}, doc="description de l'éxécutant", keys=['ip', 'name'])
    name = 'isis'
    libelle = 'Connexion'
    category = 'Outils'
    description = 'gestion des connexions'
    request = Dict(default={}, doc="arguments de la requete en cour cote frontend", keys=['server', 'action', 'stopping'])
    # variable specifique a cette action
    stopping = Boolean(default=False, doc="stop le gestionnaire")

    def execute(self):
       # on recup le numero du serveur en cour de traitement et la requete
        params, server_nb = get_request(self.request)
        if 'stopping' in params.keys():
            self.stop()
        contenu = self.get_content()
        if contenu == []:
            contenu = {'message':"Aucun utilisateur n'est connecté"}
            menu = get_menu(server_nb, page='index')
        else:
            menu = get_menu(server_nb, page='index', extended=True)
        result = {'menus':menu, 'users':contenu, 'sstitre':'index', 'titre':self.name}
        d = {'template':'isis_index', 'data':{'content':result, 'templates':used_templates}}
        return 0, uni(str(d))

    def get_content(self):
        """ renvoit la liste des utilisateurs connectés et de leur activité """
        code, result = commands.getstatusoutput('/usr/bin/smbstatus')
        if code == 0:
            try:
                users = parse_smbstatus(result)
            except:
                return None
            return users
        return None

    def stop(self):
        """ coupe les connexions et deconnecte les utilisateurs """
        users = get_active_users()
        if len(users)>0:
            for user in get_active_users():
                set_active_users(user, False)
        code, result = commands.getstatusoutput('/usr/bin/smbstatus')
        if code == 0:
            try:
                users = parse_smbstatus(result)
            except:
                users = []
            print users
            if len(users)>0:
                for user in users:
                    code, result = commands.getstatusoutput('kill -9 %s '%user['pid'])
                    print "kill"
                    print code, result
        return "La déconnexion s'est bien passée"


class IsisStop(Action):
    """ section d'arret du gestionnaire de connexion
    """
    user_description = Dict(default={}, doc="description de l'éxécutant", keys=['ip', 'name'])
    name = 'isis_stop'
    libelle = 'Arret des Connexions'
    category = None
    description = 'Arret des connexions'
    request = Dict(default={}, doc="arguments de la requete en cour cote frontend", keys=['server', 'action'])

    def execute(self):
        params, self.server_nb = get_request(self.request)
        result = {}
        result['menu'] = get_menu(self.server_nb, page='stop')
        result['valid'] = self.get_valid_btn()
        result['msg1'] = "Êtes-vous sûr de vouloir :"
        result['msg2'] = "Interdire tous les utilisateurs de pouvoir se connecter ?"
        result['msg3'] = "Et de déconnecter tous les utilisateurs actuellement connectés ?"
        result['retour'] = get_back_to_main(self.server_nb)
        d = {'template':'isis_stop', 'data':{'content':result}}
        return 0, uni(str(d)) 

    def get_valid_btn(self):
        """ renvoit le bouton de validation de l'arret des connexions """
        href = make_js_link(self.server_nb, 'isis', stopping="true")
        return {'href':href, 'icone':'/image/ok.gif'}

class IsisLogin(Action):
    """ Gestion des logins et des droits de connexion
    """
    user_description = Dict(default={}, doc="description de l'éxécutant", keys=['ip', 'name'])
    name = 'isis_login'
    libelle = 'Gestion des droits de connexion'
    category = None
    description = 'Gestionnaire de droits de connexion'
    request = Dict(default={}, doc="arguments de la requete en cour cote frontend", 
                    keys=['server', 'action', 'user_alpha', 'frag'])
    form_result = Dict(default={}, doc="arguments Json de la requete", keys=['active', 'unactive'])
            
    def execute(self):
        """ renvoit de quoi afficher la page
        """
        # recuperation du numero de serveur et de la requete sous forme d'objet
        params, self.server_nb = get_request(self.request)
        result = {}
        result['menus'] = get_menu(self.server_nb, page='login')
        result['retour'] = get_back_to_main(self.server_nb)
        result['titre'] = 'Isis' 
        result['sstitre'] = self.libelle
        a = None
        self.user_alpha = ''
        self.frag = ''
        if params.has_key('user_alpha'):
            self.user_alpha = params['user_alpha'][0]
            self.frag = params['frag'][0]
            return self.user_frag()
        if self.form_result!={}:
            try:        
                a = self.exec_changes(self.form_result)
            except:
                result['messages'] = " Une erreur s'est produite lors de la gestion des autorisations"
                d = {'template':'error', 'data':{'content':result}}
                return 0, uni(str(d))
            self.form_result = {}
        if a is not None:# si une erreur non fatale s'est produite
            result['messages'] = a
            d = {'template':'error', 'data':{'content':result}}
            return 0, uni(str(d))
        result.update(self._get_form())
        # description du contenu du corp de l'action
        d = {'template':'isis_login', 'data':{'content':result}}
        return 0, uni(str(d))

    def user_frag(self):
        """ fragment utilisateur """
        result = {}
        if self.frag == 'active': result['datas'] = self._get_active_users()
        else: result['datas'] = self._get_active_users(False)
        d = {'template':'checklist', 'data':{'content':result}}
        return 0, uni(str(d))

    def _get_form(self):
        """ renvoit la description du formulaire d'interdiction-autorisation des utilisateurs """
        left = {'title':"Utilisateurs autorisés", 'form':"active", 'datas':self._get_active_users(True)}
        right = {'title':'Utilisateurs interdits', 'form':"unactive", 'datas':self._get_active_users(False)}
        title = ''
        name = 'horus_autorized_users'
        active_user_alphas = self._get_alpha_links()
        unactive_user_alphas = self._get_alpha_links(False)
        btns = self._get_table_links()
        btns.append(self._get_valid_btn())
        user_table = {'name':name, 'btns':btns, 'left':left, 'right':right, 'title':title}
        return dict(user_table=user_table, 
                    active_user_alphas=active_user_alphas, 
                    unactive_user_alphas=unactive_user_alphas)

    def _get_active_users(self, bool=True):
        try:
            usrs = get_active_users(bool)
        except:
            usrs = []
        result = []
        if self.user_alpha:
            alpha = self.user_alpha.lower()
        else:
            alpha = 'tous'
        if alpha == 'tous':
            return [{'name':usr}for usr in usrs]
        if alpha:
            for usr in usrs:
                if '.' in usr:
                    usrname = usr.split('.')[1].lower()
                    if usrname[0] == alpha:
                        result.append({'name':usr})
                else:
                    if usr[0] == alpha:
                        result.append({'name':usr})
        else:
            result = [{'name':usr}for usr in usrs]
        return result

    def _get_alpha_links(self, active=True):
        """ renvoit la description des liens alphabetique  active: True si c'est pour les users actifs"""
        links = []
        alpha = self.user_alpha
        for i in range(97,123):
            if chr(i) != alpha: 
                if active:href = make_js_link(self.server_nb, self.name, user_alpha=chr(i), frag='active', balise='active_table')
                else:href = make_js_link(self.server_nb, self.name, user_alpha=chr(i), frag='unactive', balise='unactive_table')
            else: href = ''
            libelle = chr(i).upper()
            links.append({'href':href, 'libelle':libelle})
        if 'tous' != alpha:
            if active:href = make_js_link(self.server_nb, self.name, user_alpha='tous', frag='active', balise='active_table')
            else:href = make_js_link(self.server_nb, self.name, user_alpha='tous', frag='unactive', balise='unactive_table')
            links.append({'href':href, 'libelle':'Tous'})
        else:
            links.append({'href':'', 'libelle':'Tous'})
        return links

    def _get_table_links(self):
        """ renvoit les boutons d'entete du tableau de présentation"""
        href1 = "javascript:formTransferListElement('active','unactive_table');"
        href2 = "javascript:formTransferListElement('unactive','active_table');"
        icone1 = "/image/leftright_red.png"
        icone2 = "/image/rightleft.png"
        msg1 = "Interdire"
        msg2 = "Autoriser"
        btns = [{'href':href2, 'libelle':msg2, 'icone':icone2, 'title':msg2},
                {'href':href1, 'libelle':msg1, 'icone':icone1, 'title':msg1}]
        return btns

    def _get_links(self):
        """ renvoit le dico décrivant les boutons de la partie supérieure"""
        msg1 = 'Autoriser tout les utilisateurs'
        msg2 = 'Interdire tout les utilisateurs'
        icone1 = '/image/login_enable.gif'
        icone2 = '/image/login_disable.gif'
        href1 = "javascript:formTransferAllList('unactive', 'active_table');"
        href2 = "javascript:formTransferAllList('active', 'unactive_table');"
        description1 = 'autorise tous les utilisateurs à se connecter'
        description2 = 'interdit la connexion à tous les utilisateurs'
        links = [{'libelle':msg1,'icone':icone1, 'href':href1, 'title':description1},
                {'libelle':msg2,'icone':icone2, 'href':href2, 'title':description2},
                ]
        return links

    def _get_valid_btn(self):
        """ renvoit la description du bouton valider """
        msg = 'Valider vos changements'
        href = make_form_link(self.server_nb, self.name, True, ['active', 'unactive'])
        description = 'valider les changements effectués'
        icone = '/image/ok.gif'
        return {'libelle':msg, 'icone':icone, 'href':href, 'title':description}

    def exec_changes(self, dico):
        """ execute les changements effectues dans la section login de isis
        """
        _list_active = []
        if dico.has_key('active'):
            if dico['active'] != []:
                for item in dico['active']:
                    nom = item['name']
                    value = item['value']
                    _list_active.append(nom)
        else:
            return "Impossible de valider les changements."
        _list_unactive = []
        if dico.has_key('unactive'):
            if dico['unactive'] != []:
                for item in dico['unactive']:
                    nom = item['name']
                    value = item['value']
                    _list_unactive.append(nom)
        else:
            return "Impossible de valider les changements."
        a = set_active_users(_list_active, True)
        b = set_active_users(_list_unactive, False)
        if not a or not b:
            return "Erreur dans la gestion des autorisations"
        return None

class IsisQuota(Action):
    """ renvoit la page de description des états de disque dur"""
    user_description = Dict(default={}, doc="description de l'éxécutant", keys=['ip', 'name'])
    name = 'isis_quota'
    libelle = "Bilan de l'utilisation de l'espace disque"
    category = None
    description = "Affichage de l'utilisation de l'espace disque"
    request = Dict(default={}, doc="arguments de la requete en cour cote frontend", keys=['server', 'action'])

    def execute(self):
        """ renvoit les données pour l'affichage des quotas """
        params, self.server_nb = get_request(self.request)
        result = {}
        result['refresh'] = self._get_refresh_btn()
        try:
            result['quotas'] = self._get_quotas()
        except:
            result['quotas'] = []
        result['menus'] = get_menu(self.server_nb, page='quota')
        result['titre'] = 'Isis'
        result['sstitre'] = self.libelle
        d = {'template':'isis_quotas', 'data':{'content':result, 'templates':used_templates}}
        return 0, uni(str(d))

    def _get_refresh_btn(self):
        """ renvoit la description du bouton refresh """
        href = make_js_link(self.server_nb, self.name)
        return {'href':href, 'icone':"/image/refresh.gif", 'title':"Rafraichir l'affichage", 'libelle':'Rafraichir'}

    def _get_quotas(self):
       """ script de récupération de la sortie de repquota -a
       """
       # nous lançons findsmb et récupérons la sortie 
       commande = "/usr/sbin/repquota -a"
       code, sortie = commands.getstatusoutput(commande)
       users = get_active_users(True)
       users.extend(get_active_users(False))
       if code == 0:
           # nous mettons chaque connexion dans une liste
           lines = sortie.splitlines()
           # parsing des lignes:
           for index, line in enumerate(lines) :
               if '-------' in line:
                   spaces = lines[(index+1):]
                   break
           result = []
           # parsing des espaces disques
           for space in spaces:
               b = {}
               a = space.split()
               if len(a)>0:
                   if a[0] in users:
                       b['user'] = a[0]
                       try:
                           taille = int(a[2])
                           dispo  = int(a[3])
                           if dispo == 0:
                               b['usedspace'] = str(taille/1024)+' (Mo)'
                           else:
                               b['usedspace'] = "%d/%d (Mo)"%(taille/1024, dispo/1024)
                           if a[1] == '+-':
                               b['delay'] = a[5].replace('days',' jours').replace(':',' heures ')
                           else:
                               b['delay'] = ''
                       except:
                           b['usedspace'] = 'undefined'
                           b['delay'] = ''
                       result.append(b)
           return result

class IsisMachines(Action):
    """ Recherche des stations connectées au réseau
    """
    user_description = Dict(default={}, doc="description de l'éxécutant", keys=['ip', 'name'])
    name = 'isis_machine'
    libelle = 'Machines connectées au résea'
    category = None
    description = 'Liste les machines'
    request = Dict(default={}, doc="arguments de la requete en cour cote frontend", keys=['server', 'action', '_type'])
    # variable spécifique
    _type = Text(default='tout', doc="type de machine à afficher")

    def execute(self):
        """ renvoit de quoi afficher la page
        """
        # recuperation du numero de serveur et de la requete 
        params, self.server_nb = get_request(self.request)
        result = {} # dico renvoye au frontend
        # recuperation du type
        if '_type' in params.keys():
            self._type = str(params['_type'][0])
        result['menus'] = get_menu(self.server_nb, page='machine')
        result['titre'] = 'Isis'
        result['sstitre']="Les stations connectés au réseau"
        result['refresh'] = self.get_refresh_btn() 
        result['links'] = self.get_type_btns()
        result['msg'] = "Type de station"
        result['msg_gen'] = " au domaine"
        try:
            result['workstations'] = list_valid_workstations(self._type)
        except:
            result['workstations'] = []
        msg_dict = {'tout': 'Toutes les stations connectées', 'maitre': 'Les maîtres explorateurs connectés', 'controleur':'Les controleurs connectés' }
        result['machinetype'] = msg_dict[self._type]
        d = {'template':'isis_machine', 'data':{'content':result}}
        return 0, uni(str(d)) 

    def get_type_btns(self):
        """ renvoit la description des boutons de choix du type de station à afficher"""
        href1 = make_long_js_link(self.server_nb, self.name, _type="tout")
        href2 = make_long_js_link(self.server_nb, self.name, _type="maitre")
        href3 = make_long_js_link(self.server_nb, self.name, _type="controleur")
        msg1 = "Toutes les stations"
        msg2 = "Maîtres explorateurs"
        msg3 = "Controleurs de domaine"
        return {'href':href1, 'libelle':msg1, 'title':msg1, 'icone':''},{'href':href2, 'libelle':msg2, 'title':msg2, 'icone':''}, {'href':href3, 'libelle':msg3, 'title':msg3, 'icone':''}

    def get_refresh_btn(self):
        """ renvoit le bouton de rarfaichissement """
        href = make_js_link(self.server_nb, self.name, _type="'%s'"%self._type)
        icone = "/image/refresh.gif"
        return {'href':href, 'icone':icone, 'title':'Rafraichir', 'libelle':'Rafraichir'}

class IsisDisconnect(Action):
    """ section de deconnexion des utilisateurs
    """
    user_description = Dict(default={}, doc="description de l'éxécutant", keys=['ip', 'name'])
    name = 'isis_disconnect'
    libelle = 'Déconnection des utilisateurs'
    category = None
    description = 'Déconnecte des utilisateurs'
    request = Dict(default={}, doc="arguments de la requete en cour cote frontend", 
                        keys=['server', 'action'])
    form_result = Dict(default={}, doc="retour de formulaire en syntaxe Json", keys=['isis_index', 'disconnect'])

    def execute(self):
        """ gere la deconnection d'utilisateur 
        """
        params, self.server_nb = get_request(self.request)
        users = None
        return_msg = None
        result = {}
        result['titre'] = self.libelle
        result['retour'] = get_back_to_main(self.server_nb)
        result['valid'] = self.get_valid_btn()
        if self.form_result!={}:
            try:
                if self.form_result.has_key('isis_index'): # on vient de la page d'acceuil
                    users = self.get_users(self.form_result['isis_index'])
                elif self.form_result.has_key('disconnect'):  # on deconnecte les utilisateurs
                    return_msg = self.disconnect(self.form_result['disconnect'])
            except:
                result['messages'] = " Une erreur s'est produite lors de la déconnexion "
                d = {'template':'error', 'data':{'content':result}}
                return 0, uni(str(d))
            self.form_result = {}
        if return_msg is not None: # on a deconnecte les utilisateurs
            result['messages'] = return_msg
            result['retour'] = get_back_to_main(self.server_nb)
            d = {'template':'action_return', 'data':{'content':result}}
            return 0, uni(str(d))
        result['msg'] = "Déconnecter les utilisateurs:"
        result['users'] = users
        if users == [] or users == None:
            result['message'] = "Aucun utilisateur n'a été sélectionné."
            result['toexec'] = make_js_link(self.server_nb, "isis")
        result['formid'] = 'disconnect' # nom du formulaire
        result['checkbox'] = 'force' # nom de la variable message dans le formulaire
        result['checkboxmsg'] = 'Forcer la déconnexion ?'
        d = {'template':'isis_disconnect', 'data':{'content':result}}
        return 0, uni(str(d)) 

    def get_users(self, users_list, checked=True):
        """ renvoit la liste des utilisateurs auxquels on va envoyer un message 
            :checked: les noms d'utilisateurs viennent de checkbox (True)
                        ou les noms d'utilisateurs viennent de hiddenfiels (False)
        """
        if len(users_list) == 0: return None
        users = []
        for dico in users_list:
            if checked:
                if dico['checked'] == 'true':
                    users.append(dico['name'])
            else:
                users.append(dico['name'])
        if users == [] : return None
        return users

    def disconnect(self, form_result):
        """ deconnecte les utilisateurs """
        #FIXME : la deconnexion, qu'est ce que c'est que forcer la deconnexion ?
        force = False
        for var in form_result:# on regarde si il faut forcer la deconnexion
            if var['name'] == 'force':
                if var['checked'] == 'true':
                    force = True
                form_result.remove(var)
        users = self.get_users(form_result, False) # deconnexion des utilisateurs
        return_msg = ["Compte rendu de déconnexion :\n"]
        code, result = commands.getstatusoutput('/usr/bin/smbstatus')
        if code == 0:
            try:
                act_users = parse_smbstatus(result) # recuperation de la liste des utilisateurs
            except:
                act_users = []
            for user in act_users:
                if user['nom'] in users:                        
                    code, result = commands.getstatusoutput('kill -9 %s '%user['pid']) # on deconnecte
                    if code:                
                        return_msg.append("Echec de deconnexion (1) pour  %s.\n"%user['nom'])
                    else:
                        return_msg.append("Déconnexion réussie pour %s.\n"%user['nom'])
        force = True
        if force:
            if users != []:
                for user in users:
                    a = set_active_users(user, False) # on change le statut samba de user
                    if not a:
                        return_msg.append("Echec de deconnexion (2) pour %s.\n "%user)
                return_msg.append(" La déconnexion a été forcée.")
        return return_msg 

    def get_valid_btn(self):
        """ renvoit le bouton de validation de l'arret des connexions """
        ref = make_form_link(self.server_nb, self.name, False, ['disconnect'])
        href = make_confirm_link("Êtes-vous sûr de vouloir couper les connexions ?", ref)
        return {'href':href, 'icone':'/image/ok.gif', 'title':'Envoyer le message', 'libelle':'Envoyer'}

class IsisSendMsg(Action):
    """ section d'envoi de message du gestionnaire de connexion
    """
    user_description = Dict(default={}, doc="description de l'éxécutant", keys=['ip', 'name'])
    name = 'isis_sendmsg'
    libelle = 'Envoi de message'
    category = None
    description = 'Envoi des messages aux utilisateurs'
    request = Dict(default={}, doc="arguments de la requete en cour cote frontend", keys=['server', 'action', '_type'])
    form_result = Dict(default={}, doc="retour de formulaire en syntaxe Json", keys=['isis_index', 'sendmsg'])

    def execute(self):
        """ renvoit les donnees pour le mise en forme de l'action et traite les retours """
        params, self.server_nb = get_request(self.request)
        users = None
        return_msg = None
        result = {}
        result['titre'] = self.libelle
        result['retour'] = get_back_to_main(self.server_nb)
        result.update(self.get_valid_btn())
        if self.form_result!={}:
            if self.form_result.has_key('isis_index'):# on vient de la page d'acceuil
                users = self.get_users(self.form_result['isis_index'])
            elif self.form_result.has_key('sendmsg'): # on envoit le message
                return_msg = self.send_msg(self.form_result['sendmsg'])
            self.form_result = {}
        if return_msg is not None: # on a envoye le message
            result['messages'] = return_msg
            result['retour'] = get_back_to_main(self.server_nb)
            d = {'template':'action_return', 'data':{'content':result}}
            return 0, uni(str(d))
        result['msg'] = "Envoyer un message aux utilisateurs:"
        result['users'] = users
        if users == [] or users == None:
            result['message'] = "Aucun utilisateur n'a été sélectionné"
            result['toexec'] = make_js_link(self.server_nb, "isis")
        result.update(self._get_form())
        d = {'template':'isis_sendmsg', 'data':{'content':result, 'templates':['main', 'form', 'formselect']}}
        return 0, uni(str(d)) 

    def _get_form(self):
        """ renvoit la description du formulaire d'envoi de message """
        msg_input = {'name':'message', 'id':'message','libelle':'Message à envoyer', 'cols':'25', 'rows':'8'}
        return dict(msg_input=msg_input, focus='message')
    
    def get_users(self, users_list, checked=True):
        """ renvoit la liste des utilisateurs auxquels on va envoyer un message 
            :checked: les noms d'utilisateurs viennent de checkbox (True)
                        ou les noms d'utilisateurs viennent de hiddenfields (False)
        """
        if len(users_list) == 0: return None
        users = []
        for dico in users_list:
            if dico['name'] != 'message':
                if checked:
                    if dico['checked'] == 'true':
                        users.append(dico['name'])
                else:
                    users.append(dico['name'])
        if users == [] : return []
        users_list = [{'name':user, 'default_value':user}for user in users]
        return users_list

    def get_valid_btn(self):
        """ renvoit le bouton de validation de l'arret des connexions """
        href = make_form_link(self.server_nb, self.name, True, ['sendmsg'])
        valid = {'href':href, 'icone':'/image/ok.gif', 'libelle':'OK', 'title':"Envoyer le message"}
        validation = make_key_form_link(self.server_nb, self.name, False, 'sendmsg')
        return dict(valid=valid, validation=validation)

    def send_msg(self, form_return):
        """ envoie le message aux utilisateurs """
        message = None
        result = format_form_result(form_return)
        message = result['message']
        try:
            message = message.encode('utf-8')
        except:
            message = message
        users = [user['name'] for user in self.get_users(form_return, False)]
        return_msg = ["Compte rendu de l'envoi du message : ' %s '\n"%message]
        log.msg("message   %s"%message)
        log.msg("user %s"%users)
        try:
            message = message.encode('utf-8')
        except:
            message = message
        if users != []:
            for user in users:
                code, retour = commands.getstatusoutput("echo %s | smbclient -M %s"%(escape(message), escape(user)))
                if code:
                    return_msg.append( "Envoi à %s: échec\n"%user)
                else:
                    return_msg.append("Envoi à %s: réussi \n"%user)
        return return_msg

