# -*- coding: UTF-8 -*-
###########################################################################
# Eole NG - 2007  
# Copyright Pole de Competence Eole  (Ministere Education - Academie Dijon)
# Licence CeCill  cf /root/LicenceEole.txt
# eole@ac-dijon.fr 
#  
# Action gestion_group* 
#  
# Cree modifie et supprime des groupes
#       
###########################################################################

"""Actions permettant de gérer les utilisateurs, compte, partage, groupe (application Gestion)"""
from twisted.python import log
from ead2.backend.lib.action import * # import des actions et des types
from ead2.backend.actions.tools import get_request, make_js_link, make_form_link, make_confirm_link, format_form_result, make_key_form_link
from ead2.backend.config.config import SCRIPT_DIR
from ead2.lib.libead import uni
import commands
from os.path import join
from horus import backend as horus # import des actions (get_groups, mod_user, inscription ... )
from ead2.backend.actions.horus import gestion_tools as G
titre='Gestion des groupes'
used_templates = ['main', 'alphas', 'listlist', 'accordion', 'form', 'formselect', 'checklist' ]

class GestionGroupCreate(Action):
    """ creation d'un groupe """
    user_description = Dict(default={}, doc="description de l'éxécutant", keys=['ip', 'name'])
    name = 'gestion_group_create'
    libelle = "Création"
    category = "Outils/Gestion/Groupes"
    description = 'créer un groupe'
    request = Dict(default={}, doc="arguments de la requete en cour cote frontend", 
                        keys=['server', 'action', 'user_alpha'])
    form_result = Dict(default={}, doc="Retour de formulaire en Json", 
                        keys=['group', 'active', 'unactive', 'share_form', 'alpha_key'])
    
    def execute(self):
        """ renvoit les données pour l'affichage """
        params, self.server_nb = get_request(self.request)
        menu = G.get_group_menu(self.server_nb, self.name)
        sstitre = self.description
        result = {'sstitre':sstitre, 'titre':titre, 'menus':menu}
        result['retour']  = G.get_return_btn(self.server_nb)
        retour = ''
        self.user_alpha = ''
        if params.has_key('user_alpha'): 
            self.user_alpha = params['user_alpha'][0]
            return self.user_frag()
        if self.form_result!={}:
            try:
                retour = self.create_group(self.form_result)
            except:
                retour = G.error_msg("de la création du groupe")
            self.form_result = {}
        result['leftpart'] = self._get_groups()
        result['retour_msg'] = retour
        result.update(self._get_form())
        result.update(self._get_valid_btn())
        d = {'template':'gestion_group', 'data':{'content':result, 
                'templates':used_templates}}
        return 0, uni(str(d))

    def user_frag(self):
        """ renvoit le fragment listant les utilisateurs"""
        result = {}
        result['listlist'] = self._get_form()['group_users']
        d = {'template':'listlist', 'data':{'content':result, 'templates':['checklist']}}
        return 0, uni(str(d))

    def _get_groups(self):
        """ recupere la liste des groupes existants"""
        try:
            groups = ['<b>Groupes existants:</b>']
            groups.extend(horus.get_groups())
        except:
            groups = ["Il n'y a aucun groupe"]
        return groups

    def _get_form(self):
        """ renvoit les données pour le formulaire """
        #input pour le nom d'utilisateur
        group_name = {'name':'group_name', 'libelle':"Nom du groupe", 'onblur':"testVoid('group_name', 'error_group_name')"}
        # section ajout d'utilisateur
        left = {'title':"Utilisateurs disponibles", 'form':"active", 'datas':self._get_users()}
        if self.user_alpha != 'tous' and self.user_alpha:
            left['title'] = "Utilisateurs disponibles <br /> Lettre %s"%self.user_alpha
        right = {'title':"Utilisateurs du groupe", 'form':"unactive", 'datas':[]}
        title = ''
        name = 'group_users'
        group_users = {'name':name, 'btns':self._get_transf_btns1(), 'left':left, 'right':right, 'title':title}
        alphas=self._get_alpha_links()
        # section ajout de partage
        group_shares = {}
        group_shares['btns'] = self._get_transf_btns2()
        group_shares['input'] = {'libelle': 'Nom du partage (sans accent)', 'name':'share'}
        group_shares['liste'] = {'title':'Liste des partages associés au groupe', 'values':[], 'formid':'share_form',
                            'id':'share_table'}
        group_shares['title'] = "Ajouter des partages et les lier au groupe"
        return dict(group_name=group_name, group_users=group_users, group_shares=group_shares, 
                    user_alphas=alphas, focus='group_name')

    def _get_users(self):
        """ renvoit les utilisateurs disponibles dont le nom commence par user_alpha """
        try:
            usrs = horus.get_users()
        except:
            usrs = []
        result = []
        if self.user_alpha:
            alpha = self.user_alpha.lower()
        else:
            alpha = 'tous'
        if alpha == 'tous':
            result = [{'name':usr, 'libelle':usr}for usr in usrs]
            return result
        if alpha:
            for usr in usrs:
                if '.' in usr:
                    usrname = usr.split('.')[1].lower()
                    if usrname[0] == alpha:
                        result.append({'name':usr, 'libelle':usr})
                else:
                    result.append({'name':usr, 'libelle':usr})
        else:
            result = [{'name':usr, 'libelle':usr}for usr in usrs]
        return result

    def _get_transf_btns1(self):
        """ renvoit les btns de transfert de donnees liste a liste pour les utilisateurs"""
        href2 = "javascript:formTransferListElement('active','unactive_table');"
        href1 = "javascript:formTransferListElement('unactive','active_table');"
        icone1 = "/image/rightleft_red.png"
        icone2 = "/image/leftright.png"
        msg1 = "Retirer"
        msg2 = "Ajouter"
        btns = [{'href':href2, 'libelle':msg2, 'icone':icone2, 'title':msg2},
                {'href':href1, 'libelle':msg1, 'icone':icone1, 'title':msg1}]
        return btns
   
    def _get_alpha_links(self):
        """ renvoit la description des liens alphabetique """
        links = []
        alpha = self.user_alpha
        for i in range(97,123):
            if chr(i) != alpha:href = make_js_link(self.server_nb, self.name, balise='group_users', user_alpha=chr(i))
            else: href = ''
            libelle = chr(i).upper()
            links.append({'href':href, 'libelle':libelle})
        if 'tous' != alpha:
            links.append({'href':make_js_link(self.server_nb, self.name, balise='group_users', user_alpha='tous'), 'libelle':'Tous'})
        else:
            links.append({'href':'', 'libelle':'Tous'})
        return links

    def _get_transf_btns2(self):
        """ renvoit les btns de transfert de donnees input a liste pour les partages"""
        href1 = "javascript:formRemoveFromList('share_form');"
        href2 = "javascript:formAddNameToList1('share','share_table');"
        icone1 = "/image/rightleft_red.png"
        icone2 = "/image/leftright.png"
        msg1 = "Retirer"
        msg2 = "Ajouter"
        btns = [{'href':href2, 'libelle':msg2, 'icone':icone2, 'title':msg2},
                {'href':href1, 'libelle':msg1, 'icone':icone1, 'title':msg1}]
        return btns
    
    def _get_valid_btn(self):
        """ renvoit la description du bouton valider """
        validate = {}
        validate['href'] = make_form_link(self.server_nb, self.name, True,  ['group', 'unactive', 'share_form', 'alpha_key'])
        validate['icone'] = "/image/ok.gif"
        validate['libelle'] = "Valider"
        validate['title'] = "Créer ce groupe"
        validation = make_key_form_link(self.server_nb, self.name, False, 'group', 'unactive', 'share_form', 'alpha_key')
        return dict(validate=validate, validation=validation)

    def create_group(self, dico):
        """ cree un groupe depuis le retour formulaire du frontend """
        group = ''
        if dico.has_key('alpha_key'):
            self.user_alpha = dico['alpha_key'][0]['value']
        if dico.has_key('group'):
            result = format_form_result(dico['group']) 
            if result.has_key('group_name'):
                group = result['group_name'] # nom du groupe cree
                a = horus.add_group(group)
                if not a:
                    return "Echec lors de la creation du groupe"
            else:
                return "Erreur lors de la validation du formulaire"
        if dico.has_key('unactive'):# gere la liste des utilisateurs du groupe
            users = dico['unactive']
            if users != []:
                for user in users:
                    nom = user['name']
                    horus.inscription(nom, group) # on inscrit les utilisateurs au groupe
       # gestion des partages attaches au groupe on recupere le nom, le sticky_bit et on cree 
        if dico.has_key('share_form'):
            shares = dico['share_form']
            names = {} 
            stickies = {}
            if shares != []:
                for share in shares:
                    name = share['name']
                    value = share['value']
                    if not 'sticky' in name:
                        names[name] = name
                    else:
                        if share['checked'] == 'true':
                            checked=True
                        else:
                            checked=False
                        name = name[6:] # on prend le nom sans le sticky qui est devant
                        stickies[name] = checked
            if stickies != {}:
                for key in stickies.keys():
                    if names.has_key(key):
                        horus.add_share(key, group, sticky=stickies[key])
        if group:
            return "Le groupe %s a bien été créé"%group
        return "Erreur lors de la validation du formulaire"

class GestionGroupModify(Action):
    """ modification d'un groupe """
    user_description = Dict(default={}, doc="description de l'éxécutant", keys=['ip', 'name'])
    name = 'gestion_group_modify'
    libelle = "Modification"
    category = "Outils/Gestion/Groupes"
    description = 'Modifier un groupe'
    request = Dict(default={}, doc="arguments de la requete en cour cote frontend", 
                    keys=['server', 'action', 'current_group', 'user_alpha'])
    form_result = Dict(default={}, doc="Retour de formulaire en Json", keys=['group', 'active', 'unactive', 'share_form', 'alpha_key'])
    
    def execute(self):
        """ renvoit les données pour l'affichage """
        params, self.server_nb = get_request(self.request)
        menu = G.get_group_menu(self.server_nb, self.name)
        sstitre = self.description
        result = {'sstitre':sstitre, 'titre':titre, 'menus':menu}
        result['retour']  = G.get_return_btn(self.server_nb)
        retour = ''
        nom = ''
        self.user_alpha = ''
        # initialisation du dico de retour
        if self.form_result!={}:
            try:
                retour, nom = self.modify_group(self.form_result)
            except:
                retour = G.error_msg('de la modification du groupe')
                nom = ''
            self.form_result = {}
        if nom:
            self.current_group = nom
        elif 'current_group' in params.keys():
            self.current_group = params['current_group'][0]
        else:
            self.current_group = ''
        if params.has_key('user_alpha'): 
            self.user_alpha = params['user_alpha'][0]
            return self.user_frag()
        result.update(self._get_form())
        result.update(self._get_valid_btn())
        result['leftpart'] = self._get_groups()
        result['retour_msg'] = retour
        d = {'template':'gestion_group', 'data':{'content':result, 
                'templates':used_templates}}
        return 0, uni(str(d))

    def user_frag(self):
        """ renvoit le fragment listant les utilisateurs"""
        result = {}
        result['listlist'] = self._get_form()['group_users']
        d = {'template':'listlist', 'data':{'content':result}}
        return 0, uni(str(d))

    def _get_form(self):
        """ renvoit les données pour le formulaire """
        try:
            groups = horus.get_groups()
        except:
            groups = []
        # si on a pas encore choisit de serveur on ne renvoit que le choix du nom
        if not self.current_group:
            options = [{'name':group, 'libelle':group}for group in groups]
            options.append({'name':'', 'libelle':'', 'default':''})
            group_name = {'name':'group_name', 'options':[{'name':'groupe disponible', 'options':options}], 
                    'libelle':"Nom du groupe",
                    'onchange':make_js_link(self.server_nb, self.name, code=True, current_group='this.options[this.selectedIndex].value')            }
            return dict(group_select=group_name)
        else:
            if self.current_group in groups: groups.remove(self.current_group)
            options = [{'name':group, 'libelle':group}for group in groups]
            options.append({'name':self.current_group, 'libelle':self.current_group,'default':'te'})
            group_name = {'name':'group_name', 'options':[{'name':'groupe disponible', 'options':options}], 
                    'libelle':"Nom du groupe",
                    'onchange':make_js_link(self.server_nb, self.name, code=True, current_group='this.options[this.selectedIndex].value')            }
        # section ajout d'utilisateur
        users = self._get_users()
        members = []
        if self.current_group:
            try: 
                members = horus.get_members(self.current_group)
            except:
                members = []            
            for a in members:
                if a in users:
                    users.remove(a)
        left_opts = [{'name':user}for user in users]
        right_opts = [{'name':member}for member in members]
        left = {'title':"Utilisateurs disponibles", 'form':"active", 'datas':left_opts}
        if self.user_alpha != 'tous' and self.user_alpha:
            left['title'] = "Utilisateurs disponibles <br /> Lettre %s"%self.user_alpha
        right = {'title':"Utilisateurs du groupe", 'form':"unactive", 'datas':right_opts}
        title = ''
        name = 'group_users'
        group_users = {'name':name, 'btns':self._get_transf_btns1(), 'left':left, 'right':right, 'title':title}
        alphas = self._get_alpha_links()
        # section ajout de partage
        group_shares = {}
        group_shares['btns'] = self._get_transf_btns2()
        group_shares['input'] = {'libelle': 'Nom du partage (sans accent)', 'name':'share'}
        try:
            shares = horus.get_group_shares(self.current_group)
        except:
            shares = []
        group_shares['liste'] = {'title':'Liste des partages associés au groupe', 'values':shares, 'formid':'share_form',
                            'id':'share_table'}
        group_shares['title'] = "Ajouter des partages et les lier au groupe"
        return dict(group_select=group_name, group_users=group_users, group_shares=group_shares,
                         user_alphas=alphas)
    
    def _get_groups(self):
        """ recupere la liste des groupes existants"""
        try:
            groups = ['<b>Groupes existants:</b>']
            groups.extend(horus.get_groups())
        except:
            groups = ["Il n'y a aucun groupe"]
        return groups

    
    def _get_users(self):
        """ renvoit les utilisateurs disponibles dont le nom commence par user_alpha """
        try:
            usrs = horus.get_users()
        except:
            usrs = []
        result = []
        if self.user_alpha:
            alpha = self.user_alpha.lower()
        else:
            alpha = 'tous'
        if alpha == 'tous':
            return usrs
        if alpha:
            for usr in usrs:
                if '.' in usr:
                    usrname = usr.split('.')[1].lower()
                    if usrname[0] == alpha:
                        result.append(usr)
                else:
                    result.append(usr)
        else:
            result = usrs
        return result

    def _get_alpha_links(self):
        """ renvoit la description des liens alphabetique """
        links = []
        alpha = self.user_alpha
        for i in range(97,123):
            if chr(i) != alpha:href = make_js_link(self.server_nb, self.name, balise='group_users', user_alpha=chr(i), current_group=self.current_group)
            else: href = ''
            libelle = chr(i).upper()
            links.append({'href':href, 'libelle':libelle})
        if 'tous' != alpha:
            links.append({'href':make_js_link(self.server_nb, self.name, balise='group_users', user_alpha='tous',current_group=self.current_group), 'libelle':'Tous'})
        else:
            links.append({'href':'', 'libelle':'Tous'})
        return links

    def _get_transf_btns1(self):
        """ renvoit les btns de transfert de donnees liste a liste pour les utilisateurs"""
        href2 = "javascript:formTransferListElement('active','unactive_table');"
        href1 = "javascript:formTransferListElement('unactive','active_table');"
        icone1 = "/image/rightleft_red.png"
        icone2 = "/image/leftright.png"
        msg1 = "Retirer"
        msg2 = "Ajouter"
        btns = [{'href':href2, 'libelle':msg2, 'icone':icone2, 'title':msg2},
                {'href':href1, 'libelle':msg1, 'icone':icone1, 'title':msg1}]
        return btns
   
    
    def _get_transf_btns2(self):
        """ renvoit les btns de transfert de donnees input a liste pour les partages"""
        href1 = "javascript:formRemoveFromList('share_form');"
        href2 = "javascript:formAddNameToList1('share','share_table');"
        icone1 = "/image/rightleft_red.png"
        icone2 = "/image/leftright.png"
        msg1 = "Retirer"
        msg2 = "Ajouter"
        btns = [{'href':href2, 'libelle':msg2, 'icone':icone2, 'title':msg2},
                {'href':href1, 'libelle':msg1, 'icone':icone1, 'title':msg1}]
        return btns
    
    def _get_valid_btn(self):
        """ renvoit la description du bouton valider """
        validate = {}
        href = make_form_link(self.server_nb, self.name, False, ['group','active', 'unactive', 'share_form','alpha_key'])
        validate['href'] = make_confirm_link("Êtes-vous sûr de vouloir modifier ce groupe?", href)
#        validate['href'] = "javascript:formValidForm('%s','%s',['group','active', 'unactive', 'share_form'])"%(self.server_nb, self.name)
        validate['icone'] = "/image/ok.gif"
        validate['libelle'] = "Valider"
        validate['title'] = "Modifier ce groupe"
        validation = make_key_form_link(self.server_nb, self.name, False,'group','active','unactive','share_form','alpha_key')
        return dict(validate=validate, validation=validation)


    def modify_group(self, dico):
        """ modifie un groupe
        :dico: -unactive est la liste des users du groupe
                - active est la liste de users qui ne sont pas du groupe
               -unactive_shares est la liste des partages associés au groupe
                - active_shares est la liste des partages qui ne sont pas associés au groupe """
        if dico.has_key('alpha_key'):
            self.user_alpha = dico['alpha_key'][0]['value']
        if dico.has_key('group'):
            param = dico['group']
            if param[0]['name'] == 'group_name':
                group = param[0]['value']
            else:
                group = ''
                return "Echec lors de la modification du groupe", ''
        if group:
            # on va filtrer les valeurs modifiées, on recupere l'etat avant modification
            try:
                myusers = horus.get_members(group)
            except:
                myusers = []
            users = dico['unactive']
            no_users = dico['active']
            for user in users:
                if user['name'] not in myusers:
                    horus.inscription(user['name'], group)
            for user in no_users:
                if user['name'] in myusers:
                    horus.desinscription(user['name'], group)
            if dico.has_key('share_form'):
                try: 
                    myshares = horus.get_group_shares(group)
                except:
                    myshares = []
                shares = dico['share_form']
                names = {} 
                stickies = {}
                for share in shares:
                    name = share['name']
                    value = share['value']
                    if not 'sticky' in name:
                        names[name] = '' # on aurait pu mettre n importe quoi, c la cle qui compte
                    else:
                        if share['checked'] == 'true':
                            checked=True
                        else:
                            checked=False
                        name = name[6:] # on prend le nom sans le sticky qui est devant
                        stickies[name] = checked
                # on cree les partages qui ont ete ajoutes (les nouveaux ont obligatoirement 'sticky' dans le nom
                for key in stickies.keys():
                    if names.has_key(key):
                        if key not in myshares:
                            a = horus.add_share(key, group, sticky=stickies[key])
                            if not a:
                                return "Erreur lors de l'association du partage au groupe", group
                # on supprime les partages qui ont ete enleve
                for name in myshares:
                    if not names.has_key(name):
                        horus.del_share(name)
            return "Le groupe %s a bien été modifié"%group, group
        return "Probleme de validation de formulaire", ''

class GestionGroupDelete(Action):
    """ gere la suppression de partage """
    user_description = Dict(default={}, doc="description de l'éxécutant", keys=['ip', 'name'])
    name = 'gestion_group_suppr'
    libelle = "Suppression"
    category = "Outils/Gestion/Groupes"
    description = 'supprimer un groupe'
    request = Dict(default={}, doc="arguments de la requete en cour cote frontend", keys=['server', 'action'])
    form_result = Dict(default={}, doc="Retour de formulaire en Json", keys=['group'])
    
    def execute(self):
        """ renvoit les données pour l'affichage """
        params, self.server_nb = get_request(self.request)
        menu = G.get_group_menu(self.server_nb, self.name)
        sstitre = self.description
        result = {'sstitre':sstitre, 'titre':titre, 'menus':menu}
        retour = ''
        return_msg = None
        if self.form_result!={}:
            retour = self.del_group(self.form_result)
            self.form_result = {}
        result['retour_msg'] = retour
        result['group_select'] = self._get_form()
        result['validate'] = self._get_valid_btn()
        result['retour']  = G.get_return_btn(self.server_nb)
        d = {'template':'gestion_group', 'data':{'content':result, 'templates':used_templates}}
        return 0, uni(str(d))

    def _get_form(self):
        form = []
        groups = self._get_groups()
        if 'DomainUsers' in groups:
            groups.remove('DomainUsers')
        opts = [{'name':group, 'libelle':group}for group in groups]
        group_select = {'name':'group_name', 'options':[{'name':'groupe existant','options':opts}], 
                        'libelle':"Nom du Groupe"}
        return group_select
    
    def _get_groups(self):
        """ recupere la liste des groupes existants"""
        try:
            groups = horus.get_groups()
        except:
            groups = []
        return groups


    def del_group(self, dico):
        """ supprimer un groupe """
        if dico.has_key('group'):
            result = format_form_result(dico['group'])
            try:
                name = result['group_name']
            except:
                name = None
            if name is not None:
                d = horus.del_group(name)
                if d:
                    return "Le groupe %s a bien été supprimé"%name
                else:
                    return "Erreur lors de la suppression du groupe"
        return "Probleme de validation de formulaire"

    def _get_valid_btn(self):
        """ renvoit la description du bouton valider """
        validate = {}
        href= make_form_link(self.server_nb, self.name, False, ['group'])
        validate['href'] = make_confirm_link("Êtes-vous sûr de vouloir supprimer ce groupe ?", href) 
        validate['icone'] = "/image/ok.gif"
        validate['libelle'] = "Valider"
        validate['title'] = "Supprimer ce groupe"
        return validate
