# -*- coding: UTF-8 -*-
###########################################################################
# Eole NG - 2007  
# Copyright Pole de Competence Eole  (Ministere Education - Academie Dijon)
# Licence CeCill  cf /root/LicenceEole.txt
# eole@ac-dijon.fr 
#  
# Action gestion_share*
#  
# Cree, Modifie, Supprime les partages
#       
###########################################################################
"""Actions permettant de gérer les partages"""
from twisted.python import log
from ead2.backend.lib.action import * # import des actions et des types
from ead2.backend.actions.tools import get_request, make_js_link, make_form_link, make_confirm_link, format_form_result, make_key_form_link
from ead2.lib.libead import uni
import commands
from os.path import join
from horus import backend as horus # import des actions (get_groups, mod_user, inscription ... )
from ead2.backend.actions.horus import gestion_tools as G

titre='Gestion des partages'
used_templates = ['main', 'form', 'formselect']

class GestionShareCreate(Action):
    """ gere la creation de partage """
    user_description = Dict(default={}, doc="description de l'éxécutant", keys=['ip', 'name'])
    name = 'gestion_share_create'
    libelle = "Création"
    category = "Outils/Gestion/Partages"
    description = 'Créer un partage'
    request = Dict(default={}, doc="arguments de la requete en cour cote frontend", keys=['server', 'action'])
    form_result = Dict(default={}, doc="Retour de formulaire en Json", keys=['share'])
    
    def execute(self):
        """ renvoit les données pour l'affichage """
        params, self.server_nb = get_request(self.request)
        retour = ''
        # on regarde si on a un resultat de formulaire et on essaye de valider la creation
        if self.form_result!={}:
            try:
                retour = self.create_share(self.form_result)
            except:
                retour = G.error_msg("de la création du partage")
            self.form_result = {}
        menu = G.get_share_menu(self.server_nb, self.name)
        sstitre = self.description
        result = {'sstitre':sstitre, 'titre':titre, 'menus':menu}
        result['retour_msg'] = retour
        result.update(self._get_form())
        result.update(self._get_valid_btn())
        result['shares'] = self._get_shares()
        d = {'template':'gestion_share', 'data':{'content':result, 'templates':used_templates}}
        return 0, uni(str(d))

    def _get_shares(self):
        """ renvoit les données pour la présentation des partages existant """
        try:
            datas = horus.get_special_shares()
        except:
            datas = {}
        return {"message":"Lettres de partages déjà attribuées", "datas":datas}

    def _get_form(self):
        """ renvoit les données pour le formulaire """
        share_name = {'name':'share_name', 'libelle':"Nom du partage", 
                        'onblur':"testVoid('share_name', 'error_share_name')", 'inline':'ok'}
        group = {'name':'group', 'libelle':"Groupe associé (existant ou non)", 
                    'onblur':"testVoid('group', 'error_group')", 'inline':'ok'}
        filepath = {'name':'filepath', 'libelle':"Chemin spécifique au partage (facultatif,<br /> /home/workgroups/+nom du partage par defaut)", 'inline':'ok'}
        disque = {'name':'disque', 'libelle':"Lettre de lecteur", 'inline':'ok'}
        sticky_bit = {'name':'sticky_bit', 'value':'on', 
                        'libelle':"Activation du sticky bit", 'inline':'ok'}
        try:
            models = horus.get_share_templates()
        except:
            models = []
        if 'standard' in models:
            models.remove('standard')
        options = [{'name':model, 'libelle':model}for model in models]
        options.append({'name':'standard', 'libelle':'standard', 'default':'ok'})
        model = {'name':'model', 'options':[{'name':'modeles', 'options':options}], 
                                        'libelle':"Modèle de partage", 'inline':'ok'}
        return dict(share_name=share_name, group=group, filepath=filepath, 
                    disque=disque, sticky_bit=sticky_bit, model=model)

    def _get_valid_btn(self):
        """ renvoit la description du bouton valider """
        validate = {}
        validate['href'] = make_form_link(self.server_nb, self.name, True, ['share'])
        validate['icone'] = "/image/ok.gif"
        validate['libelle'] = "Valider"
        validate['title'] = "Créer ce utilisateur"
        validation =  make_key_form_link(self.server_nb, self.name, False, 'share')
        return dict(validate=validate, validation=validation)

    def create_share(self, dico):
        """ cree un partage 
            :dico: dico venant de l'interface
        """
        if dico.has_key('share'):
            filepath, disque, sticky_bit, model = [None, None, False, 'standard']
            params = format_form_result(dico['share'])
            nom, group, filepath, disque, model = params['share_name'], params['group'], params['filepath'], params['disque'], params['model']
            if not filepath: filepath = None
            if not model: model = None
            # controle de la lettre de lecteur
            if not disque: disque = None
            else:
                shares = horus.get_special_shares()
                if not ':' in disque: disque = disque+':'
                if len(disque)>2 : 
                    return "La syntaxe est incorrecte (ex: k:)"
                if disque.upper() in [letter.upper()for letter  in shares.values()]:
                    return "La lettre de lecteur est déjà attribuée"
            if params.has_key('sticky_bit'):
                if params['sticky_bit'] == 'on': sticky_bit = True
                else: sticky_bit = False
            if nom and group:
                if group not in horus.get_groups():
                    if not horus.add_group(group):
                        return "Echec lors de la creation du groupe %s"%group
                if horus.add_share(nom, group, filepath, disque, sticky_bit, model):
                    return "Le groupe %s et le partage %s ont bien été créés et associés"%(group, nom) 
                else:
                    return "Echec lors de la création du partage %s"%nom
            else:
                return "Il manque des paramètres pour la création du partage %s"%nom
        return "Erreur de validation de formulaire"

class GestionShareModify(Action):
    """ gere la modification de partage """
    user_description = Dict(default={}, doc="description de l'éxécutant", keys=['ip', 'name'])
    name = 'gestion_share_modify'
    libelle = "Modification"
    category = "Outils/Gestion/Partages"
    description = 'Modifier un partage'
    request = Dict(default={}, doc="arguments de la requete en cour cote frontend", 
                        keys=['server', 'action', 'current_share'])
    form_result = Dict(default={}, doc="Retour de formulaire en Json", keys=['share'])
    
    def execute(self):
        """ renvoit les données pour l'affichage """
        params, self.server_nb = get_request(self.request)
        retour = ''
        nom = ''
        # on regarde si on a un resultat de formulaire et on essaye de valider la creation
        if self.form_result!={}:
            #try:
            retour, nom = self.modify_share(self.form_result)
            #except:
            #    retour = G.error_msg("de la modification du partage")
            self.form_result = {}
        if nom:
            self.current_share = nom
        else:
            self.current_share = ''
        # on recupere le nom de l'utilisateur si le choix a ete fait
        if 'current_share' in params.keys():
            self.current_share = params['current_share'][0]
        menu = G.get_share_menu(self.server_nb, self.name)
        sstitre = self.description
        result = {'sstitre':sstitre, 'titre':titre, 'menus':menu}
        result['retour_msg'] = retour
        result.update(self._get_form())
        result['shares'] = self._get_shares()
        d = {'template':'gestion_share', 'data':{'content':result, 'templates':used_templates}}
        return 0, uni(str(d))

    def _get_shares(self):
        """ renvoit les données pour la présentation des partages existant """
        try:
            datas = horus.get_special_shares()
        except:
            datas = {}
        return {"message":"Lettres de partages déjà attribuées", "datas":datas}

    def _get_form(self):
        """ renvoit les datas pour le formulaire """
        # si on a pas choisit de partage
        try:
            shares_dict = horus.get_shares()
        except:
            shares_dict = {}
        shares = shares_dict.keys()
        if not self.current_share:
            options = [{'name':share, 'libelle':share}for share in shares]
            options.append({'name':'', 'libelle':'', 'default':'ok'})
            share_select = {'name':'share_name', 'options':[{'name':'partages existants','options':options}],
                    'libelle':"Nom du partage", 'inline':'ok',
                    'onchange':make_js_link(self.server_nb, self.name, code=True, current_share='this.options[this.selectedIndex].value')}
            return dict(share_select=share_select)
        if self.current_share in shares:
            shares.remove(self.current_share)
        options = [{'name':share, 'libelle':share}for share in shares]
        options.append({'name':self.current_share, 'libelle':self.current_share, 'default':'ok'})
        share_select = {'_type':'choice', 'name':'share_name', 'options':[{'name':'partages existants','options':options}],
                'libelle':"Nom du partage", 'inline':'ok',
                'onchange':make_js_link(self.server_nb, self.name, code=True, current_share='this.options[this.selectedIndex].value')              }
        letter = shares_dict[self.current_share]
        if letter == 'None': letter = 'aucune'
        disque = {'name':'disque', 'libelle':"Lettre de lecteur", 'default_value':letter,'inline':'ok'
                }
        try:
            models = horus.get_share_templates()
        except:
            models = []
        if 'standard' in models:
            models.remove('standard')
        options = [{'name':model, 'libelle':model}for model in models]
        options.append({'name':'standard', 'libelle':'standard', 'default':'ok'})
        model = {'name':'model', 'options':[{'name':'modeles', 'options':options}], 
                                        'libelle':"Modèle de partage", 'inline':'ok'}
        return dict(share_select=share_select, disque=disque, model=model, validate=self._get_valid_btn())

    def _get_valid_btn(self):
        """ renvoit la description du bouton valider """
        validate = {}
        href = make_form_link(self.server_nb, self.name, False, ['share'])
        validate['href'] = make_confirm_link("Êtes-vous sûr de vouloir modifier ce partage?", href)
        validate['icone'] = "/image/ok.gif"
        validate['libelle'] = "Valider"
        validate['title'] = "Modifier ce partage"
        return validate

    def modify_share(self, dico):
        """ modifie un partage 
            :dico: dico venant de l'interface
        """
        if dico.has_key('share'):
            nom = ''
            drive = None
            model = None
            params = dico['share']
            for param in params:
                name = param['name']
                value = param['value']
                if name == 'share_name':
                    nom = value
                if name == 'disque':
                    drive = value
                if name == 'share_model':
                    model = value
            # controle de la lettre de lecteur
            shares = horus.get_special_shares()
            if drive == 'aucune': drive=None
            elif drive:
                if not ':' in drive: drive += ':'
                if len(drive)!=2:
                    return "La syntaxe est incorrecte (ex: k:)",""
                if drive.upper() in [letter.upper()for letter  in shares.values()]:
                    return "La lettre de lecteur est déjà attribuée" , ""
            if nom and nom not in ['icones$', 'groupes']:
                horus.mod_share(nom, drive, model)
                return "Le partage %s a bien été modifié"%nom, nom
        return "Problème de validation de formulaire", ''

class GestionShareDelete(Action):
    """ gere la suppression de partage """
    user_description = Dict(default={}, doc="description de l'éxécutant", keys=['ip', 'name'])
    name = 'gestion_share_suppr'
    libelle = "Suppression"
    category = "Outils/Gestion/Partages"
    description = 'supprimer un partage'
    request = Dict(default={}, doc="arguments de la requete en cour cote frontend", keys=['server', 'action'])
    form_result = Dict(default={}, doc="Retour de formulaire en Json", keys=['share'])
    
    def execute(self):
        """ renvoit les données pour l'affichage """
        params, self.server_nb = get_request(self.request)
        menu = G.get_share_menu(self.server_nb, self.name)
        retour = ''
        removing = None
        if self.form_result!={}:
            retour = self.del_share(self.form_result)
            self.form_result = {}
        sstitre = self.description
        result = {'sstitre':sstitre, 'titre':titre, 'menus':menu}
        result['retour_msg'] = retour
        result.update(self._get_form())
        result['validate'] = self._get_valid_btn()
        d = {'template':'gestion_share', 'data':{'content':result, 'templates':used_templates}}
        return 0, uni(str(d))

    def _get_form(self):
        # si on a pas choisit de partage
        try:
            shares_dict = horus.get_shares()
        except:
            shares_dict = {}
        shares = shares_dict.keys()
        options = [{'name':share, 'libelle':share}for share in shares]
        share_select = {'name':'share_name', 'options':[{'name':'partages existants','options':options}],
                'libelle':"Nom du partage", 'inline':'ok'}
        return dict(share_select=share_select)

    def del_share(self, dico):
        """ supprimer un partage """
        if dico.has_key('share'):
            nom = None
            removing=False
            params = dico['share']
            for param in params:
                name = param['name']
                value = param['value']
                if name == 'share_name':
                    nom = value
            if nom is not None:
                horus.del_share(nom)
                return "Le partage %s a bien été supprimé"%nom
        return "Problème de validation de formulaire"

    def _get_valid_btn(self):
        """ renvoit la description du bouton valider """
        validate = {}
        href = make_form_link(self.server_nb, self.name, False, ['share'])
        validate['href'] = make_confirm_link("Êtes-vous sûr de vouloir supprimer ce partage?", href)
        validate['icone'] = "/image/ok.gif"
        validate['libelle'] = "Valider"
        validate['title'] = "Supprimer ce partage"
        return validate
