# -*- coding: UTF-8 -*-
###########################################################################
# Eole NG - 2007  
# Copyright Pole de Competence Eole  (Ministere Education - Academie Dijon)
# Licence CeCill  cf /root/LicenceEole.txt
# eole@ac-dijon.fr 
#  
# Action gestion_user*
#  
# Cree Modifie et Supprime des utilisateurs
#       
###########################################################################

"""Actions permettant de gérer les utilisateurs"""
from twisted.python import log
from ead2.backend.lib.action import * # import des actions et des types
from ead2.backend.actions import tools
from ead2.backend.config.config import SCRIPT_DIR
from ead2.lib.libead import uni
import commands
from os.path import join
from horus import backend as horus # import des actions (get_groups, mod_user, inscription ... )
from ead2.backend.actions.horus import gestion_tools as G

titre = 'Gestion des utilisateurs'
used_templates = ['form', 'main', 'formselect', 'listlist', 'accordion', 'checklist']

def _get_groups():
    """ renvoit la liste des groupes """
    try:
        grs = horus.get_groups()
    except:
        grs = []
    if "DomainAdmins" in grs: grs.remove("DomainAdmins")
    return grs

class GestionUserCreate(Action):
    """ création d'utilisateur """
    user_description = Dict(default={}, doc="description de l'éxécutant", keys=['ip', 'name'])
    name = 'gestion_user_create'
    libelle = "Création"
    category = "Outils/Gestion/Utilisateurs"
    description = 'Créer un utilisateur'
    request = Dict(default={}, doc="arguments de la requete en cour cote frontend", keys=['server', 'action'])
    form_result = Dict(default={}, doc="Retour de formulaire en Json", keys=['user'])

    def execute(self):
        """ renvoit les données pour l'affichage """
        params, self.server_nb = tools.get_request(self.request)
        retour = ''
        # on regarde si on a un resultat de formulaire et on essaye de valider la creation
        if self.form_result!={}:
            try:
                retour = self.create_user(self.form_result)
            except Exception, mess:
                retour = mess.message #G.error_msg("de la création de l'utilisateur")
            self.form_result = {}
        menu = G.get_user_menu(self.server_nb, self.name)
        sstitre = self.description
        result = {'sstitre':sstitre, 'titre':titre, 'menus':menu}
        result['retour_msg'] = retour
        result.update(self._get_form())
        result.update(self._get_valid_btn())
        result['users'] = ['<b>Utilisateurs existants:</b>']
        result['users'].extend(self._get_users())
        d = {'template':'gestion_users', 'data':{'content':result, 'templates':used_templates}}
        return 0, uni(str(d))

    def _get_form(self):
        """ renvoit les datas pour la creation du formulaire """
        user_name = {'name':'user_name', 'libelle':"Nom de l'utilisateur",'onblur':"testVoid('user_name', 'error_user_name')", 'inline':'ok'}
        password= {'name':'password', 'libelle':"Mot de passe", 'onblur':"testVoid('password', 'error_password')", 'inline':'ok'}
        quota = {'name':'quota', 'libelle':"Quota utilisateur", 'default_value':'0', 'inline':'ok'}
        grs = _get_groups()
        groups = [{'name':gr, 'libelle':gr}for gr in grs]
        group_select = {'name':'group', 'options':[{'name':'groupes disponibles','options': groups}],
                'libelle':'Groupe principal', 'inline':'ok'}
        profils = [{'name':'itinerant', 'libelle':'itinerant'}, {'name':'local', 'libelle':'local', 'default':'ok'}]
        profil = {'name':'profil', 'options': [{'name':'profils disponibles', 'options':profils}], 
                'libelle':'Profil utilisateur', 'inline':'ok'}
        disque = {'name':'disque', 'libelle': "Lettre de lecteur", "default_value":"U:", 'disabled':'true', 'inline':'ok'}
        enable = {'name':'enable', 'value':'on', 'checked':'True', 'libelle':"Activer l'utilisateur", 'inline':'ok'}
        usrs = [{'name':user, 'libelle':user}for user in self._get_users()]
        usrs.append({'name':'', 'libelle':'', 'default':''})
        group_copy = {'name':'copy', 'options':[{'name':'utilisateurs diponibles', 'options':usrs}], 
                'libelle':"Copier les groupes d'un autre utilisateur", 'inline':'ok'}
        admin = {'name':'admin', 'libelle': "Membre du groupe DomainAdmins", 'inline':'ok', 'value':'admin'}
        return dict(user_name=user_name, password=password, quota=quota, group_select=group_select,
                profil=profil, disque=disque, enable=enable, group_copy=group_copy, admin=admin, focus='user_name')

    def _get_valid_btn(self):
        """ renvoit la description du bouton valider """
        validate = {}
        validate['href'] = tools.make_form_link(self.server_nb, self.name, True, ['user'])
        validate['icone'] = "/image/ok.gif"
        validate['libelle'] = "Valider"
        validate['title'] = "Créer ce nouvel utilisateur"
        validation = tools.make_key_form_link(self.server_nb, self.name, False,  'user')
        return dict(validate=validate, validation=validation)

    def _get_users(self):
        try:
            usrs = horus.get_users()
        except:
            usrs = []
        return usrs

    def create_user(self, dico):
        """ modifie un utilisateur depuis un dico récup depuis l'interface
            :user_name: nom utilisateur
            :group: groupe principal
            :disque: lettre du lecteur associe
            :enable: actif ou pas
            :profil: 'local' ou 'itinerant'
            :copy: utilisateur modele pour la copie des groupes d'appartenance
        """
        result = tools.format_form_result(dico['user'], True)
        for key in ['user_name', 'enable', 'disque', 'quota', 'password', 'copy', 'profil', 'admin', 'group']:
            if key not in result.keys():
                raise Exception, "Erreur : il manque des données pour la création d'utilisateur."
        name = result['user_name'].strip()
        if result['profil'] == 'itinerant': profil = True
        else: profil = False
        # si l'utilisateur doit être domainAdmin, cela devient son groupe principale
        dd = result['disque'].strip()
        copy = result['copy'].strip()
        if copy != '':
            try:
                grs = horus.get_user_groups(copy)
            except:
                raise Exception , "Erreur : erreur à la récupération des groupes de %s"%copy
        else:grs = []
        gr = result['group'].strip()
        admin = result['admin']
        if admin: 
            grs.append(gr)
            gr = "DomainAdmins"
        if result['quota'].strip() != '': quota = result['quota'].strip()
        else:quota = '0'
        try:
            int(quota)
        except:
            raise Exception, "Erreur : le quota doit être un nombre."
        pwd = result['password']
        activity = result['enable']
        if '' in [name, dd, pwd, gr]: raise Exception, "Erreur : problème de validation de formulaire."
        if not horus.is_user(name):
            a = horus.add_user(name, gr, grs, dd, profil)
            if a:
                    commande = '%s %s %s' % (join(SCRIPT_DIR,'set_quota.py'), name, quota)
                    code, sortie = commands.getstatusoutput(commande)
                    horus.mod_password(name, pwd)
                    horus.set_active_users(name, activity)
                    return "L'utilisateur %s a bien été créé"%name
        raise Exception, "Erreur : Problème de validation de formulaire"

class GestionUserModify(Action):
    """ modification d'utilisateur """
    user_description = Dict(default={}, doc="description de l'éxécutant", keys=['ip', 'name'])
    name = 'gestion_user_modify'
    libelle = "Modification"
    category = "Outils/Gestion/Utilisateurs"
    description = 'modifier un utilisateur'
    request = Dict(default={}, doc="arguments de la requete en cour cote frontend", 
                            keys=['server', 'action', 'current_user'])
    form_result = Dict(default={}, doc="retour de formulaire en Json", keys=['user', 'active', 'unactive'])
    
    def execute(self):
        """ renvoit les données pour l'affichage """
        params, self.server_nb = tools.get_request(self.request)
        retour =''
        nom = ''
        # on regarde si on a un resultat de formulaire et on essaye de valider la modification
        if self.form_result!={}:
            try:
                retour, nom = self.modify_user(self.form_result)
            except:
                import traceback
                traceback.print_exc()
                retour = G.error_msg("de la modification de l'utilisateur")
            self.form_result = {}
        if nom: self.current_user = nom
        else: self.current_user = ''
        # on recupere le nom de l'utilisateur si le choix a ete fait
        if 'current_user' in params.keys():
            self.current_user = params['current_user'][0]
        menu = G.get_user_menu(self.server_nb, self.name)
        sstitre = self.description
        result = {'sstitre':sstitre, 'titre':titre, 'menus':menu}
        result['retour_msg'] = retour
        result.update(self._get_form())
        d = {'template':'gestion_users', 'data':{'content':result, 'templates':used_templates}}
        return 0, uni(str(d))
    
    def _get_user_listlist(self):
        """ renvoit les datas pour les listes de choix des groupes """
        result = {}
        result['btns'] = self._get_transf_btns() # boutons de transfert liste a liste
        right_opts = [{'name':gr}for gr in self._get_unactive_groups(self.current_user)]
        result['right'] = {'title':"Groupes disponibles", 'form':"unactive", 
                        'datas':right_opts}
        left_opts = [{'name':gr}for gr in self._get_active_groups(self.current_user)]
        result['left'] = {'title':"Groupes de l'utilisateur", 'form':"active",
                            'datas':left_opts} # les autres groupes
        result['title'] = "Associer des groupes à l'utilisateur"
        result['name'] = 'user_group_list'
        return result
    
    def _get_transf_btns(self):
        """ renvoit les btns de transfert de donnees liste a liste"""
        href1 = "javascript:formTransferListElement('active','unactive_table');"
        href2 = "javascript:formTransferListElement('unactive','active_table');"
        icone1 = "/image/leftright_red.png"
        icone2 = "/image/rightleft.png"
        libelle1 = "Retirer"
        libelle2 = "Ajouter"
        btns = [{'href':href2, 'libelle':libelle2, 'icone':icone2, 'title':libelle1},
                {'href':href1, 'libelle':libelle1, 'icone':icone1, 'title':libelle2}]
        return btns

    def _get_users(self):
        """ renvoit la liste des utilisateurs """
        try:
            usrs = horus.get_users()
        except:
            usrs = []
        return usrs

    def _get_form(self):
        """ renvoit les datas pour le formulaire """
        usrs = self._get_users()
        if not self.current_user:
            options = [{'name':usr, 'libelle':usr}for usr in usrs]
            options.append({'name':'','libelle':'', 'default':'ok'})
            user_name = {'name':'user_name', 'options':[{'name':'utilisateurs disponibles','options':options}],
                     'libelle':"Nom de l'utilisateur",
                    'onchange':tools.make_js_link(self.server_nb, self.name, code=True,  current_user = 'this.options[this.selectedIndex].value') }
            return dict(user_select=user_name)
        datas = self._get_user_data()
        if self.current_user in usrs: 
            usrs.remove(self.current_user)
        # liste des utilisateurs a modifier
        users = [{'name':usr, 'libelle':usr}for usr in usrs]
        users.append({'name':self.current_user, 'libelle':self.current_user, 'default':'ok'})
        options = [{'name':usr, 'libelle':usr}for usr in usrs]
        options.append({'name':self.current_user,'libelle':self.current_user, 'default':'ok'})
        user_name = {'name':'user_name', 'options':[{'name':'utilisateurs disponibles','options':options}],
                 'libelle':"Nom de l'utilisateur",
                 'onchange':tools.make_js_link(self.server_nb, self.name, code=True,  current_user = 'this.options[this.selectedIndex].value') , 'inline':'ok'}
        password= {'name':'password', 'libelle':"Mot de passe", 'inline':'ok'}
        # quota
        if datas.has_key('current_quota'):current_quota = datas['current_quota']
        else:current_quota=''
        quota = {'name':'quota', 'libelle':"Quota utilisateur", 'default_value':current_quota, 'inline':'ok'}
        #construction de la liste des groupes
        grs = _get_groups()
        if datas.has_key('current_gr'):current_gr = datas['current_gr']
        else:current_gr = ''
        if current_gr in grs: grs.remove(current_gr)
        gr_opts = [{'name':gr, 'libelle':gr}for gr in grs]
        gr_opts.append({'name':current_gr, 'libelle':current_gr,  'default':'ok'})
        group_select = {'name':'group', 'options': [{'options':gr_opts}], 'libelle':'Groupe principal', 'inline':'ok'}
        # construction de la liste des profils
        profils = ['itinerant', 'local']
        if datas.has_key('current_profil'):current_profil = datas['current_profil']
        else:current_profil = 'local'
        if current_profil in profils:profils.remove(current_profil)
        opt_profils = [{'name':profil, 'libelle':profil}for profil in profils]
        opt_profils.append({'name':current_profil, 'libelle':current_profil, 'default':'ok'})
        profil = {'name':'profil', 'options':[{'options':opt_profils}], 'libelle':'Profil utilisateur', 'inline':'ok'}
        #construction de la liste des lettres de lecteur
        dds = G._get_dds()
        if datas.has_key('current_dd'): current_dd = datas['current_dd']
        else: current_dd = ''
        if current_dd: dds.remove(current_dd)
        dds_opts = [{'name':dd, 'libelle':dd}for dd in dds]
        dds_opts.append({'name':current_dd, 'libelle':current_dd,'default':'ok'})
        disque = {'name':'disque', 'options': [{'options':dds_opts}], 'libelle':'Lecteur associé', 'inline':'ok'}
        #statut d'activite d'un utilisateur
        #FIXME: il manque la recuperation du statut d'activite de l'utilisateur
        enable = {'name':'enable', 'value':'on', 'libelle':"Activer l'utilisateur", 'inline':'ok'}
        activity = True
        if datas.has_key('enable'): activity = datas['enable']
        if activity: enable['checked'] = 'True'
        users = [{'name':usr, 'libelle':usr}for usr in usrs]
        users.append({'name':'', 'libelle':'','default':'ok'})
        group_copy = {'name':'copy', 'options':[{'options':users}], 'libelle':"Copier les groupes d'un autre utilisateur",
                    'inline':'ok'}
        user_group_list = self._get_user_listlist()
        return dict(user_select=user_name, password=password, quota=quota, group_select=group_select,
                profil=profil, disque=disque, enable=enable, group_copy=group_copy, validate=self._get_valid_btn(),
                user_group_list=user_group_list)

    def modify_user(self, dico):
        """ modifie les attributs d'un utilisateur """
        # gestion des parametres utilisateurs
        if dico.has_key('user'):
            params = dico['user']
            recup = tools.format_form_result(dico['user'], True)
            nom = recup['user_name']
            pwd, profil, dd, enable, group, enable, groups = [None, None, None, None, None, False, []]
            if recup.has_key('password'):
                if recup['password'] != '':  pwd = recup['password']
            if recup.has_key('profil'):
                if recup['profil'] == 'itinerant': profil = True
                else: profil = False
            if recup.has_key('quota'):
                if recup['quota'] != '':
                    commande = '%s %s %s'%(join(SCRIPT_DIR,'set_quota.py'), nom, recup['quota'])
                    code, sortie = commands.getstatusoutput(commande)
            if recup.has_key('disque'):
                if recup['disque'] != '': dd = recup['disque']
            if recup.has_key('enable'):enable = recup['enable']
            if recup.has_key('group'):
                if recup['group'] != '': gr = recup['group']
            if recup.has_key('copy'):
                try:
                    groups = horus.get_user_groups(recup['copy'])
                except:
                    groups = []
            if nom is not None:
                a = horus.mod_user(nom, homedrive=dd, enable=enable, profile=profil, gid=gr)
                if not a:
                    return "Echec lors de la modification de l'utilisateur %s"%nom, nom
                horus.set_active_users(nom, enable) # on gere l activite d'un utilisateur
                if pwd != None:
                    horus.mod_password(nom, pwd) # on change le mot de passe
                if dico.has_key('active') and dico.has_key('unactive'):
                    active = dico['active']
                    unactive = dico['unactive']
                    # on teste les changements pour eviter les changements dans le vide
                    actual_groups = self._get_active_groups(nom)
                    actual_no_groups = self._get_unactive_groups(nom)
                    for gr in active:
                        if gr['name'] not in actual_groups:
                            horus.inscription(nom, gr['name'])
                    for ugr in unactive:
                        if ugr['name'] not in actual_no_groups:
                            if ugr['name'] not in groups:
                                horus.desinscription(nom, ugr['name'])
                                # si on a demande a copier des groupes depus un utilisateur, on y donne priorite
                        elif ugr['name'] in groups:
                            horus.inscription(nom, ugr['name'])
            return "L'utilisateur %s a bien été modifié"%nom, nom
        return "Problème de validation de formulaire", None


    def _get_user_data(self):
        """ renvoit les attributs de user"""
        user = self.current_user
        if not user:
            return {}
        if not horus.is_user(user):
            return {}
        try:
            datas = horus.get_user_data(user)
        except:
            return {}
        if 'sambaProfilePath' in datas.keys():
            profil = "itinerant"
        else:
            profil = 'local'
        try:
            actives = horus.get_active_users()
            if user in actives:
                enable = True
            else:
                enable = False
        except:        
            enable = True
        dd = datas['sambaHomeDrive'][0]
        gr = horus.get_gid(user)
        commande = '%s %s' % (join(SCRIPT_DIR,'get_quota.py'), user)
        code, sortie = commands.getstatusoutput(commande)
        quota = 0
        if code == 0:
            quota = sortie
        return {'current_gr':gr, 'current_profil': profil, 'current_dd': dd, 'current_quota':quota, 'enable':enable}

    def _get_active_groups(self, user):
        """ renvoit les groupes actifs pour user"""
        if user is None:
            return []
        try:
            usr_grs = horus.get_user_groups(user)
        except:
            usr_grs = []
        return usr_grs
    
    def _get_unactive_groups(self, user):
        """ renvoit les groupes inactifs pour user"""
        if user is None:
            return []
        try:
            usr_grs = horus.get_user_no_groups(user)
        except:
            usr_grs = []
        return usr_grs

    def _get_valid_btn(self):
        """ renvoit la description du bouton valider """
        validate = {}
        validate['href'] = tools.make_form_link(self.server_nb,self.name, True, ['user', 'active', 'unactive'])
        validate['icone'] = "/image/ok.gif"
        validate['libelle'] = "Valider"
        validate['title'] = "Modifier l'utilisateur"
        return validate

class GestionUserDelete(Action):
    """ Suppression d'utilisateur """
    user_description = Dict(default={}, doc="description de l'éxécutant", keys=['ip', 'name'])
    name = 'gestion_user_suppr'
    libelle = "Suppression"
    category = "Outils/Gestion/Utilisateurs"
    description = 'Supprimer un utilisateur'
    request = Dict(default={}, doc="arguments de la requete en cour cote frontend", keys=['server', 'action'])
    form_result = Dict(default={}, doc='retour formulaire pour la suppression', keys=['user'])
    
    def execute(self):
        """ renvoit les données pour l'affichage """
        params, server_nb = tools.get_request(self.request)
        menu = G.get_user_menu(server_nb, self.name)
        retour = None
        removing = None
        if self.form_result!={}:
            retour, removing = self.del_user(self.form_result)
            self.form_result = {}
        sstitre = self.description
        result = {'sstitre':sstitre, 'titre':titre, 'menus':menu}
        if removing:
            retour += " ainsi que son répertoire"
        result['retour_msg'] = retour
        result.update(self._get_form())
        result['validate'] = self._get_valid_btn(server_nb)
        d = {'template':'gestion_users', 'data':{'content':result, 'templates':used_templates}}
        return 0, uni(str(d))

    def _get_form(self):
        form = []
        try: 
            usrs = horus.get_users()
        except:
            usrs = []
        users = [{'name':usr, 'libelle':usr}for usr in usrs]
        user_select = {'name':'user_name', 'options':[{'name':'utilisateurs existants', 'options':users}],
                        'libelle':"Nom d'utilisateur"}
        del_rep = {'name':'remove_data', 'value':'on', 'checked':'True', 'libelle':"Supprimer son répertoire"}
        return dict(user_select=user_select, enable=del_rep)

    def del_user(self, dico):
        """ supprimer un utilisateur """
        if dico.has_key('user'):
            name = None
            removing=False
            params = dico['user']
            for param in params:
                nom = param['name']
                value = param['value']
                if param.has_key('checked'):
                    checked = param['checked']
                if nom == 'user_name':
                    name = value
                if nom == 'remove_data':
                    if checked=='true':
                        removing = True
                    else:
                        removing = False
            if name is not None:
                a = horus.del_user(name, removing)
                if a:
                    return  "L'utilisateur %s a bien été supprimé"%name, removing
        return "Problème de validation de formulaire", False


    def _get_valid_btn(self, server_nb):
        """ renvoit la description du bouton valider """
        validate = {}
        href = tools.make_form_link(server_nb, self.name, False, ['user'])
        ref = tools.make_confirm_link("Etes-vous sûr de vouloir supprimer l'utilisateur", href)
#        ref = "javascript:formConfirmExecute('message' , '%s')"%href
        validate['href'] =ref# "javascript:formValidForm('%s','%s',['user'])"%(server_nb, self.name)
        validate['icone'] = "/image/ok.gif"
        validate['libelle'] = "Valider"
        validate['title'] = "Supprimer cet utilisateur"
        return validate

