# -*- coding: UTF-8 -*-
###########################################################################
# Eole NG - 2007  
# Copyright Pole de Competence Eole  (Ministere Education - Academie Dijon)
# Licence CeCill  cf /root/LicenceEole.txt
# eole@ac-dijon.fr 
#  
# Action scribe_connexion
# Utilisateur connectés
# Machine connectée
# Gestion des quotas utilisateurs
# Gestion des acls des partages
# Gestion des virus
# Historique des connexions
#       
###########################################################################
"""Actions permettant de gérer les connexions"""
from twisted.python import log
from ead2.backend.lib.action import * # import des actions et des types
from ead2.backend.actions import tools
from ead2.lib.libead import uni
from ead2.lib.error import MissingKey, MissingValue
import commands
from scribe import backend
from ead2.backend.config.filenames import get_client_scribe_script
from ead2.backend.actions.scribe import scribe_tools
from re import escape
from creole import parsedico
eth0 = parsedico.parse_dico()['adresse_network_eth0']
# liste des templates complémentaires utilisés
used_templates = ['main', 'form', 'formselect', 'accordion']

def get_menu(server_nb, page=''):
    """ renvoie la description du menu 
        :page: page en cour d'affichage
    """
    index = {'title':'', 'icone':'/image/refresh.gif', 
                'href':tools.make_js_link(server_nb, "scribe_connexion_index"), 'libelle':"Connectés"}
    quota = {'title':'', 'icone':'/image/quota.gif',
                'href':tools.make_js_link(server_nb, "scribe_connexion_quota"), 'libelle':"Quota"}
    machine = {'title':'', 'icone':'/image/machines.gif',
            'href':tools.make_js_link(server_nb, "scribe_connexion_machine"), 'libelle':"Machines"}
    virus = {'title':'', 'icone':'/image/virus.gif',
            'href':tools.make_js_link(server_nb, "scribe_connexion_virus"), 'libelle':"Virus"}
    history = {'title':'', 'icone':'/image/hour.png',
            'href':tools.make_js_link(server_nb, "scribe_connexion_history"), 'libelle':"Historique"}
    if page == 'index':
        index['href'] = ''
    elif page == 'quota':
        quota['href'] = ''
    elif page == 'machine':
        machine['href'] = ''
    elif page == 'virus':
        virus['href'] = ''
    elif page == 'history':
        history['href'] = ''
    return dict(menus=(index, machine, quota, virus, history))

class Connected(Action):
    """ 
    gestion des utilisateurs connectés
    """
    user_description = Dict(default={}, doc="description de l'éxécutant", keys=['ip', 'name'])
    name = 'scribe_connexion_index'
    libelle = "Utilisateurs connectés"
    category = "Outils/Connexion/"
    description = 'Afficher les utilisateurs connectés'
    request = Dict(default={}, doc="arguments de la requete en cour cote frontend", keys=['server', 'action', 'user_file', 'sendmsg', 'sendmsgs'])
    form_result = Dict(default={}, doc="Retour de formulaire en Json", keys=['sendmsg', 'user_list'])
    
    def execute(self):
        """ renvoit les données pour l'affichage """
        tools.record_action(self.description, self.user_description)
        params, self.server_nb = tools.get_request(self.request)
        if params.has_key('user_file'):
            user_machine = params['user_file'][0]
            datas = self._get_user_files(user_machine)
            return self.frag(datas)
        #envoi de message à un utilisateur
        if params.has_key('sendmsg'):
            user_machine = params['sendmsg'][0]
            datas = self._get_sendmsg_form(user_machine)
            return self.frag(datas)
        #envoi de message à de multiples utilisateurs
        if params.has_key('sendmsgs'):
            datas = self._get_sendmsg_form()
            return self.frag(datas)
        if self.form_result != {}:
            if self.form_result.has_key('user_list'):
                try:
                    datas = self._valid_sendmsgs()
                except (MissingKey, MissingValue), mess:
                    datas = {'message':mess.message}
                self.form_result = {}
                return self.frag(datas)
            if self.form_result.has_key('sendmsg'):
                try:
                    datas = self._valid_sendmsg()
                except (MissingKey, MissingValue), mess:
                    datas = {'message':mess.message}
                self.form_result = {}
                return self.frag(datas)
            else:
                result.update(self._get_filtered_users())
            self.form_result = {}
        result = {'titre':"Gestion des connexions", 'sstitre':"Gestion des utilisateurs connectés"}
        result.update(get_menu(self.server_nb, page='index'))
        result.update(self._get_users())
        result.update(self._get_messages_btn())
        result['aucun'] = self._get_none_users_btn()
        result['tous'] = self._get_all_users_btn()
        d = {'template':'scribe_connexion_index', 'data':{'content':result, 'templates':used_templates}}
        return 0, uni(str(d))

    def frag(self, datas):
        result = datas
        d = {'template':'scribe_connexion_index', 'data':{'content':result, 'templates':used_templates}}
        return 0, uni(str(d))

    def _get_users(self):
        """ renvoie la liste des utilisateurs avec leur fichier en cour de consultation, nom de machine, ip """
        a = commands.getoutput("/usr/bin/smbstatus")
        users = tools.parse_smbstatus(a)
        for user in users:
            user['btn'] = self._get_message_btn(user['nom'], user['machine'])
            user['checkbox'] = {'name':user['machine'], 'libelle':'', 'value':user['machine']}
            user['show'] = self._get_show_btn(user['nom'], user['machine'])
            user['hide'] = self._get_hide_btn(user['nom'], user['machine'])
        return dict(users=users)
    
    def _get_sendmsg_form(self, user_machine=''):
        """ renvoie la description du formulaire d'envoi de message """
        close_btn = {'href':"javascript:setVoid('div_message_container')", 'title':"Fermer", 
                'libelle':'Fermer','_class':'btn', 'icone':"/image/suppr.gif"}
        if user_machine:
            sendmsg = {'name':'message', 'libelle':"Message à envoyer à %s"%user_machine, "rows":8, "cols":15}
            user = {'name':'user_machine', 'default_value':user_machine}
            validate=self._get_valid_sendmsg_btn()
            return dict(sendmsg=sendmsg, close_btn=close_btn,  user=user, validate=validate)
        else:
            sendmsg = {'name':'message', 'libelle':"Message à envoyer aux utilisateurs sélectionnés", "rows":8, "cols":15}
            validate=self._get_valid_sendmsg_btn(True)
            return dict(sendmsg=sendmsg, close_btn=close_btn,  validate=validate)

    def _valid_sendmsg(self):
        """ envoi un message à un utilisateur concerné """
        result = tools.format_form_result(self.form_result['sendmsg'])
        if not result.has_key('user_machine') or not result.has_key('message'):
            raise MissingKey, "Erreur : il manque des données pour l'envoi de message"
        machine = result['user_machine'].strip()
        message = result['message'].strip()
        return_msg = self.send_msg([machine], message)
        return dict(message=return_msg)
    
    def _valid_sendmsgs(self):
        """ envoi un message à un groupe d'utilisateur"""
        if not self.form_result.has_key('sendmsg'):
            raise MissingKey, "Erreur : Manque de données pour l'envoi groupé de message"
        result = tools.format_form_result(self.form_result['sendmsg'])
        user_list = tools.format_form_result(self.form_result['user_list'])
        if not result.has_key('message'):
            raise MissingKey, "Erreur : il manque des données pour l'envoi de message"
        machines = [machine.strip() for machine in user_list.keys()]
        message = result['message'].strip()
        return_msg = self.send_msg(machines, message)
        return dict(message=return_msg)

    def send_msg(self, machines, message):
        """envoi les messages"""
        if machines ==[]:
            raise MissingValue, "Erreur : aucune machine n'a été sélectionnée pour l'envoi de message."
        # on recoit le message en unicode utf-8
        try:
            message = message.encode('utf-8')
        except:
            raise MissingValue, "Erreur : le message n'a pas pu etre encode"
        for machine in machines:
            code, retour = commands.getstatusoutput("echo %s | smbclient -M %s"%(escape(message), escape(machine)))
            if code:
                raise MissingKey, "Erreur : une erreur s'est produite lors de l'envoi d'un message à %s"%escape(machine)
        return "L'envoi du message aux utilisateurs s'est bien passée."

    def _get_valid_sendmsg_btn(self, multi=False):
        """ renvoie la description du bouton valider pour l'envoi de message 
        :multi: si True, on valide la liste des utilisateurs sélectionnés
        """
        if multi:
            href=tools.make_form_link(self.server_nb, self.name, True, ['sendmsg', 'user_list'],"div_message_container")
        else:
            href=tools.make_form_link(self.server_nb, self.name, True, ['sendmsg'],"div_message_container")
        valid = {'href':href, 
                    'title':"Envoyer le message", 'libelle':'Valider','_class':'btn', 
                    'icone':"/image/ok.gif"}
        return valid

    def _get_message_btn(self, user, machine):
        """ renvoie la description du bouton d'envoi de message à l'utilisateur user """
        href = tools.make_js_link(self.server_nb, self.name, balise='div_message_container', sendmsg=machine)
        title = "Envoi de message à %s"%user
        libelle = ""
        icone = "/image/mail_mini.png"
        return dict(href=href, title=title, libelle=libelle, icone=icone, _class='btn')

    def _get_messages_btn(self):
        """ renvoie la description du bouton d'envoi groupé de message """
        href = tools.make_js_link(self.server_nb, self.name, balise='div_message_container', sendmsgs="ok")
        title = "Envoi groupé de message"
        libelle = "Envoi groupé"
        icone = "/image/mail.gif"
        return dict(group_sending=dict(href=href, title=title, libelle=libelle, icone=icone, _class='btn'))
    
    def _get_show_btn(self, user, machine):
        """ renvoie la description du bouton d'affichage des fichiers en cour d'utilisation """
        href = tools.make_js_link(self.server_nb, self.name, balise='div_file_%s_container'%machine, user_file=machine)
        title = "Afficher les fichiers de à %s"%user
        libelle = "Afficher"
        icone = ""
        return dict(href=href, title=title, libelle=libelle, icone=icone, _class='btn')
    
    def _get_hide_btn(self, user, machine):
        """ renvoie la description du bouton de masquage des fichiers en cour d'utilisation """
        href = "javascript:setVoid('div_file_%s_container');"%machine
        title = "Masquer les fichiers de à %s"%user
        libelle = "Masquer"
        icone = ""
        return dict(href=href, title=title, libelle=libelle, icone=icone, _class='btn')
    
    def _get_user_files(self, usermachine):
        users = self._get_users()['users']
        for user in users:
            if user['machine'] == usermachine:
                if user['files'] is not None:
                    return dict(user_files=[fichier['nom'] for fichier in user['files']])
        return dict(user_files=[])
    
    def _get_none_users_btn(self):
        """ renvoie la description du bouton de déselection des utilisateurs de la liste """
        icone = ''
        title = 'Aucun utilisateur'
        libelle = 'Aucun'
        href = "javascript:formSelectNone('user_list')"
        return dict(icone=icone, title=title, libelle=libelle,
                href=href)

    def _get_all_users_btn(self):
        """ renvoie la description du bouton de selection de
        tous les utilisateurs de la liste """
        icone = ''
        title = 'Tous les utilisateurs'
        libelle = 'Tous'
        href = "javascript:formSelectAll('user_list')"
        return dict(icone=icone, title=title, libelle=libelle, href=href)

class ConnectedMachine(Action):
    """ 
    affichage des stations du réseau
    """
    user_description = Dict(default={}, doc="description de l'éxécutant", keys=['ip', 'name'])
    name = 'scribe_connexion_machine'
    libelle = "Machines"
    category = "Outils/Connexion/"
    description = 'Stations connectées au réseau'
    request = Dict(default={}, doc="arguments de la requete en cour cote frontend", keys=['server', 'action', 'machine_type' ])
    form_result = Dict(default={}, doc="Retour de formulaire en Json", keys=['user_list', 'actions'])
    
    def execute(self):
        """ renvoit les données pour l'affichage """
        params, self.server_nb = tools.get_request(self.request)
        if params.has_key('machine_type'):
            machine_type = params['machine_type'][0]
            try:
                datas = self._get_machines(machine_type)
            except (MissingKey, MissingValue), mess:
                datas = {'message':mess.message}
            return self.frag(datas)
        if self.form_result != {}:
            if self.form_result.has_key('actions'):
                try:
                    datas = self._valid_action()
                except (MissingKey, MissingValue), mess:
                    datas = {'message':mess.message}
                self.form_result = {}
                return self.frag(datas)
        result = {'titre':"Gestion des connexions", 'sstitre':"%s %s"%(self.description, eth0)}
        result.update(get_menu(self.server_nb, page='machine'))
        result.update(self._get_machines())
        result.update(self._get_controleur_btn())
        result.update(self._get_maitre_btn())
        result.update(self._get_all_btn())
        result.update(self._get_client_btn())
        d = {'template':'scribe_connexion_index', 'data':{'content':result, 'templates':used_templates}}
        return 0, uni(str(d))

    def frag(self, datas):
        result = datas
        d = {'template':'scribe_connexion_index', 'data':{'content':result, 'templates':used_templates}}
        return 0, uni(str(d))

    def _get_machines(self, _type=None):
        if _type == 'client':
            return self._get_client_scribe()
        elif _type in [None, 'maitre', 'controleur', '']:
            return dict(machines=tools.list_valid_workstations(_type))
        else:
            raise MissingKey, "Erreur : Problème de validation."

    def _get_client_scribe(self):
        """ renvoie la liste des clients scribe connectés au domaine """
        a,b = commands.getstatusoutput("%s -l -v"%get_client_scribe_script)
        if a : raise MissingKey, "Erreur : impossible de récupérer les clients du domaine."
        else:
            if b == '': return dict(clients=[]) # un strip sur une chaine vide ne renvoie pas une liste vide
            clients = [cl.split(',') for cl in b.split(';')]
            client_list = [{'netbiosname':a[1].strip(), 'nom':a[2].strip(), 'ip':a[0].strip(), 'checkbox':{'name':a[0].strip(), 'libelle':'', 'value':a[0].strip()}}for a in clients]
            return dict(clients=client_list, actions_radio=self._get_action_form(), 
                            actions_validate=self._get_valid_actions_btn())

    def _get_action_form(self):
        """ renvoie la description des radios pour les actions sur les postes connectés """
        stop = {'name':'action', 'value':'stop', 'libelle':"Eteindre", 'inline':'ok', 'checked':'ok'}
        restart = {'name':'action', 'value':'restart', 'libelle':'Redémarrer', 'inline':'ok'}
        session_close = {'name':'action', 'value':'close_session', 'libelle':'Fermer la session', 'inline':'ok'}
        return [stop, restart, session_close]

    def _valid_action(self):
        """ valide l'éxécution d'action sur les clients scribe """
        if not self.form_result.has_key('user_list'):
            raise MissingKey, "Erreur : il manque des utilisateurs pour l'éxécution d'action."
        result = tools.format_form_result(self.form_result['actions'])
        if not result.has_key('action'):
            raise MissingKey, "Erreur : il manque des valeurs pour l'éxécution d'action."
        user_list = tools.format_form_result(self.form_result['user_list'])
        action = result['action']
        users = [user for user in user_list.keys()]
        print action, users
        if action == 'stop':return self._stop_client(users)
        elif action =='restart':return self._restart_client(users)
        elif action == 'close_session':return self._close_session(users)
        return dict(message="Les stations %s ont bien été redémarrées")

    def _stop_client(self, users):
        if users == []: raise MissingValue, "Erreur : aucun utilisateurs n'a été sélectionné."
        for user in users:
            a,b = commands.getstatusoutput("%s -s %s -t 0"%(get_client_scribe_script, user))
            if a:raise MissingKey, "Erreur : un problème s'est produit à l'arrêt du client %s"%user
        return dict(message = "Les stations %s ont bien été arrétées."%user)

    def _restart_client(self, users):
        if users == []: raise MissingValue, "Erreur : aucun utilisateurs n'a été sélectionné."
        for user in users:
            a,b = commands.getstatusoutput("%s -s %s -t 1"%(get_client_scribe_script, user))
            if a:raise MissingKey, "Erreur : un problème s'est produit au redémarrage du client %s"%user
        return dict(message = "Les stations %s ont bien été redémarrées."%user)

    def _close_session(self, users):
        if users == []: raise MissingValue, "Erreur : aucun utilisateurs n'a été sélectionné."
        for user in users:
            a,b = commands.getstatusoutput("%s -s %s -t 2"%(get_client_scribe_script, user))
            if a:raise MissingKey, "Erreur : un problème s'est produit à la fermeture de session du client %s"%user
        return dict(message = "Les session sur les stations %s ont bien été fermées."%user)

    def _get_valid_actions_btn(self):
        """ renvoie la description du bouton de validation de l'execution d'action sur les postes clients """
        href = tools.make_form_link(self.server_nb, self.name, True, ['actions', 'user_list'], 'div_message')
        title = "Exécuter une action sur le client"
        libelle = "Exécuter"
        icone = "/image/ok.gif"
        return dict(href=href, title=title, libelle=libelle, icone=icone)

    def _get_controleur_btn(self):
        """ renvoie la description des boutons d'affichage des types de stations """
        href = tools.make_js_link(self.server_nb, self.name, balise='div_station_container', machine_type="controleur")
        title = "Afficher les controleur de domaine du réseau"
        libelle = "Controleur de domaine"
        icone = ""
        return dict(controleur_btn=dict(href=href, title=title, libelle=libelle, icone=icone))

    def _get_maitre_btn(self):
        href = tools.make_js_link(self.server_nb, self.name, balise='div_station_container', machine_type="maitre")
        title = "Afficher les maîtres explorateurs du réseau"
        libelle = "Maîtres explorateurs"
        icone = ""
        return dict(maitre_btn=dict(href=href, title=title, libelle=libelle, icone=icone))

    def _get_all_btn(self):
        href = tools.make_js_link(self.server_nb, self.name, balise='div_station_container', machine_type="")
        title = "Afficher toutes les stations"
        libelle = "Toutes les stations"
        icone = ""
        return dict(all_btn=dict(href=href, title=title, libelle=libelle, icone=icone))

    def _get_client_btn(self):
        href = tools.make_js_link(self.server_nb, self.name, balise='div_station_container', machine_type='client')
        title = "Afficher les clients du domaine"
        libelle = "Clients du domaine"
        icone = ""
        return dict(client_btn=dict(href=href, title=title, libelle=libelle, icone=icone))

class ConnectedQuotas(Action):
    """ 
    affichage des quotas des utilisateurs du réseau
    """
    user_description = Dict(default={}, doc="description de l'éxécutant", keys=['ip', 'name'])
    name = 'scribe_connexion_quota'
    libelle = "Quotas"
    category = "Outils/Connexion/"
    description = 'Affichage des quotas utilisateurs'
    request = Dict(default={}, doc="arguments de la requete en cour cote frontend", keys=['server', 'action', 'quota'])
    form_result = Dict(default={}, doc="Retour de formulaire en Json", keys=[])
    
    def execute(self):
        """ renvoit les données pour l'affichage """
        params, self.server_nb = tools.get_request(self.request)
        result = {'titre':"Gestion des connexions", 'sstitre':self.description}
        if params.has_key('quota'):
            try:
                datas = self._valid_quota(params)
            except (MissingValue, MissingKey, Exception), mess:
                datas = {'message':mess.message}
            self.form_result = {}
            return self.frag(datas)
        result.update(self._get_form())
        result.update(get_menu(self.server_nb, page='quota'))
        d = {'template':'scribe_connexion_quota', 'data':{'content':result, 'templates':used_templates}}
        return 0, uni(str(d))

    def frag(self, datas):
        """ renvoie un fragment de page"""
        result = datas
        d = {'template':'scribe_connexion_quota', 'data':{'content':result, 'templates':used_templates}}
        return 0, uni(str(d))
    
    def _get_form(self):
        """ renvoie les données pour la mise en page du formulaire """
        quota_types = [{'libelle':"dépassement de quota", 'name':'depasse'},
                        {'libelle':'quota bien utilisé', 'name':'proche'},
                        {'libelle':'tous', 'name':'tous'}, 
                        {'name':'aucun', 'libelle':'','default':'ok' }]
        quota_select = {'name':'quota', 'libelle':'Afficher les quotas selon le filtre: ' , 
                'options':[{'name':'', 'options':quota_types}], 'inline':'ok',
                'onchange':tools.make_js_link(self.server_nb, self.name, code=True, balise="quota_table", quota='this.options[this.selectedIndex].value')
                }
        return dict(quota_select=quota_select)

    def _valid_quota(self, params):
        """ valide le formulaire pour l'affichage des quotas """
        quota = params['quota'][0]
        quotas = scribe_tools._get_quotas(quota)
        print quotas, "ON a les quotas %s"%quota
        return dict(quotas = quotas, quota_entete1="Utilisateur", quota_entete2="Espace utilisé", 
                    quota_entete3="Délai éventuel")

class ConnectedVirus(Action):
    """ 
    affichage des virus mise en quarantaine
    """
    user_description = Dict(default={}, doc="description de l'éxécutant", keys=['ip', 'name'])
    name = 'scribe_connexion_virus'
    libelle = "Virus"
    category = "Outils/Connexion/"
    description = 'Virus détectés'
    request = Dict(default={}, doc="arguments de la requete en cour cote frontend", keys=['server', 'action', 'quota'])
    form_result = Dict(default={}, doc="Retour de formulaire en Json", keys=[])
    
    def execute(self):
        """ renvoit les données pour l'affichage """
        params, self.server_nb = tools.get_request(self.request)
        result = {'titre':"Gestion des connexions", 'sstitre':self.description}
        result.update(self._get_virus())
        result.update(get_menu(self.server_nb, page='virus'))
        d = {'template':'scribe_connexion_quota', 'data':{'content':result, 'templates':used_templates}}
        return 0, uni(str(d))

    def _get_virus(self):
        """ renvoie les données pour la mise en page du formulaire """
        return dict(virus=scribe_tools._get_viruses())

class ConnectedHistory(Action):
    """ 
    affichage de l'historique de connexion
    """
    user_description = Dict(default={}, doc="description de l'éxécutant", keys=['ip', 'name'])
    name = 'scribe_connexion_history'
    libelle = "Historique"
    category = "Outils/Connexion/"
    description = 'Historique des connexions'
    request = Dict(default={}, doc="arguments de la requete en cour cote frontend", keys=['server', 'action', 'quota'])
    form_result = Dict(default={}, doc="Retour de formulaire en Json", keys=[])
    
    def execute(self):
        """ renvoit les données pour l'affichage """
        params, self.server_nb = tools.get_request(self.request)
        result = {'titre':"Gestion des connexions", 'sstitre':self.description}
        result.update(self._get_history())
        result.update(get_menu(self.server_nb, page='history'))
        d = {'template':'scribe_connexion_quota', 'data':{'content':result, 'templates':used_templates}}
        return 0, uni(str(d))

    def _get_history(self):
        """ renvoie les données pour la mise en page du formulaire """
        return dict(history=scribe_tools._get_history(), hist_entete1="Date", 
                    hist_entete2="Utilisateurs", hist_entete3="Ordinateurs", hist_entete4="OS")
