# -*- coding: UTF-8 -*-
###########################################################################
# Eole NG - 2007  
# Copyright Pole de Competence Eole  (Ministere Education - Academie Dijon)
# Licence CeCill  cf /root/LicenceEole.txt
# eole@ac-dijon.fr 
#  
# Action scribe_devoir
# outil de gestion des devoirs
#       
###########################################################################
from twisted.python import log
from ead2.backend.lib.action import * # import des actions et des types
from ead2.backend.actions import tools # import get_request, make_js_link, make_form_link, format_form_result
from ead2.lib.error import MissingKey, MissingValue
from ead2.lib.libead import uni
from ead2.backend.actions.scribe import scribe_tools
from scribe import backend
from ead2.config.config import ip_locale
from ead2.backend.scribe_devoir import gest_devoir, pbgest
from urllib import unquote
from ead2.backend.config.config import TEMP_DIR
import tempfile, os
from os import listdir
from os.path import isdir, join, isfile
from shutil import rmtree

used_templates = ['main', 'formselect', 'form']
filedir = join(TEMP_DIR, "tempfiledir")
datadir = join(TEMP_DIR, "tempdatadir")


class Devoir(Action):
    """ outils de gestion des devoirs
        envoi de devoir

    """
    user_description = Dict(default={}, doc="description de l'éxécutant", keys=['ip', 'name'])
    name = 'scribe_devoir'
    libelle = "Devoirs"
    category = "Outils/Devoirs"
    description = "Gestion des devoirs"
    request = Dict(default={}, doc="arguments de la requete en cour cote frontend", 
                                keys=['server', 'action', 'fichier', 'filename', 'fichier_datas', 'nomdefichier'])
    form_result = Dict(default={}, doc="Retour de formulaire en Json", keys=['group'])
    filename = Text(default='', doc="Nom du fichier devoir temporaire")
    datafile = Text(default='', doc="Nom du fichier de donnees")

    def execute(self):
        """ renvoie les données pour l'affichage """
        self.params, self.server_nb = tools.get_request(self.request)
        self.user_login = self.user_description['name']['uid'][0]
        pbroot = pbgest.PBgest()
        pbroot.d.addCallback(self._set_pbroot)
        pbroot.d.addCallback(self._write_files)
        pbroot.d.addCallback(self._valid_form)
        pbroot.d.addCallback(self.main)
        pbroot.d.addErrback(self.err)
#        pbroot.d.addCallback(self.erreur)
        return pbroot.d

    def err(self, mess):
        self.form_result = {}
        if isdir(filedir):
            print "ENVIRONNEMENT"
            print os.system('whoami &>/tmp/toto')
            rmtree(filedir)
        if isdir(datadir):
            rmtree(datadir)
        return self.frag(dict(message=mess.getErrorMessage()))

    def _set_pbroot(self, pbroot):
        """ initialise du gestionnaire de devoir """
        self.gest_devoir = gest_devoir.GestDevoir(pbroot, self.user_login)
        return ""

    def _write_files(self, toto):
        """ appel l'écriture d'éventuels fichiers pour les devoirs """
        if self.params.has_key('fichier'):
            ## initialisation du rep de devoir
            if not isdir(filedir):
                os.mkdir(filedir)
            #fichier du devoir
            file_datas = unquote(self.params['fichier'][0]).strip()
            if file_datas != '':
                file_name = self.params['nomdefichier'][0].split('/')[-1]
                file_name = join(filedir, file_name) #self.params['nomdefichier'][0])
                self.filename = self.write_tempfile(file_datas, file_name)
                print "On a écrit dans le fichier ", self.filename
        if self.params.has_key('fichier_datas'):
            ## initialisation du répertoire de données pour les devoirs
            if not isdir(datadir):
                os.mkdir(datadir)
            # fichiers de donnees pour les devoirs
            file_datas = unquote(self.params['fichier_datas'][0]).strip()
            if file_datas != '':
                file_name = self.params['nomdefichier'][0].split('/')[-1]
                file_name = join(datadir, file_name) #self.params['nomdefichier'][0])
                self.datafile = self.write_tempfile(file_datas, file_name)
                print "On a écrit les données dans le fichier ", self.datafile
        return {'fichier':self.filename,'donnees':self.datafile}

    def main(self, message):
        self.form_result = {}
#        user_type = self.user_description['name']['role']
#        tempfilename, tempdataname = '', ''
#        if params.has_key('fichier'):
#            file_datas = unquote(params['fichier'][0])
#            tempfilename = write_tempfile(file_datas)
#            print "On a écrit dans le fichier ", tempfilename
#        if params.has_key('fichier_datas'):
#            file_datas = unquote(params['fichier_datas'][0])
#            tempdataname = write_tempfile(file_datas)
#            print "On a écrit les données dans le fichier ", tempdataname
#        if self.form_result != {}:
#            try:
#                self.gest_devoir = gest_devoir.GestDevoir(self, user_login)
#                datas = self._valid_form(tempfilename, tempdataname)
#            except (Exception, MissingKey, MissingValue), mess:
#                datas = {'message':mess.message}
#            return self.frag(datas)
        result = {'titre':self.description}
        if type(message) == dict:
            result.update(message)
        result.update(self._get_form())
        result.update(self._get_file_download_link())
        result.update(self._get_data_file_download_link())
        result.update(self._get_distribute_btn())
        result.update(self._get_collect_btn())
        result.update(self._get_giveback_btn())
        result.update(self._get_delete_btn())
        d = {'template':'scribe_devoir', 'data':{'content':result, 'templates':used_templates}}
        return 0, uni(str(d))

    def _valid_form(self, toto):
        """ valide les retours d'action """
        if self.form_result.has_key('group'):
            return self._distribute()
        elif self.form_result.has_key('collect'):
            return self._collect()
        elif self.form_result.has_key('giveback'):
            return self._giveback()
        elif self.form_result.has_key('delete'):
            return self._delete()
        return ''

    def frag(self, datas):
        """ renvoie des données pour un fragment de page """
        result = datas
        d = {'template':'scribe_devoir', 'data':{'content':result, 'templates':used_templates}}
        return 0, uni(str(d))

    def _get_form(self):
        """ renvoie les données de mise en forme du formulaire de gestion de devoir """
        gr_types = ['Groupe', 'Matiere', "Responsables" , "Equipe", "Classe"]
        groups = [{'name':key, 'options':[{'name':gr, 'libelle':gr}for gr in backend.list_groups(key)]}for key in gr_types]
        group_select = {'name':'group', 'libelle':'Choix du groupe', 'options':groups}
        collect_options = [{'name':nom, 'libelle':nom}for nom in self.gest_devoir.get_dist_devoirs()]
        collect = {'name':'collect', 'libelle':"Ramasser un devoir", 
                    'options':[{'name':'Devoirs en attente de ramassage', 'options':collect_options}]}
        giveback_options = [{'name':nom, 'libelle':nom}for nom in self.gest_devoir.get_ram_devoirs()]
        giveback = {'name':'giveback', 'libelle':"Rendre un devoir", 
                    'options':[{'name':'Devoirs ramassés', 'options':giveback_options}]}
        delete_options = [{'name':nom, 'libelle':nom}for nom in self.gest_devoir.get_ren_devoirs()]
        delete = {'name':'delete', 'libelle':"Supprimer les données relatives à un devoir",
                    'options':[{'name':'Devoirs rendus', 'options':delete_options}]}
        return dict(group_select = group_select, collect=collect, giveback=giveback, delete=delete)

    def write_tempfile(self,datas, tempfilename=''):
        """ écrit les données du devoir dans un fichier temporaire """
        if not tempfilename:
            while True:
                tempfilename = tempfile.mktemp()
                if not os.path.isfile(tempfilename) and not os.path.isdir(tempfilename):
                    break
        if datas.strip() != '':
            if isfile(tempfilename):raise Exception, "Erreur : ce fichier a déjà été chargé"
            log.msg("ECRITURE DE FICHIER %s"%tempfilename)
            fil = file(tempfilename, 'w')
            fil.write(datas)
            fil.close()
            return tempfilename
        return ""

    def _distribute(self):
        """ distribue des devoirs """
        result = tools.format_form_result(self.form_result['group'])
        if not result.has_key('group'):
            raise Missingkey, "Erreur : il manque un groupe pour la distribution des devoirs."
        devoir = self._get_downloaded_filelist()
        datas = self._get_downloaded_datalist()
        group = result['group']
        print "distribution de %s avec %s pour le groupe %s"%(devoir,datas,group)
        a = self.gest_devoir.distribute(devoir, datas, group)
        a.addCallback(self.reussite)
        a.addErrback(self.err)

    def _get_downloaded_filelist(self):
        if isdir(filedir):
            files = listdir(filedir)
            if files != []:
                return [join(filedir, file) for file in files]
        raise Exception, "Erreur: aucun devoir n'a été téléchargé."
    
    def _get_downloaded_datalist(self):
        if isdir(datadir):
            files = listdir(datadir)
            return [join(datadir, file) for file in files]
        return []

    def reussite(self, toto):
        print "self.reussite REUSSITE", toto
        return dict(message = "Le devoir a bien été distribué.")

    def erreur(self, toto):
        print "self.erreur ERREUR", toto
        return dict(message = "Erreur : échec de la distribution du devoir")

    def _get_file_download_link(self):
        """ renvoie le lien vers le formulaire de téléchargement de fichier """
        return dict(devoir_file="./file_download/?server=%s&filename=fichier&action=%s"%(self.server_nb, self.name))
    
    def _get_data_file_download_link(self):
        """ renvoie le lien vers le formulaire de téléchargement de fichier de données du devoir"""
        return dict(data_file="./file_download/?server=%s&filename=fichier_datas&action=%s"%(self.server_nb, self.name))

    def _get_distribute_btn(self):
        """ renvoie les données pour la mise en forme du bouton distribuer """
        _href = tools.make_form_link(self.server_nb, self.name, True, ['group'])
        href = "javascript:document.forms['devoir'].submit(); document.forms['donnees'].submit(); %s"%_href
        title=''
        libelle="Distribuer"
        icone="/image/ok.gif"
        return dict(distribute_btn={'href':_href,'title':title,'libelle':libelle,'icone':icone})
    
    def _get_collect_btn(self):
        """ renvoie les données pour la mise en forme du bouton de rammassage """
        href = tools.make_form_link(self.server_nb, self.name, True, ['collect'])
        title=''
        libelle="Ramassage des devoirs"
        icone="/image/ok.gif"
        return dict(collect_btn={'href':href,'title':title,'libelle':libelle,'icone':icone})
    
    def _get_giveback_btn(self):
        """ renvoie les données pour la mise en forme du bouton de rendu """
        href = tools.make_form_link(self.server_nb, self.name, True, ['giveback'])
        title=''
        libelle="Rendre les devoirs"
        icone="/image/ok.gif"
        return dict(giveback_btn={'href':href,'title':title,'libelle':libelle,'icone':icone})

    def _get_delete_btn(self):
        """ renvoie les données pour la mise en forme du bouton de rendu """
        href = tools.make_form_link(self.server_nb, self.name, True, ['delete'])
        title='Supprimer les données relatives au devoir'
        libelle="Supprimer les données"
        icone="/image/ok.gif"
        return dict(delete_btn={'href':href,'title':title,'libelle':libelle,'icone':icone})
