# -*- coding: UTF-8 -*-
###########################################################################
# Eole NG - 2007  
# Copyright Pole de Competence Eole  (Ministere Education - Academie Dijon)
# Licence CeCill  cf /root/LicenceEole.txt
# eole@ac-dijon.fr 
#  
# Action scribe_group*
# Creation de groupe
# Recherche puis modification de groupe
#       
###########################################################################
"""Actions permettant de gérer les groupes """
from twisted.python import log
from ead2.backend.lib.action import * # import des actions et des types
from ead2.backend.actions.tools import get_request, make_js_link, make_form_link, make_confirm_link, format_form_result, make_key_form_link
from ead2.lib.error import MissingKey, MissingValue
from os.path import join
from scribe import backend
from ead2.lib.libead import uni
from ead2.backend.actions.scribe import scribe_tools
import sys
# liste des templates complémentaires utilisés
used_templates = ['main', 'form', 'formselect', 'accordion', 'listlist', 'checklist']

# dico util pour la sérialisation
# pour les messages
messdict = {'niveau':'du niveau', 'groupe':'du groupe', 'matiere':'de la matière'}
# pour les appels au backend.py
typedict = {'niveau':'Niveau', 'matiere':'Matiere', 'groupe':'Groupe', 'classe':'Classe',
        'responsable de classe':"Responsables", "équipe pédagogique":"Equipe"}
# description des éléments à afficher pour la modification d'un groupe
listingdict = {'Niveau':['Nom', 'Liste de diffusion', 'Partages'],
                'Classe':['Nom', 'Liste de diffusion', 'Partages'],
                'Matiere':['Nom', 'Liste de diffusion', 'Partages', 'Suppression'],
                "Equipe":['Nom', 'Liste de diffusion', 'Partages', 'Suppression'],
                'Groupe':['Nom', 'Liste de diffusion', 'Partages', 'Suppression'],
                "Responsables":["Classe", "Responsable"]
                }

def _get_menu(server_nb, _type=None):
    """ renvoie la description du menu pour la creation de groupes """
    create_class = {'icone':'/image/scribe/user_create.gif', 'libelle':"Classe",
               'title':"Créer une classe", 'href':make_js_link(server_nb, 'scribe_group_create', _type='classe')}
    create_niveau = {'icone':'/image/scribe/user_create.gif', 'libelle':"Niveau",
            'title':"Créer un groupe niveau", 'href':make_js_link(server_nb, 'scribe_group_create', _type='niveau')}
    create_matiere = {'icone':'/image/scribe/user_create.gif', 'libelle':"Matière",
            'title':"Créer un groupe matière", 'href':make_js_link(server_nb, 'scribe_group_create', _type='matiere')}
    create_groupe = {'icone':'/image/scribe/user_create.gif', 'libelle':"Groupe",
            'title':"Créer un groupe", 'href':make_js_link(server_nb, 'scribe_group_create', _type='groupe')}
    liste_group = {'icone':'/image/scribe/user_create.gif', 'libelle':"Lister des groupes",
            'title':"Lister des groupes", "href":make_js_link(server_nb, 'scribe_group_list')}
    if _type == 'classe':
        create_class['href'] = ''
    elif _type == 'niveau':
        create_niveau['href'] = ''
    elif _type == 'matiere':
        create_matiere['href'] = ''
    elif _type == 'groupe':
        create_groupe['href'] = ''
    else:
        liste_group['href'] = ''
    return dict(menus=[create_groupe, create_class, create_niveau, create_matiere, liste_group])

class GroupCreate(Action):
    """ creation de groupe 
        :_type: type de groupe à créer, classe / niveau / matière / groupe
    """
    user_description = Dict(default={}, doc="description de l'éxécutant", keys=['ip', 'name'])
    name = 'scribe_group_create'
    libelle = "Création de groupe"
    category = "Gestion/Groupes/"
    description = 'Créer un groupe'
    request = Dict(default={}, doc="arguments de la requete en cour cote frontend", keys=['server', 'action', '_type'])
    form_result = Dict(default={}, doc="Retour de formulaire en Json", keys=['current_type', 'add_group'])
    
    def execute(self):
        """ renvoit les données pour l'affichage """
        params, self.server_nb = get_request(self.request)
        result = {'titre':"Gestion des groupes"}
        self._type = "groupe"
        if params.has_key('_type'):self._type = params['_type'][0].strip()
        if self.form_result != {}:
            if self.form_result.has_key('current_type'):
                self._type = format_form_result(self.form_result['current_type'])['_type']
                self.form_result.pop('current_type')
            try:
                result['message'] = self._valid_form()
            except (MissingValue, MissingKey), mess:
                result['message'] = mess.message
            except:
                result['message'] = "Une erreur inconnue s'est produite.\\n Vérifier l'absence d'accent dans le nom de groupe."
            self.form_result = {}
        typedico = {'classe':'une classe', 'niveau':'un niveau', 'groupe':'un groupe', 'matiere':"une matière"}
        result['sstitre'] = "Créer %s"%typedico[self._type]
        result.update(_get_menu(self.server_nb, self._type))
        result.update(self._get_form())
        result.update(self._get_valid_btn())
        result['_type'] = {'name':'_type', 'default_value':self._type } 
        d = {'template':'scribe_group', 'data':{'content':result, 'templates':used_templates}}
        return 0, uni(str(d))

    def _get_form(self):
        """ renvoie la description du formulaire pour la creation de groupe """
        if self._type == 'classe':return self._classe_form()
        else:return self._groupe_form()

    def _classe_form(self):
        """ Dans le cas de la création d'une classe
        @group_name : nom de la classe (input)
        @level : niveau de la classe (select)
        @liste_status : statut de la liste de diffusion (restreint, internet)
        """
        group_name = {'name':'group_name', 'libelle':'Intitulé de la classe', 'inline':'ok','onblur':"testVoid('group_name', 'error_group_name')"}
        niveaux = backend.list_groups('niveau')
        if niveaux == []:
            return dict(redirect={})
        level = {'name':'level', 'libelle':'Niveau associé', 'inline':'ok', 
                'options':[{'name':'', 'options':[{'name':nivo, 'libelle':nivo}for nivo in niveaux]}]}
        liste_status = {'name':'liste_status', 'libelle':"Liste de diffusion",
                    'options':[{'name':'', 'options':[{'name':'secondaire', 'libelle':"domaine restreint (conseillé)"},
                                                        {'name':'principal', 'libelle':"domaine internet"}]}],
                    'inline':'ok'}
        return dict(group_name=group_name, level=level, liste_status=liste_status)

    def _groupe_form(self):
        """ renvoie la description du formulaire pour la création d'un groupe, une matière ou un niveau
        @group_name : nom du groupe (input)
        @share_check : avec partage ? (checkbox)
        @share_status : statut du partage
        @liste_check : avec liste de diffusion ? (checkbox)
        @liste_status : statut de la liste de diffusion (internet ou restreint)
        """
        group_name = {'name':'group_name', 'libelle':'Intitulé %s'%messdict[self._type], 
                        'inline':'ok','onblur':"testVoid('group_name', 'error_group_name')"}
        share_check = {'name':'share_check', 'libelle':"Avec Partage", 'inline':'ok', 'value':'share_check'}
        share_status = {'name':'share_status', 'libelle':"Partage", 
                    'options':[{'name':'', 'options':[{'name':'rw', 'libelle':'lecture/écriture'}, 
                        {'name':'ro', 'libelle':'lecture seule'},
                        {'name':'dt', 'libelle':'modèle données/travail'}
                        ]}], 'inline':'ok'
                    }
        liste_check = {'name':'liste_check', 'libelle':"Avec liste de diffusion", 'inline':'ok', 'value':'liste_check'}
        liste_status = {'name':'liste_status', 'libelle':"Liste de diffusion",
                    'options':[{'name':'', 'options':[{'name':'restreint', 'libelle':"domaine restreint (conseillé)"},
                                                        {'name':'internet', 'libelle':"domaine internet"}]}],
                    'inline':'ok'}
        return dict(group_name=group_name, share_check=share_check, share_status=share_status, 
                    liste_check=liste_check, liste_status=liste_status)

    def _valid_form(self):
        if self._type == 'classe':return self._valid_classe()
        else:return self._valid_groupe()

    def _valid_classe(self):
        """ ajoute une classe dans l'annuaire """
        if not self.form_result.has_key('add_group'): 
            raise MissingKey, "Le résultat de formulaire est vide"
        creation = format_form_result(self.form_result['add_group'])
        for i in ['group_name', 'level', 'liste_status']:
            if i not in creation.keys():
                raise MissingKey, "Erreur : Des données manque pour la création de la classe"
        name, level, liste_status = creation['group_name'], creation['level'], creation['liste_status']
        if not self._test_name(name):
            raise MissingValue, "Erreur : Le nom comporte des accents ou que des chiffres"
        for i in name, level, liste_status:
            if not i.strip():raise MissingValue, "Il manque des valeurs"
        a = backend.ajout_classe(name, level, liste_status)
        if a:
            return "Réussite de l'ajout de la classe %s"%name
        else:
            return "Echec à la création de la classe %s"%name

    def _valid_groupe(self):
        """ ajoute un niveau dans l'annuaire"""
        if not self.form_result.has_key('add_group'): 
            raise MissingKey, "Le résultat de formulaire est vide"
        creation = format_form_result(self.form_result['add_group'])
        for i in ['group_name', 'share_status', 'liste_status']:
            if i not in creation.keys():
                raise MissingKey, "Erreur : Des données manque pour la création %s"%messdict[self._type]
        name = creation['group_name']
        if not self._test_name(name):
            raise MissingValue, "Erreur : Le nom n'est pas au bon format"
        if creation.has_key('share_check'): share_status = creation['share_status']
        else: share_status = ''
        if creation.has_key('liste_check'): liste_check = creation['liste_check']
        else: liste_check = ''
        a = backend.ajout_groupe(name, typedict[self._type], domaine=liste_check, optrw=share_status)
        if a:
            return "Réussite de l'ajout %s %s"%( messdict[self._type], name)
        else:
            return "Echec à la création %s %s"%(messdict[self._type], name)
               
    def _get_valid_btn(self):
        """ bouton valider pour la creation d'user passe en meme temps le type en cour (classe, nivo...)"""
        validate={}
        validate['href'] = make_form_link(self.server_nb, self.name, True, ['current_type', 'add_group'])
        validate['icone'] = "/image/ok.gif"
        validate['libelle'] = "Valider"
        validate['title'] = "Créer ce groupe"
        return dict(validate=validate)
    
    def _test_name(self, group_name):
        """ teste la validité d'un nom de groupe : absence d'espace et pas que numérique """
        if group_name != group_name.strip() or ' ' in group_name:
            return False
        nums = [str(num) for num in range(9)]
        test = [c in nums for c in group_name]
        if False not in test:
            return False
        else:
            return True

class GroupListing(Action):
    """ Liste les groupes 
        Permet le listing des groupes par type et par partie du nom.
    """
    user_description = Dict(default={}, doc="description de l'éxécutant", keys=['ip', 'name'])
    name = 'scribe_group_list'
    libelle = "Recherche de groupe"
    category = "Gestion/Groupes/"
    description = 'Rechercher un groupe'
    request = Dict(default={}, doc="arguments de la requete en cour cote frontend", keys=['server', 'action', 'group'])
    form_result = Dict(default={}, doc="Retour de formulaire en Json", keys=['list_group'])
    
    def execute(self):
        """ renvoit les données pour l'affichage """
        params, self.server_nb = get_request(self.request)
        result = {'titre': "Gestion des groupes", 'sstitre':self.description}
        if self.form_result != {}:
            if self.form_result.has_key('list_group'):
                try:
                    groups = self._valid_form()
                except (MissingValue, MissingKey), mess:
                    groups = {'message':mess.message}
            self.form_result = {}
            return self.frag(groups)
        result.update(_get_menu(self.server_nb, 'liste'))
        result.update(self._get_form())
        d = {'template':'scribe_group', 'data':{'content':result, 'templates':used_templates}}
        return 0, uni(str(d))

    def frag(self, datas):
        """ renvoie un fragment pour le listing des groupes """
        result = datas
        d = {'template':'scribe_group', 'data':{'content':result, 'templates':used_templates}}
        return 0, uni(str(d))
    
    def _get_form(self):
        """ renvoit le formulaire de listing de groupe 
        @group_type : type de groupe à lister (select)
        @namepart : partie du nom du groupe (input)
        """
        types = [{'name':keyword, 'libelle':libelle}for libelle, keyword in typedict.items()]
        group_type = {'name':'group_type', 'libelle': 'Type de groupe', 
                    'options':[{'name':'', 'options':types}], 'inline':'ok'}
        namepart = {'name':'namepart', 'libelle':"Partie du nom", 'inline':'ok'}
        return dict(namepart=namepart, group_type=group_type, validate=self._get_valid_btn())

    def _valid_form(self):
        """ renvoie la liste des groupes du type demandé et dont le nom contient namepart """
        if self.form_result['list_group'] == {}:
            raise MissingKey, "Erreur : problème de validation de formulaire"
        resultat = format_form_result(self.form_result['list_group'])
        if resultat.has_key('namepart'): namepart = resultat['namepart']
        else: namepart = ''
        namepart = namepart.strip()
        if resultat.has_key('group_type'): _type = resultat['group_type']
        else:raise MissingKey, "Erreur : Problème de validation de formulaire, le type n'est pas passé"
        return self.get_groups(_type, namepart)

    def _get_valid_btn(self):
        """ bouton valider pour la creation d'user passe en meme temps le type en cour (classe, nivo...)"""
        validate={}
        validate['href'] = make_form_link(self.server_nb, self.name, True, ['list_group'], "group_list_div_container")
        validate['icone'] = "/image/ok.gif"
        validate['libelle'] = "Lister"
        validate['title'] = "Lister ces groupes"
        return validate

    def get_groups(self, _type, name_part=''):
        """ renvoit la description des groupes de type _type 
            dont le nom contient namepart
            les données sont formatées pour la templatisation
        """
        all_groups = backend.list_groups(_type)
        # filtrage par partie du nom
        if name_part: groups = [{'name':name}for name in all_groups if name_part in name]
        else:groups = [{'name':name}for name in all_groups]
        if 'Membres' in listingdict[_type]:
            for group in groups:
                group['membres'] = self.get_membres_btn(group['name'])
        if 'Partages' in listingdict[_type]:
            for group in groups: 
                if backend.get_group_shares(group['name']):
                    group['share'] = {'text':'oui'}
                else:
                    group['share'] = self.get_add_share_btn(group['name'])
        if 'Liste de diffusion' in listingdict[_type]:
            for group in groups: 
                if backend.get_group_maillist(group['name'])!=[]:
                    group['maillist'] = {'text':backend.get_group_maillist(group['name'])[0]}
                else:
                    group['maillist'] = self.get_maillist_btn(group['name'])
        if 'Suppression' in listingdict[_type]:
            for group in groups:
                group['delete'] = self.get_delete_btn(group['name'])
        return dict(group_listed=groups, entete=listingdict[_type])
    
    def get_add_share_btn(self, name):
        """ renvoie la description du bouton d'ajout de partage """
        btn={}
        btn['href'] = make_js_link(self.server_nb, "scribe_group_modify", balise='mod_group_div_container', group_name=name, add_share='True')
        btn['icone'] = ""
        btn['libelle'] = "Créer"
        btn['title'] = "Créer un partage pour ce groupe"
        btn['_class'] = 'scribe_simple_link'
        return dict(btn=btn)
    
    def get_maillist_btn(self, name):
        """ renvoie la description du bouton d'ajout de maillist """
        btn={}
        btn['href'] = make_js_link(self.server_nb, "scribe_group_modify", balise='mod_group_div_container',
                                        group_name=name, add_maillist='True')
        btn['icone'] = ""
        btn['libelle'] = "Créer"
        btn['title'] = "Créer une liste de diffusion pour ce groupe"
        btn['_class'] = 'scribe_simple_link'
        return dict(btn=btn)
    
    def get_delete_btn(self, name):
        """ renvoie la description du bouton de suppression de groupe """
        btn = {}
        href = make_js_link(self.server_nb, "scribe_group_delete", balise='mod_group_div_container',
                                    confirm=True, group_name=name, del_group='True')
        btn['href'] = make_confirm_link("Etes-vous sûr de vouloir supprimer ce groupe", href)
        btn['icone'] = ""
        btn['libelle'] = "Supprimer ce groupe"
        btn['title'] = "Supprimer ce groupe"
        btn['_class'] = 'scribe_simple_link'
        return dict(btn=btn)
    
    def get_membres_btn(self, name):
        """ renvoie la description du lien vers la section de gestion des membres """
        btn = {}
        btn['href'] = make_js_link(self.server_nb, "scribe_group_members", balise='mod_group_div_container', group_name=name)
        btn['icone'] = ""
        btn['libelle'] = "Membres"
        btn['title'] = "Gérer les membres de ce groupe"
        btn['_class'] = 'scribe_simple_link'
        return dict(btn=btn)

class GroupModify(Action):
    """ Renvoie le formulaire de modification d'un groupe et gère le retour """
    user_description = Dict(default={}, doc="description de l'éxécutant", keys=['ip', 'name'])
    name = 'scribe_group_modify'
    libelle = "Modifier un groupe"
    category = None
    description = 'Modifier un groupe'
    request = Dict(default={}, doc="arguments de la requete en cour cote frontend", keys=['server', 'action', 'group_name', 
        'add_share', 'add_maillist'])
    form_result = Dict(default={}, doc="Retour de formulaire en Json", keys=['list_group', 'add_maillist'])
    
    def execute(self):
        """ renvoit les données pour l'affichage """
        params, self.server_nb = get_request(self.request)
        result = {}
        # ajout de partage au groupe
        if params.has_key('add_share'):
            if params.has_key('group_name'): sharename = params['group_name'][0]
            if backend.add_share(sharename, sharename): 
                message = "Le partage %s a bien été créé et associé au groupe du même nom"%sharename
            else:
                message = "Erreur : Echec de la création du partage %s"%sharename
            # repris du fichier /usr/share/eole/backend/
            if 'norestart' in sys.argv:
                backend.synchronize(False)
            else:
                backend.synchronize(True)
            params.pop('add_share')
            return self.frag({'message':"Le partage %s a bien été créé et associé au groupe du même nom"%sharename})
        # ajout d'une liste de diffusion
        if params.has_key('add_maillist'):
            if params.has_key('group_name'): name = params['group_name'][0]
            params.pop('add_maillist')
            return self.frag(self._get_add_maillist_form(name))
        # validation de l'ajout de liste de diffusion
        if self.form_result != {}:
            if self.form_result.has_key('add_maillist'):
                try:
                    message = {'message':self.add_maillist()}
                except (MissingValue, MissingKey), mess:
                    message = {'message':mess.message}
                self.form_result = {}
                return self.frag(message)
        d = {'template':'scribe_group', 'data':{'content':result, 'templates':used_templates}}
        return 0, uni(str(d))

    def frag(self, datas):
        """ renvoie un fragment """
        result = datas
        result['toexec'] = make_form_link(self.server_nb, 'scribe_group_list', False,
                                                ['list_group'], "group_list_div_container")
        d = {'template':'scribe_group', 'data':{'content':result, 'templates':used_templates}}
        return 0, uni(str(d))

    def add_maillist(self):
        """ ajoute une liste de diffusion pour un groupe """
        if not self.form_result.has_key('add_maillist'):
            raise MissingValue, "Erreur : il manque une clé à la validation du formulaire"
        resultat = format_form_result(self.form_result['add_maillist'])
        if not resultat.has_key('add_mail_group') or not resultat.has_key('domain'):
            raise MissingValue, "Erreur : il manque une clé à la validation du formulaire"
        group = resultat['add_mail_group']
        domain = resultat['domain']
        if backend.add_maillist(group, domain):
            return "Une liste de diffusion %s a été créée et associée au groupe %s"%(domain, group)
        else:
            return "Erreur lors de la création de la liste de diffusion"

    def _get_add_maillist_form(self,name):
        """ renvoie la description du formulaire d'ajout de liste de diffusion au groupe de nom :name:
        """
        add_mail_group = {'name':'add_mail_group', 'libelle':"Nom du groupe", 'inline':'ok', 'disabled':'ok',
                            'default_value':name}
        opts = [{'name':"internet", "libelle":"Domaine internet"}, {'name':"restreint", "libelle":"Domaine restreint"}]
        domain = {'name':'domain', 'libelle':"Domaine de la liste", 
                                'options':[{'name':'', 'options':opts}],'inline':'ok'}
        return dict(add_mail_group=add_mail_group, domain=domain, validate=self._get_maillist_valid_btn())

    def _get_maillist_valid_btn(self):
        """ renvoie la description du bouton valider pour une maillist """
        validate={}
        validate['href'] = make_form_link(self.server_nb, self.name, True, ['add_maillist'], "mod_group_div_container")
        validate['icone'] = "/image/ok.gif"
        validate['libelle'] = "Créer"
        validate['title'] = "Créer les liste de diffusion pour ce groupe"
        return validate

class GroupDelete(Action):
    """ Supprime un groupe """
    user_description = Dict(default={}, doc="description de l'éxécutant", keys=['ip', 'name'])
    name = 'scribe_group_delete'
    libelle = "Supprimer un groupe"
    category = None
    description = 'Supprimer un groupe'
    request = Dict(default={}, doc="arguments de la requete en cour cote frontend", keys=['server', 'action', 'group_name', 
        'del_group'])
    form_result = Dict(default={}, doc="Retour de formulaire en Json", keys=['group_deleting'])
    
    def execute(self):
        """ renvoit les données pour l'affichage """
        params, self.server_nb = get_request(self.request)
        result = {}
        if params.has_key('del_group'):
            if params.has_key('group_name'): 
                todel = params['group_name'][0]
                if backend.get_group_shares(todel):## il y a des partages, on demande si il faut supprimer les reps
                    result.update(self._get_form(todel))
                else:## pas de partages, on supprime directement
                    try:
                        datas = self._remove_group(todel)
                    except (MissingKey, MissingValue), mess:
                        datas = {'message':mess.message}
                    # apres suppression on rafraichit la liste des groupes
                    datas.update({'toexec':make_form_link(self.server_nb, 'scribe_group_list', False,
                                                     ['list_group'], "group_list_div_container")})
                    return self.frag(datas)
        if self.form_result != {}:
            try:
                datas = self._valid_form()
            except (MissingKey, MissingValue), mess:
                datas = {'message':mess.message}
            self.form_result = {}
            datas.update({'toexec':make_form_link(self.server_nb, 'scribe_group_list', False,
                                ['list_group'], "group_list_div_container")})
            return self.frag(datas)
        d = {'template':'scribe_group', 'data':{'content':result, 'templates':used_templates}}
        return 0, uni(str(d))

    def frag(self, datas):
        """ renvoie un fragment 
            :datas: dictionnaire contenant les variables mise en forme par le template
        """
        result = datas
        d = {'template':'scribe_group', 'data':{'content':result, 'templates':used_templates}}
        return 0, uni(str(d))

    def _get_form(self, todel):
        """ renvoie la decsription du formulaire de suppression de groupe """
        del_share_dirs = {'name':'del_share_dirs', 
            'libelle':"Voulez-vous également supprimer les répertoires associés aux partages du groupe '%s' ?"%todel,
            'value':'del_share_dirs', 'checked':''
            }
        todel = {'name':'group_todel', 'default_value':todel}
        return dict(del_share_dirs=del_share_dirs, todel=todel, validate=self._get_valid_btn())

    def _get_valid_btn(self):
        """ renvoie la description du bouton valider """
        validate={}
        validate['href'] = make_form_link(self.server_nb, self.name, True, ['group_deleting'], 
                                        balise='mod_group_div_container')
        validate['icone'] = "/image/ok.gif"
        validate['libelle'] = "Valider"
        validate['title'] = "Supprimer le groupe"
        return validate

    def _valid_form(self):
        """ valide le formulaire de suppression de groupe """
        if not self.form_result.has_key('group_deleting'):
            raise MissingKey, "Erreur : il manque des données pour la suppression de groupe."
        resultat = format_form_result(self.form_result['group_deleting'], check=True)
        if resultat.keys() != ['del_share_dirs', 'group_todel']:
            raise MissingKey, "Erreur : il manque des données pour la suppression de groupe."
        todel = resultat['group_todel'].strip()
        del_share = resultat['del_share_dirs']
        message = "Le groupe %s a été supprimé ainsi que :\\n     - ses partages\\n      - sa liste de diffusion"%todel
        if del_share:
            if not backend.del_group(todel):
                raise MissingKey, "Erreur : erreur lors de la suppression du groupe %s."%todel
            message += "\\n ainsi que les répertoires de ses partages"
        else:
            if not backend.del_group(todel, rmdirs=False):
                raise MissingKey, "Erreur : erreur lors de la suppression du groupe %s."%todel        
        return dict(message=message)
    
    def _remove_group(self, todel):
        """ suppression d'un groupe """
        if not backend.is_group(todel):
            raise MissingValue, "Erreur : le groupe %s n'existe pas."%todel
        if not backend.del_group(todel):
            raise MissingValue, "Erreur : erreur lors de la suppression de %s."%todel
        return dict(message="Le groupe %s a bien été supprimé ainsi que: \\n    - ses partages \\n    - sa liste de diffusion"%todel)

class GroupMembers(Action):
    """ Permet de gérer les membres d'un groupe """
    user_description = Dict(default={}, doc="description de l'éxécutant", keys=['ip', 'name'])
    name = 'scribe_group_members'
    libelle = "Gérer les membres d'un groupe"
    category = None
    description = "Gérer les membres d'un groupe"
    request = Dict(default={}, doc="arguments de la requete en cour cote frontend", 
                                keys=['server', 'action', 'group_name'])
    form_result = Dict(default={}, doc="Retour de formulaire en Json", keys=[])
    
    def execute(self):
        """ renvoit les données pour l'affichage """
        params, self.server_nb = get_request(self.request)
        result = {}
        if params.has_key('group_name'):
            self.group_name = params['group_name']
            datas = self._get_form()
            params.pop('group_name')
            return self.frag(datas)
        else:
            return self.frag({'toexec':make_js_link(self.server_nb, 'scribe_user_list')})
        result.update(self._get_form())
        d = {'template':'scribe_group', 'data':{'content':result, 'templates':used_templates}}
        return 0, uni(str(d))

    def frag(self, datas):
        """ renvoie un fragment vers la page """
        result = datas
        d = {'template':'scribe_group', 'data':{'content':result, 'templates':used_templates}}
        return 0, uni(str(d))

    def _get_form(self):
        """ renvoie la description du formulaire de gestion des membres de groupe """
        members = backend.get_members(self.group_name)
        groups = backend.list_groups('Groupe')
