# -*- coding: UTF-8 -*-
###########################################################################
# Eole NG - 2007  
# Copyright Pole de Competence Eole  (Ministere Education - Academie Dijon)
# Licence CeCill  cf /root/LicenceEole.txt
# eole@ac-dijon.fr 
#  
# Action scribe_grouped_edition
# Edition d'utilisateur par groupe
#       
###########################################################################
from twisted.python import log
from ead2.backend.lib.action import * # import des actions et des types
from ead2.backend.actions.tools import get_request, make_js_link, make_form_link, format_form_result
from ead2.lib.error import MissingKey, MissingValue
from scribe import backend
from ead2.lib.libead import uni
from ead2.backend.actions.scribe import scribe_tools

used_templates = ['main', 'form', 'formselect', 'accordion', 'listlist', 'checklist']

class GroupedEdition(Action):
    """ permet l'édition groupé d'utilisateur """
    user_description = Dict(default={}, doc="description de l'éxécutant", keys=['ip', 'name'])
    name = 'scribe_grouped_edition'
    libelle = "Edition groupée"
    category = "Gestion/Utilisateurs/"
    description = "édition groupée d'utilisateurs"
    request = Dict(default={}, doc="arguments de la requete en cour cote frontend", 
                                keys=['server', 'action', 'btns', 'form', 'btn'])
    form_result = Dict(default={}, doc="Retour de formulaire en Json", keys=['search', 'userlist', 'inscription_edit', 'quota_edit', 'domain_edit', 'profil_edit'])

    def execute(self):
        """ renvoit les données pour l'affichage """
        params, self.server_nb = get_request(self.request)
        result = {'titre':self.description}
        if params.has_key('form'):
            ## appel du formulaire d'action (edition de profil, quota, inscription a des groupes)
            datas = self._get_action_form(params['form'][0])
            return self.frag(datas)
        if params.has_key('btn'):
            return self.frag(self._get_action_btns(int(params['btn'][0])))
        if params.has_key('btns'):
            return self.frag(self._get_action_btns())
        if self.form_result != {}:# validation de formulaire
            try:
                filtered_datas = self._valid_form()
            except (MissingKey, MissingValue), mess:
                filtered_datas = {'message':mess.message}
            filtered_datas.update({'toexec':make_js_link(self.server_nb, self.name, balise='action_div_container',confirm=True, btns='True')})
            self.form_result = {}
            return self.frag(filtered_datas)
        result.update(scribe_tools._get_user_listing_form())
        result.update({'validate':self._get_valid_btn()})
        d = {'template':'scribe_edition', 'data':{'content':result, 'templates':used_templates}}
        return 0, uni(str(d))
    
    def frag(self, filtered_datas):
        """ renvoit les datas pour l'affichage de la liste des utilisateurs correpondant à la recherche"""
        result = filtered_datas
        d = {'template':'scribe_edition', 'data':{'content':result, 'templates':used_templates}}
        return 0, uni(str(d))
    
    def _valid_form(self):
        """ valide les retours de formulaire """
        if self.form_result.has_key('search'):
            return self._valid_user_listing()
        elif self.form_result.has_key('inscription_edit'):
            return self._valid_inscription()
        elif self.form_result.has_key('quota_edit'):
            return self._valid_quota()
        elif self.form_result.has_key('domain_edit'):
            return self._valid_domain()
        elif self.form_result.has_key('profil_edit'):
            return self._valid_profil()

    def _valid_inscription(self):
        """ valide les inscriptions à des groupes """
        if not self.form_result.has_key('userlist'):
            raise MissingKey, "Erreur : il manque des données pour l'inscription."
        userlist = format_form_result(self.form_result['userlist'], False).keys()
        resultat = format_form_result(self.form_result['inscription_edit'])
        if not resultat.has_key('group_name'):
            raise MissingKey, "Erreur : il manque des données pour l'inscription."
        group = resultat['group_name']
        for user in userlist:
            if not backend.inscription(user, group):
                raise MissingKey, "Erreur : une erreur s'est produite lors de l'inscription de %s"%user
        return dict(message="Les utilisateurs ont bien été inscrits au groupe")

    def _valid_quota(self):
        """ valide l'attribution de quota à un groupe """
        if not self.form_result.has_key('userlist'):
            raise MissingKey, "Erreur : il manque des données pour l'édition de quota."
        userlist = format_form_result(self.form_result['userlist'], False).keys()
        resultat = format_form_result(self.form_result['quota_edit'])
        if not resultat.has_key('quota'):
            raise MissingKey, "Erreur : il manque des données pour l'édition de quota."
        quota = resultat['quota']
        for user in userlist:
            if not backend.set_quota(user, quota):
                raise MissingKey, "Erreur : une erreur s'est produite lors de \\n l'attribution de quota à %s"%user
        return dict(message="%s Mo de quota disque ont été attribué aux utilisateurs sélectionnés"%quota)

    def _valid_domain(self):
        """ valide le changement de domaine mail d'un groupe """
        if not self.form_result.has_key('userlist'):
            raise MissingKey, "Erreur : il manque des données pour le changement de domaine mail."
        userlist = format_form_result(self.form_result['userlist'], False).keys()
        resultat = format_form_result(self.form_result['domain_edit'])
        if not resultat.has_key('domain'):
            raise MissingKey, "Erreur : il manque des données pour le changement de domaine mail."
        domain = resultat['domain']
        for user in userlist:
            if not backend.mod_mail(user, 'local', "%s@%s"%(user,domain)):
                raise MissingKey, "Erreur : erreur lors du changement de domaine mail de %s."%user
        return dict(message="Les utilisateurs sélectionnés ont désormais un mail de type %s."% domain)

    def _valid_profil(self):
        """ valide le changement de profil """
        if not self.form_result.has_key('userlist'):
            raise MissingKey, "Erreur : il manque des données pour le changement de profil."
        userlist = format_form_result(self.form_result['userlist'], False).keys()
        resultat = format_form_result(self.form_result['profil_edit'])
        if not resultat.has_key('_sambaProfilePath'):
            raise MissingKey, "Erreur : il manque des données pour le changement de profil."
        profil = resultat['_sambaProfilePath']
        profil = int(profil)
        for user in userlist:
            if not backend.set_profil(user, profil):
                raise MissingKey, "Erreur : erreur lors du changement de profil de l'utilisateur %s"%user
        return {'message':"Le profil %s a bien été attribué aux utilisateurs sélectionnés."%profil}

    def _valid_user_listing(self):
        """ renvoie la liste des utilisateurs """
        request = format_form_result(self.form_result['search'])
        keys = request.keys()
        if 'alpha_key' not in keys or 'domains' not in keys or 'user_class' not in keys or 'user_type' not in keys or 'namepart' not in keys or 'user_group' not in keys:
            return ""
        alpha_key, _type, domain, _class, namepart, group = request['alpha_key'], request['user_type'], request['domains'], request['user_class'], request['namepart'], request['user_group']
        userlist = []
        for user in scribe_tools.user_filter(_type, domain, _class, namepart, alpha_key, group):
            ## construction d'un lien pour l'édition de l'utilisateur
#            btn = {'href':make_js_link(self.server_nb, 'scribe_user_modify', balise='user_edit_div_container',user=user),
#                   'libelle':user, 'title':"Editer %s"%user, 'icone':'', '_class':'scribe_simple_link'}
            if user in backend.get_profs():_type = 'prof'
            else:_type='élève'
            userlist.append({'name':user, 'libelle':"%s (%s)"%(user, _type), 'value':user, 'checked':'true', 'inline':'ok'})#, 'btn':btn})
        return dict(userlist=userlist, all_users_btn=self._get_all_users_btn(), none_users_btn=self._get_none_users_btn())

    def _get_action_form(self, _type):
        """ renvoie la description du formulaire d'édition groupée """
        if _type == 'inscription':
            return self._get_inscription_form()
        elif _type == 'quota':
            return self._get_quota_form()
        elif _type == 'domain':
            return self._get_domain_form()
        elif _type == 'profil':
            return self._get_profil_form()

    def _get_inscription_form(self):
        groups = [{'name':gr, 'libelle':gr}for gr in backend.list_groups('Groupe')]
        subscribe_to_group = {'name':'group_name', 'options':[{'name':'', 'options':groups}]}

        toexec=make_js_link(self.server_nb, self.name, balise='div_quota_container', confirm=True, btn='1')
        toexec+=make_js_link(self.server_nb, self.name, balise='div_domain_container', confirm=True, btn='2')
        toexec+=make_js_link(self.server_nb, self.name, balise='div_profil_container', confirm=True, btn='3')
        texte = "Inscrivez les utilisateurs sélectionnés au groupe:"
        validate = {'href':make_form_link(self.server_nb, self.name, True, ['inscription_edit', 'userlist'], 
                            balise='div_inscription_container'),
                            'libelle':"Valider", 'title':"Inscrire à ce groupe",
                            'icone':"/image/ok.gif"}
        return dict(subscribe_to_group=subscribe_to_group, toexec=toexec, texte=texte, validate=validate)

    def _get_quota_form(self):
        quota_input = {'name':'quota', 'libelle':'Quota disque (en Mo)', 'onblur':"testNumber('quota', 'error_quota')"}
        toexec=make_js_link(self.server_nb, self.name, balise='div_inscription_container', confirm=True, btn='0')
        toexec+=make_js_link(self.server_nb, self.name, balise='div_domain_container', confirm=True, btn='2')
        toexec+=make_js_link(self.server_nb, self.name, balise='div_profil_container', confirm=True, btn='3')
        texte = "Attribuez un quota disque aux utilisateurs sélectionnés"
        validate = {'href':make_form_link(self.server_nb, self.name, True, ['quota_edit', 'userlist'], 
                            balise='div_quota_container'),
                            'libelle':"Valider", 'title':"Attribuer les quotas",
                            'icone':"/image/ok.gif"}
        return dict(quota_input=quota_input, toexec=toexec, texte=texte, validate=validate)

    def _get_domain_form(self):
        domain_select = {'name':'domain', 'libelle':"Domaine mail", 
                        'options':[{'name':'', 'options':[{'name':backend.get_domains()[0], 'libelle':"domaine restreint"},
                                                        {'name':backend.get_domains()[1], 'libelle':"domaine internet"}]}]
                    }
        toexec=make_js_link(self.server_nb, self.name, balise='div_inscription_container', confirm=True, btn='0')
        toexec+=make_js_link(self.server_nb, self.name, balise='div_quota_container', confirm=True, btn='1')
        toexec+=make_js_link(self.server_nb, self.name, balise='div_profil_container', confirm=True, btn='3')
        texte = "Changez le domaine mail des utilisateurs sélectionnés"
        validate = {'href':make_form_link(self.server_nb, self.name, True, ['domain_edit', 'userlist'], 
                            balise='div_domain_container'),
                            'libelle':"Valider", 'title':"Changer le domaine mail",
                            'icone':"/image/ok.gif"}
        return dict(domain_select=domain_select, toexec=toexec, texte=texte, validate=validate)

    def _get_profil_form(self):
        """ renvoie la description du formulaire d'édition de profil """
        profil_opts = [{'name':'1', 'libelle':'local'}, 
                        {'name':'2', 'libelle':"obligatoire - profil"},
                        {'name':'3', 'libelle':"obligatoire - profil2"}]
        profil_select = {'name':'_sambaProfilePath', 'libelle':"Profil utilisateur", 
                         'options':[{'name':"Profil disponible", 'options':profil_opts}]}
        toexec=make_js_link(self.server_nb, self.name, balise='div_inscription_container', confirm=True, btn='0')
        toexec+=make_js_link(self.server_nb, self.name, balise='div_domain_container', confirm=True, btn='2')
        toexec+=make_js_link(self.server_nb, self.name, balise='div_quota_container', confirm=True, btn='1')
        texte = "Changer le profil des utilisateurs sélectionnés"
        validate = {'href':make_form_link(self.server_nb, self.name, True, ['profil_edit', 'userlist'], 
                            balise='div_profil_container'),
                            'libelle':"Valider", 'title':"Changer le profil",
                            'icone':"/image/ok.gif"}
        return dict(validate=validate, profil_select=profil_select, toexec=toexec, texte=texte)
        
    def _get_action_btns(self, num=''):
        """ renvoie la description des boutons pour les actions possibles """
        # l'attribut name permet de créer la div où s'effctuera l'action
        btn1 = {'href': make_js_link(self.server_nb, self.name, balise='div_inscription_container', form='inscription'),
                    'libelle':"Inscrire ces utilisateurs à d'autres groupes", 
                    'title': '', 'icone': "/image/scribe/action_over.png", '_class':'scribe_action_btn',
                    'name':"div_inscription_container"}
        btn2 = {'href': make_js_link(self.server_nb, self.name, balise='div_quota_container', form='quota'), 
                    'libelle':"Définir des quotas disques pour ces utilisateurs", 
                    'title': '', 'icone': "/image/scribe/action_over.png", '_class':'scribe_action_btn',
                    'name':"div_quota_container"}
        btn3 = {'href': make_js_link(self.server_nb, self.name, balise='div_domain_container', form='domain'),
                    'libelle':"Changer le domaine mail pour ces utilisateurs", 
                    'title': '', 'icone': "/image/scribe/action_over.png", '_class':'scribe_action_btn',
                    'name':"div_domain_container"}
        btn4 = {'href': make_js_link(self.server_nb, self.name, balise='div_profil_container', form='profil'),
                    'libelle':"Changer le profil pour ces utilisateurs", 
                    'title': '', 'icone': "/image/scribe/action_over.png", '_class':'scribe_action_btn',
                    'name':"div_profil_container"}
        action_btns = [btn1, btn2, btn3, btn4]
        if num !='' :return dict(action_btn=action_btns[num])
        return dict(action_btns = action_btns)

    def _get_valid_btn(self):
        """ renvoie la description du bouton de validation de la recherche
            la cible du retour est la balise : user_div_container
        """
        icone = '/image/ok.gif'
        title = 'Lister selon les filtres'
        libelle = 'Lister'
        href = make_form_link(self.server_nb, self.name, True, ['search'], 'user_td_container')
        return dict(icone=icone, title=title, libelle=libelle, href=href)

    def _get_none_users_btn(self):
        """ renvoie la description du bouton de déselection des utilisateurs de la liste """
        icone = ''
        title = 'Aucun utilisateur'
        libelle = 'Aucun'
        href = "javascript:formSelectNone('userlist')"
        return dict(icone=icone, title=title, libelle=libelle,
                href=href)

    def _get_all_users_btn(self):
        """ renvoie la description du bouton de selection de
        tous les utilisateurs de la liste """
        icone = ''
        title = 'Tous les utilisateurs'
        libelle = 'Tous'
        href = "javascript:formSelectAll('userlist')"
        return dict(icone=icone, title=title, libelle=libelle,
                href=href)
