# -*- coding: UTF-8 -*-
###########################################################################
# Eole NG - 2007  
# Copyright Pole de Competence Eole  (Ministere Education - Academie Dijon)
# Licence CeCill  cf /root/LicenceEole.txt
# eole@ac-dijon.fr 
#  
# Action scribe_printers
# outil de gestion des imprimantes (gestion simplifiée et lien vers cups)
#       
###########################################################################
from twisted.python import log
from ead2.backend.lib.action import * # import des actions et des types
from ead2.backend.actions import tools # import get_request, make_js_link, make_form_link, format_form_result
from ead2.lib.error import MissingKey, MissingValue
from ead2.lib.libead import uni
from ead2.backend.actions.scribe import scribe_tools
from ead2.config.config import ip_locale

used_templates = ['main']

class Printers(Action):
    """ outils de gestion d'imprimante 
        trois types de page: la gestion simplifiée des imprimantes, la liste des travaux en cour, l'interface cups
        retour possible:
            - la liste des jobs d'une imprimante est renvoyée dans un fragment en réponse à la clé 'printer' 
              dans la requête (params)
    """
    user_description = Dict(default={}, doc="description de l'éxécutant", keys=['ip', 'name'])
    name = 'scribe_printers'
    libelle = "Imprimantes"
    category = "Outils/Imprimantes"
    description = "Gestion des imprimantes"
    request = Dict(default={}, doc="arguments de la requete en cour cote frontend", 
                                keys=['server', 'action', 'page', 'printer', 'stop', 'start'])
    form_result = Dict(default={}, doc="Retour de formulaire en Json", keys=[])

    def execute(self):
        """ renvoie les données pour l'affichage """
        params, self.server_nb = tools.get_request(self.request)
        self.page = 'all'
        if params.has_key('page'):
            self.page = params['page'][0]
        if params.has_key('stop'):
            try:
                datas  = self.stop_printer(params['stop'][0])
            except (MissingKey, MissingValue, Exception), mess:
                datas = {'message':mess.message}
            params.pop('stop')
            return self.frag(datas)
        if params.has_key('start'):
            try:
                datas  = self.start_printer(params['start'][0])
            except (MissingKey, MissingValue, Exception), mess:
                datas = {'message':mess.message}
            params.pop('start')
            return self.frag(datas)
        if params.has_key('printer'):
            try:
                datas  = self._get_printer_jobs(params['printer'][0])
            except (MissingKey, MissingValue, Exception), mess:
                datas = {'message':mess.message}
            params.pop('printer')
            return self.frag(datas)
        result = {'titre':self.description, 'sstitre':''}
        result.update(self._get_datas())
        result.update(self._get_menu())
        d = {'template':'scribe_printers', 'data':{'content':result, 'templates':used_templates}}
        return 0, uni(str(d))

    def frag(self, datas):
        """ retourne un fragment """
        result = datas
        d = {'template':'scribe_printers', 'data':{'content':result, 'templates':used_templates}}
        return 0, uni(str(d))

    def _get_datas(self):
        """ renvoie les datas pour la page d'imprimante """
        if self.page == 'all':
            return self._get_printer_form()
        if self.page == 'jobs':
            return self._get_printer_jobs()

    def _get_printer_form(self):
        """ renvoie la description pour la page d'action sur les imprimantes 
            :stopped: illustration du statut de l'imprimante
            :name_link: lien appelant les travaux en attente de l'imprimante concernée
            :stop_link: lien appelant l'arrêt de l'imprimante
        """
        try:
            printers = scribe_tools._get_printers()
        except:
            printers = []
        for printer in printers:
            if printer['stopped'] == 'ok':
                printer['stopped'] = "/image/scribe/printer-stopped.gif"
                printer['stop_link'] = {'href':tools.make_js_link(self.server_nb, self.name, 
                                                    start=printer['name'], balise='div_message_container'),
                                                    'title':'Redémarrer cette imprimante', "libelle":"", 
                                                    "icone":"/image/redemarrer_serv.gif", '_class':'btn'}
            elif printer['stopped'] == 'idle':
                printer['stopped'] = "/image/scribe/printer-idle.gif"
                printer['stop_link'] = {'href':tools.make_js_link(self.server_nb, self.name, 
                                                    stop=printer['name'], balise='div_message_container'),
                                                    'title':'Arrêter cette imprimante', "libelle":"", 
                                                    "icone":"/image/arreter_serv.gif", '_class':'btn'}
            else:
                printer['stopped'] = "/image/scribe/printer-processing.gif"
                printer['stop_link'] = {'href':tools.make_js_link(self.server_nb, self.name, 
                                                    stop=printer['name'], balise='div_message_container'),
                                                    'title':'Arrêter cette imprimante', "libelle":"", 
                                                    "icone":"/image/arreter_serv.gif", '_class':'btn'}
            printer['name_link'] = {'href':tools.make_js_link(self.server_nb, self.name, 
                                                    printer=printer['name'], balise="div_%s"%printer['name']),
                                                    'title':'Voir cette imprimante', 
                                                    "libelle":"Imprimante %s"%printer['name'], 'icone':"",
                                                    '_class':'scribe_simple_link'}
        return dict(printers=printers)

    def _get_printer_jobs(self, printer_name=None):
        """ renvoie les travaux en cour pour l'imprimante printer_name
            Toutes les imprimantes par défaut
            :close_btn: bouton pour vider le container recevant les travaux d'un imprimante
        """
        uncompleted_jobs = ["Cette Section est en cour de développement"]
        completed_jobs = ["Cette Section est en cour de développement"]
        close_btn = {'href':"javascript:setVoid('div_%s')"%printer_name, 'title':"Fermer", 
                        'libelle':'Fermer','_class':'btn', 'icone':"/image/suppr.gif"}
        if printer_name != None:
            return dict(jobs=[{'entete':"Travaux actifs", 'jobs':uncompleted_jobs},
                           {'entete':"Travaux terminés", 'jobs':completed_jobs}],
                            close_btn=close_btn)
        else:
            return dict(jobs=[{'entete':"Travaux actifs", 'jobs':uncompleted_jobs},
                              {'entete':"Travaux terminés", 'jobs':completed_jobs}])

    def stop_printer(self, printer):
        """ stoppe une imprimante """
        message = scribe_tools.stop_printer(printer)
        return dict(message=message)

    def start_printer(self, printer):
        """ démarre une imprimante """
        message = scribe_tools.start_printer(printer)
        return dict(message=message)

    def _get_menu(self):
        """ renvoie le menu pour les imprimantes (toutes les imprimantes, 
                               travaux d'impression en cour, cups)
        """
        all = {'title':'', 'icone':'/image/refresh.gif', 
                    'href':tools.make_js_link(self.server_nb, "scribe_printers", page='all'), 'libelle':"Imprimantes"}
        jobs = {'title':"Travaux d'impression en cour", 'icone':'/image/quota.gif',
                    'href':tools.make_js_link(self.server_nb, "scribe_printers", page='jobs'), 'libelle':"Travaux"}
        cups_url = "https://%s:631/"%ip_locale
        cups = {'title':"Interface d'administration CUPS", 'icone':'/image/machines.gif',
                'href':"javascript:OuvreFenetre('%s', 'cups')"%(cups_url), 'libelle':"Cups"}
        if self.page == 'all':
            all['href'] = ''
        elif self.page == 'jobs':
            jobs['href'] = ''
        return dict(menus=(all, jobs, cups))
