# -*- coding: UTF-8 -*-
###########################################################################
# Eole NG - 2007  
# Copyright Pole de Competence Eole  (Ministere Education - Academie Dijon)
# Licence CeCill  cf /root/LicenceEole.txt
# eole@ac-dijon.fr 
#  
# Outils pour l'ead de scribe
#       
###########################################################################
"""Actions permettant de gérer les utilisateurs"""
from twisted.python import log
from scribe import backend
import commands
from os.path import isfile
from os import popen

def filter_by_alpha(users, alpha):
    """ filtre une liste d'utilisateur par nom de famille sur la première lettre"""
    result = []
    for user in users:
        cut = user.split('.')
        if len(cut) == 2:
            if cut[1][0].upper() == alpha.upper():
                result.append(user)
        else:
            result.append(user)
    return result
        
def user_filter(_type='', domain='', _class='', namepart='', alpha_key='', group=''):
    """ filtre les utilisateurs selon les parametres  on cree le filtre ldap et on lance une recherche
        :alpha_key: première lettre du nom de l'utilisateur
        :_type: type d'utilisateur prof/eleve (pupil/teacher)
        :domain: domaine mail
        :group: groupe des utilisateurs
        :class: classe de l'utilisateur (pour les élèves à priori, marche aussi pour les profs admin de classe)
        :size: nombre d'utilisateur par page
    """
    filtre = "(&%s"%backend.user_filters
    if _type:
        if _type == 'pupil': 
            filtre += backend.eleve_filters 
        else: 
            filtre += backend.prof_filters
    if domain:
        filtre += "(mail=*@%s)"%domain
    if _class:
        filtre += "(Divcod=%s)"%_class
    if namepart:
        filtre += "(sn=*%s*)"%namepart
    filtre += ")"
    log.msg(filtre)
    usrs = backend.ldapsearch(filtre, ['uid'])
    users = [user[1]['uid'][0]for user in usrs]
    if group:
        gr_users = backend.get_members(group)
        users = [user for user in users if user in gr_users]
    users = sort_users(users)
    if alpha_key:
        users = filter_by_alpha(users, alpha_key)
    return users

def _get_alpha_links():
    """ renvoit la description des liens alphabetique """
    alphas = [_get_select_default_value()]
    alphas.extend([{'name':chr(i).upper(), 'libelle':chr(i).upper()}for i in range(97,123)])
    return alphas
    
def _get_select_default_value():
    """ renvoie l'option vide pour un objet select """
    return {'name':'', 'libelle':'', 'default':'ok'}

def _get_user_listing_form():
    alpha_keys = {'name':'alpha_key', 'libelle': 'par la 1ère lettre du nom',
            'options':[{'name':'commençant par', 'options':_get_alpha_links()}] }
    user_types = [_get_select_default_value(),{'name':'teacher','libelle':'professeur'},
                    {'name':'pupil','libelle':'élève'}]
    user_type = {'name':'user_type', 'libelle':"de type", 'options':[{'name':'', 'options':user_types}]}
    # classe
    opts = backend.get_classes()
    classes = [{'name':key, 'options':[{'name':item, 'libelle':item}for item in items]}for key, items in opts.items()]
    classes.insert(0,{'name':'', 'options':[_get_select_default_value()]})
    classe = {'name':'user_class', 'libelle':'de la classe', 'options':classes}
    # groupes
    gr_types = ['Groupe', 'Matiere', "Responsables" , "Equipe"]
    groups = [{'name':key, 'options':[{'name':gr, 'libelle':gr}for gr in backend.list_groups(key)]}for key in gr_types] 
    groups.insert(0, {'name':'', 'options':[_get_select_default_value()]})
    group = {'name':'user_group', 'libelle':'associé au groupe', 'options':groups}
    #domaine
    domains = [{'name':domain, 'libelle':domain}for domain in backend.get_domains()]
    domains.insert(0 , _get_select_default_value())
    domain = {'name':'domains', 'libelle':'du domaine', 'options':[{'name':'', 'options':domains}]}
    nums = [{'name':str(10*num), 'libelle':str(10*num)}for num in range(6)[1:]]
    #nom
    namepart = {'name':'namepart', 'libelle':"Partie du nom"}
    #nombre d'utilisateur
    size = {'name':'size', 'libelle':'par page de', 'options':[{'name':'', 'options':nums}]}
    return dict(alpha_keys=alpha_keys, user_type=user_type, classe=classe, domain=domain, namepart=namepart,
                size=size, group=group)

def sort_users(users):
    """ trie les nom des utilisateurs par ordre alphabétique 
        prenom.nom par nom 
    """
    _list = []
    for user in users:
        login = user.split('.')
        if len(login)>1:_list.append("%s.%s"%(login[1], login[0]))
        else:_list.append("%s"%login[0])
    _list.sort()
    users=[]
    for user in _list:
        login = user.split('.')
        if len(login)>1:users.append("%s.%s"%(login[1], login[0]))
        else:users.append("%s"%login[0])
    return users

def test_date(date):
    """ controle le format **/**/**** d'une date """
    a = date.split('/')
    if len(a) != 3: return False
    if len(a[0])>2 or len(a[0])==0: return False
    if len(a[1])>2 or len(a[1])==0: return False
    if len(a[2])!=4 : return False
    return True

def test_ldap_attribute(attr):
    """ controle un attribut à rentrer dans l'annuaire pas seulement des chiffres"""
    try:
        int(attr)
        return False
    except:
        return True

def test_ldap_entry(entry):
    """ controle une entrée dans l'annuaire, pas seulement des chiffres, pas d'espace """
    if not test_ldap_attribute(entry):return False
    if ' ' in entry: return False
    if '.' in entry:
        entry = entry.replace('.', '')
        if not test_ldap_attribute(entry):return False
    return True

def _get_quotas(_type='depasse'):
    """ renvoie les quotas des utilisateurs du domaine 
        dans le cas _type: qui prend les valeurs "proche", "depasse" ou "tous"
    """
    cmd = "/usr/sbin/repquota -a"
    return_code, quota_lines = commands.getstatusoutput(cmd)
    if return_code:
        raise Exception, "Erreur : un problème s'est produit à la récupération des quotas"
    quota_lines = quota_lines.splitlines()[5:]
    result = []
    for line in quota_lines:
        user = line.split()
        if len(user)>5:
            quota = int(user[3]) / 1000
            if quota >0:
                username = user[0]
                used = int(user[2])/1000
                delai = user[5]
                if used>quota:
                    result.append({'username':username,'used':used, 'quota':quota, 'delai':delai})
                elif used+2000>quota and _type=='proche':
                    result.append({'username':username,'used':used, 'quota':quota})
                elif _type=='tous':
                    result.append({'username':username,'used':used, 'quota':quota})
    return result

def _get_viruses():
    """ renvoie la liste des virus détectés """
    f1 = "/var/log/syslog"
    f2 = "/var/log/syslog.1.gz"
    max_vir = 30
    mois={}
    mois["jan"] = "janvier" #idem
    mois["feb"] = "février"
    mois["fév"] = "février"
    mois["mar"] = "mars" #idem
    mois["apr"] = "avril"
    mois["avr"] = "avril"
    mois["may"] = "mai"
    mois["mai"] = "mai"
    mois["jun"] = "juin"
    mois["jui"] = "juin"
    mois["jul"] = "juillet" #idem
    mois["aug"] = "août"
    mois["aou"] = "août"
    mois["sep"] = "septembre" #idem
    mois["oct"] = "octobre"
    mois["nov"] = "novembre"
    mois["dec"] = "décembre"
    result = []
    viruses = []
    if isfile(f1):
        fp = popen('grep "infected with virus" '+f1)
        lines = fp.readlines()
        lines.reverse()
        result.extend(lines)
        fp.close()
    else:
        pass
    if isfile(f2):
        fp = popen('zgrep "infected with virus" '+f2)
        lines = fp.readlines()
        lines.reverse()
        result.extend(lines)
        fp.close()
    else:
        pass
    if len(result)>max_vir:
        result = result[:max_vir]
    for ligne in result:
        try :
            moisfr = mois[ligne[0:3].lower()]
        except :
            moisfr = ligne[0:3]
        date = ligne[4:6]+" "+moisfr
        ligne = ligne[ligne.find("'")+1:]
        next = ligne.find("'")
        ## fichier
        fichier = ligne[0:next]
        ligne = ligne[next+23:]
        next = ligne.find("'")
        ## virus
        virus = ligne[0:next]
        ## client
        client = ligne[next+12:-2]
        viruses.append({'client':client, 'fichier':fichier, 'date':date, 'virus':virus})
    if len(result) == 0:
        return []
    return viruses

def _get_history():
    """ renvoie l'historique des connexions """
    cmd = "/usr/share/ead/historique.sh"
    return_code, result = commands.getstatusoutput(cmd)
    if return_code:
        raise Exception, "Erreur à la récupération de l'historique des connexions."
    lines = [{'date':" ".join(line.split()[0:5]), 'user':line.split()[5], 'poste':"%s %s"%(line.split()[6], line.split()[8]), 'system':line.split()[7]} for line in result.splitlines()]
    if len(lines) == 0:
        return [{'default':"Aucune entrée n'a été enregistrée"}]
    return lines

def _parse_printers(printers_str):
    """ parse le résultat du script /user/share/ead/printers.sh 
        les tests s'effectue sur les deux langues (an et fr)
        normalement dans l'ead2 c'est fr.
    """
    printers = []
    printer = {}
    for line in printers_str.splitlines():
        line = line.strip()
        if line.startswith("printer") or line.startswith("l'imprimante"):
            if printer != {}:
                printers.append(printer)
                printer = {}
            elt = line.split()
            printer_name = elt[1]
            printer['name'] = printer_name
            if "disabled" in line or "désactivée" in line:
                printer['stopped'] = 'ok'
            elif "idle" in line or "ne fait rien" in line:
                printer['stopped'] = "idle"
            else:
                printer['stopped'] = ""
        elif line.startswith('Description'):
            description = line[13:].strip()
            if "No Information Available" in description:
                printer['description'] = "Description : %s"%printer_name
            else:
                printer['description'] = line
        elif line.startswith('Location') or line.startswith('Lieu'):
            lieu = line.split(':')[1]
            if 'Location Unknown' in lieu:
                printer['lieu'] = "Emplacement non renseigné"
            else:
                printer['lieu'] = "Emplacement : %s"%lieu
    printers.append(printer)
    return printers       

printer_script = "/usr/share/ead/printers.sh"
def _get_printers():
    """ renvoie la description des imprimantes disponibles """
    cmd = "%s all 2>&1"%printer_script
    return_code, printers_str = commands.getstatusoutput(cmd)
    if return_code:
        raise Exception, "Erreur à la récupération de la liste des imprimantes."
    printers = _parse_printers(printers_str)
    return printers

def stop_printer(printer_name):
    """ stoppe une imprimante """
    cmd = "%s disable %s 2>&1"%(printer_script, printer_name)
    return_code, return_str = commands.getstatusoutput(cmd)
    if return_code:
        raise Exception, "Erreur à l'arrêt de l'imprimante %s, \\n %s"%(printer_name, return_str)
    # on controle que ça a marché
    printers = _get_printers()
    for pr in printers:
        if pr['name'] == printer_name:
            if pr['stopped'] != 'ok':
                return "Compte rendu de l'arrêt de %s \\n Erreur : vérifiez votre configuration."%printer_name
    return "Compte rendu de l'arrêt de %s \\n Réussite de l'arrêt de l'imprimante"%printer_name

def start_printer(printer_name):
    """ démarre une imprimante """
    cmd = "%s enable %s 2>&1"%(printer_script, printer_name)
    return_code, return_str = commands.getstatusoutput(cmd)
    if return_code:
        raise Exception, "Erreur au redémarrage de l'imprimante %s, \\n %s"%(printer_name, return_str)
    # on controle que ça a marché
    printers = _get_printers()
    for pr in printers:
        if pr['name'] == printer_name:
            if pr['stopped'] == 'ok':
                return "Compte rendu du démarrage de %s. \\n Erreur : vérifiez votre configuration."%printer_name
    return "Compte rendu du démarrage de %s. \\n Réussite du démarrage de l'imprimante."%printer_name

