# -*- coding: UTF-8 -*-
###########################################################################
# Eole NG - 2007  
# Copyright Pole de Competence Eole  (Ministere Education - Academie Dijon)
# Licence CeCill  cf /root/LicenceEole.txt
# eole@ac-dijon.fr 
#  
# Action scribe_user*
# Creation d'utilisateur
# Recherche puis modification d'utilisateur
#       
###########################################################################
"""Actions permettant de gérer les utilisateurs"""
from twisted.python import log
from ead2.backend.lib.action import * # import des actions et des types
from ead2.backend.actions.tools import get_request, make_js_link, make_form_link, make_confirm_link, format_form_result, make_key_form_link
from ead2.lib.libead import uni
from ead2.lib.error import MissingKey, MissingValue
import commands
from os.path import join
from scribe import backend
from ead2.backend.actions.scribe import scribe_tools

# liste des templates complémentaires utilisés
used_templates = ['main', 'form', 'formselect', 'accordion', 'listlist', 'checklist']

#FIXME: lors du rafraichissement des resultats de recherche post-suppression d'un utilisateur, 
#la gestion des pages n'est pas effectuée.

def _get_menu(server_nb, current, _type=None):
    """ renvoit la description du menu de gestion utilisateur 
    :current: section en cour de traitement (ex: ajout utilisateur user_create)
    :_type: parametre de la section en cour de traitement (ex: pupil/teacher)
    """
    create_pupil = {'icone':'/image/scribe/user_create.gif', 'libelle':"Création d'élève", 
            'title':"Créer un utilisateur", 'href':make_js_link(server_nb, 'scribe_user_create', _type='pupil')}
    create_teacher = {'icone':'/image/scribe/teacher_create.gif', 'libelle':"Création de professeur",
                        'title':"Créer un utilisateur", 'href':make_js_link(server_nb, 'scribe_user_create', _type='teacher')}
    _list = {'icone':'/image/scribe/user_list.gif', 'libelle':"Liste", 
            'title':"Lister des utilisateurs", 'href':make_js_link(server_nb, 'scribe_user_list')}
    if current == 'scribe_user_create' and _type=='pupil':
        create_pupil['href'] = ''
    elif current == 'scribe_user_create' and _type=='teacher':
        create_teacher['href'] = ''
    elif current == 'scribe_user_list':
        _list['href'] = ''
    return dict(menus=[create_pupil, create_teacher, _list])

class UserCreate(Action):
    """ création d'utilisateur 
    :type: type d'utilisateur en cour de création ( élève, prof )
    """
    user_description = Dict(default={}, doc="description de l'éxécutant", keys=['ip', 'name'])
    name = 'scribe_user_create'
    libelle = "Création d'utilisateur"
    category = "Gestion/Utilisateurs/"
    description = 'Créer un utilisateur'
    request = Dict(default={}, doc="arguments de la requete en cour cote frontend", keys=['server', 'action', '_type'])
    form_result = Dict(default={}, doc="Retour de formulaire en Json", keys=['add_user', 'current_type'])
    
    def execute(self):
        """ renvoit les données pour l'affichage """
        params, self.server_nb = get_request(self.request)
        result = {'titre':"Gestion des utilisateurs"}
        self._type = 'pupil'
        if params.has_key('_type'):
            self._type = params['_type'][0].strip()
        if self.form_result !={} :
            log.msg('Retour formulaire %s'%self.form_result)
            if self.form_result.has_key('current_type'):
                self._type = format_form_result(self.form_result['current_type'])['_type']
                self.form_result.pop('current_type')
            log.msg(self._type)
            try:
                result['message'] = self._valid_form()
            except MissingKey, mess:
                result['message'] = mess.message
            except MissingValue, mess:
                result['message'] = mess.message
            self.form_result = {}
        typedico={'pupil':'élève', 'teacher':'professeur'}
        result['sstitre'] = "Créer un %s"%typedico[self._type]
        result['_type'] = {'name':'_type', 'default_value':self._type }
        result.update(_get_menu(self.server_nb, self.name, self._type))
        result.update(self._get_form())
        result.update(self._get_valid_btn())
        d = {'template':'scribe_users', 'data':{'content':result, 'templates':used_templates}}
        return 0, uni(str(d))

    def _get_form(self):
        if self._type == 'pupil':
            return self._get_pupil_form()
        else:
            return self._get_teacher_form()

    def _get_pupil_form(self):
        """ renvoit la description de creation d'eleve 
        :user, name, forname, password, date: inputs de login, nom, prenom, mot de passe, date de naissance
        :civilite: select pour Mr/Mme/Mlle
        :user_number:input pour le numéro elenoet de l'élève
        :quota: input pour quotas disque attribué
        :profile: select pour profil utilisateur local/profil1/profil2
        :divcod: select pour le choix de la classe
        """
        user = {'name':'user', 'libelle':'Login (prenom.nom conseillé)',
                'onblur':"testVoid('user', 'error_user')", 'inline':'ok'}
        domains = [{'name':domain, 'libelle':domain}for domain in backend.get_domains()]
        domains[0]['default'] = 'ok'
        domains = {'name':'domains', 'libelle':'@', 'inline':'ok',
                   'options':[{'name':'', 'options':domains}]}
        name = {'name':'name', 'libelle':"Nom de l'utilisateur",
                'onblur':"testVoid('name', 'error_name')", 'inline':'ok'}
        forname = {'name':'forname', 'libelle':"Prénom de l'utilisateur",
                'onblur':"testVoid('forname', 'error_forname');", 'inline':'ok'}
        password = {'name':'password', 'libelle':"Mot de passe",
                    "onblur": "testVoid('password', 'error_password');", 'inline':'ok'}
        date = {'name':'date', 'libelle':"Date de naissance (format jj/mm/aaaa)",
                'onblur':"testVoid('date', 'error_date');", 'inline':'ok'}
        civilite_opts = [{'name':'1', 'libelle':'Mr.'}, {'name':'2', 'libelle':'Mme'}, {'name':'3', 'libelle':'Mlle'}]
        civilite = {'name':'civilite', 'options':[{'name':'', 'options':civilite_opts}],
                    'libelle':"Civilité", 'inline':'ok'}
        user_number = {'name':'user_number', 'libelle':"Numéro de l'élève (ELENOET dans GEP)", 'inline':'ok',
                'onblur':"testVoid('user_number', 'error_user_number')"}
        quota = {'name':'user_quota', 'libelle':"Quota disque (0 pour inactif)", 'inline':'ok',
                    'default_value':'0', 'onblur':"testVoid('user_quota', 'error_user_quota')"}
        options = [{'name':'1', 'libelle':'local'}, {'name':'2', 'libelle':"obligatoire - profil"},
                    {'name':'3', 'libelle':"obligatoire - profil2"}]
        profile = {'name':'_sambaProfilePath', 'libelle':"Profile utilisateur", 
                'options':[{'name':"Profil disponible", 'options':options}], 'inline':'ok'}
        classes = backend.get_classes()
        options = [{'name':key, 'options':[{'name':item, 'libelle':item}for item in items]}for key, items in classes.items()]
        divcod = {'name':'divcod', 'options':options,
                    'inline':'ok', 'libelle':"Classe"}
        return dict(user=user, domains=domains, name=name, forname=forname, password=password, date=date,
                civilite=civilite, user_number=user_number, quota=quota, 
                profile=profile, divcod=divcod)

    def _get_teacher_form(self):
        """ renvoit la description du formulaire de création de professeur 
        :user, name, forname, password: inputs de login nom prenom, mot de passe
        :civilite: select pour Mr/Mme/Mlle
        :mail:radios et input pour la configuration du mail
        :quota: input pour quotas disque attribué
        :profile: select pour profil utilisateur local/profil1/profil2
        :principal: select pour prof principal?
        :admin: checkbox pour administrateur?
        """
        user = {'name':'user', 'libelle':'Login (prenom.nom conseillé)',
               'onblur':"testVoid('user', 'error_user')", 'inline':'ok'}
        name = {'name':'name', 'libelle':"Nom de l'utilisateur",
               'onblur':"testVoid('name', 'error_name')", 'inline':'ok'}
        forname = {'name':'forname', 'libelle':"Prénom de l'utilisateur",
                'onblur':"testVoid('forname', 'error_forname');", 'inline':'ok'}
        password = {'name':'password', 'libelle':"Mot de passe",
                    "onblur": "testVoid('password', 'error_password');", 'inline':'ok'}
        civilite_opts = [{'name':'1', 'libelle':'Mr.'}, {'name':'2', 'libelle':'Mme'}, 
                        {'name':'3', 'libelle':'Mlle'}]
        civilite = {'name':'civilite', 'options':[{'name':'', 'options':civilite_opts}],
                    'libelle':"Civilité", 'inline':'ok'}
        default_domain = backend.get_domains()[1]
        mail = {'libelle':"Adresse e-mail (personalisée <b>ou</b> locale)",
                'radios':[{'name':'mailtype', 'value':'perso','inline':'ok'},
                    {'name':'mailtype', 'value':'local','inline':'ok', 'checked':'ok'}],
                'radios_libelle':['<b>Adresse personalisée :</b>', '<b>Adresse locale:</b> <br />login@%s'%default_domain],
                'input':{'name':"mailperso", "libelle":''}
                }
        quota = {'name':'user_quota', 'libelle':"Quota disque (0 pour inactif)", 'inline':'ok',
                    'default_value':'0', 'onblur':"testVoid('user_quota', 'error_user_quota')"}
        options = [{'name':'1', 'libelle':'local'}, {'name':'2', 'libelle':"obligatoire - profil"},
                    {'name':'3', 'libelle':"obligatoire - profil2"}]
        profile = {'name':'_sambaProfilePath', 'libelle':"Profile utilisateur", 
                'options':[{'name':"Profil disponible", 'options':options}], 'inline':'ok'}
        opts = [{'name':'', 'libelle':'non', 'default':'ok'}]
        opts = backend.get_classes()
        classes = [{'name':key, 'options':[{'name':item, 'libelle':"de la classe de %s"%item}for item in items]}for key, items in opts.items()]
        classes.insert(0,{'name':'', 'options':[{'name':'non', 'libelle':'non', 'default':'ok'}]})
        principal = {'name':'principal', 'libelle':'Prof principal', 'inline':'ok', 'options':classes}
        admin = {'name':'admin', 'libelle':"Membre du groupe DomainAdmins", 'inline':'ok', 'value':'admin'}
        return dict(user=user, name=name, forname=forname, password=password, civilite=civilite, mail=mail,
                quota=quota, profile=profile, principal=principal, admin=admin)

    def _get_valid_btn(self):
        """ bouton validere pour la creation d'user passe en meme temps le type en cour de creation eleve/prof"""
        validate={}
        validate['href'] = make_form_link(self.server_nb, self.name, True, ['current_type', 'add_user'])
        validate['icone'] = "/image/ok.gif"
        validate['libelle'] = "Valider"
        validate['title'] = "Créer cet utilisateur"
        return dict(validate=validate)

    def _valid_form(self):
        """ valider les formulaires """
        if self._type == 'pupil':
            return self.create_pupil()
        else:
            return self.create_teacher()

    def create_pupil(self):
        """ cree un élève """
        if not self.form_result.has_key('add_user'):
            return ''
        ## on teste le retour de toutes les valeurs
        formkeys = ['user', 'name', 'forname', 'password', 'domains', 'civilite', '_sambaProfilePath', 'divcod', 'user_number', 'user_quota', 'date']
        creation = format_form_result(self.form_result['add_user'])
        for key in formkeys: 
            if key not in creation: raise MissingKey, "Il manque une information : %s"%key
        ## on récupère les valeurs
        user, name, forname, password = creation['user'].strip(),  creation['name'].strip(),  creation['forname'].strip(),  creation['password'].strip()
        domain, civilite, profil, divcod = creation['domains'].strip(), creation['civilite'].strip(), creation['_sambaProfilePath'].strip(), creation['divcod'].strip()
        user_number, quota = creation['user_number'].strip(), creation['user_quota'].strip()
        date = creation['date'].strip()
        if not scribe_tools.test_date(date): raise MissingKey, "Erreur : le format de la date de naissance %s est incorrect."%date
        ## on teste les valeurs vides
        for a in[date, user_number,quota,domain,civilite, profil, divcod, user, name, forname, password]:
            if not a: raise MissingValue, "La variable %s n'est pas renseignée."%a
        for a in[domain,user, name, forname, password]:
            if not scribe_tools.test_ldap_entry(a): raise MissingValue, "La variable n'est pas au bon format %s."%a
        try:
            quota = int(quota)
        except:
            raise MissingValue, "La variable quota n'est pas un entier"
        ## on ajoute l'utilisateur
        a = backend.ajout_eleve(user, password, name, forname, date, divcod, user_number, civilite, domain,
                quota=quota, profil=profil)
        
        if a :
            return "Réussite de la création de l'élève %s %s"%(forname, name)
        else:
            return "Echec lors de la création de l'élève %s %s"%(forname, name)

    def create_teacher(self):
        """ cree un prof """
        if not self.form_result.has_key('add_user'):
            return ''
        creation = format_form_result(self.form_result['add_user'])
        formkeys = ['user', 'name', 'forname', 'password', 'civilite', 'user_quota', '_sambaProfilePath', 'mailtype', 'mailperso', 'principal']
        for val in formkeys:
            if val not in creation.keys():raise MissingKey, "Il manque une information : %s"%val
        user, name, forname, password = creation['user'].strip(),  creation['name'].strip(),  creation['forname'].strip(),  creation['password'].strip()
        civilite,  quota, profil = creation['civilite'].strip(), creation['user_quota'].strip(),  creation['_sambaProfilePath'].strip()
        mailtype, mailperso = creation['mailtype'].strip(), creation['mailperso'].strip()
        principal = creation['principal'].strip()
        for a in [user,name,forname,password,civilite,quota,profil, mailtype, principal]:
            if not a: raise MissingValue, "L'information %s n'est pas renseignée"%a
        for a in [user,name,forname,password,mailtype]:    
            if not scribe_tools.test_ldap_entry(a): raise MissingValue, "La variable n'est pas au bon format %s."%a
        if mailtype == 'perso': mail = mailperso
        else:mail = 'local'
        #FIXME: probleme de quota 
        a = backend.ajout_prof(user, password, name, forname, principal, civilite, mail, quota, profil)
        if a :
            if creation.has_key('admin'):
                backend.inscription(user, 'DomainAdmins')
            return "Réussite de la création du professeur %s %s"%(forname, name)
        else:
            return "Echec lors de la création du professeur %s %s"%(forname, name)

class UserListing(Action):
    """ liste les utilisateurs selon des filtres
    :alpha: alphabétique
    :domain: domaine mail
    :classe: classe de l'utilisateur
    :_type: eleve/prof
    :namepart: portion du nom des utilisateurs recherches
    :size: nombre d'utilisateur par page
    """
    user_description = Dict(default={}, doc="description de l'éxécutant", keys=['ip', 'name'])
    name = 'scribe_user_list'
    libelle = "Recherche d'utilisateur"
    category = "Gestion/Utilisateurs/"
    description = 'Lister des utilisateurs'
    request = Dict(default={}, doc="arguments de la requete en cour cote frontend", 
                    keys=['server', 'action', 'page', 'alpha_key', '_type', 'domain', '_class', 'size', 'namepart'])
    form_result = Dict(default={}, doc="Retour de formulaire en Json", keys=['search'])
    
    def execute(self):
        """ renvoit les données pour l'affichage """
        params, self.server_nb = get_request(self.request)
        result = {'titre':"Gestion des utilisateurs", "sstitre":self.description}
        self.user_alpha = 'a'
        page = 1 # permet de savoir quelle page de listing afficher en cas de nombreux utilisateurs
        if params.has_key('page'): # dans le cas de page multiple
            page = int(params['page'][0])
            params.pop('page')
        if params.has_key('alpha_key'):
            alpha_key, _type, domain = params['alpha_key'][0], params['_type'][0], params['domain'][0]
            _class, namepart = params['_class'][0], params['namepart'][0]
            size = int(params['size'][0]) #nombre d'item par page
            filtered_datas = self.filter_users(alpha_key, _type, domain, _class, namepart, size, page)
            return self.frag(filtered_datas)
        if self.form_result != {}:# validation de formulaire
            filtered_datas = self._valid_form()
            self.form_result = {}
            return self.frag(filtered_datas)
        result.update(_get_menu(self.server_nb, self.name))
        result.update(scribe_tools._get_user_listing_form())
        result.update({'validate':self._get_valid_btn()})
        d = {'template':'scribe_users', 'data':{'content':result, 'templates':used_templates}}
        return 0, uni(str(d))

    def frag(self, filtered_datas):
        """ renvoit les datas pour l'affichage de la liste des utilisateurs correpondant à la recherche"""
        result = filtered_datas
        d = {'template':'scribe_users', 'data':{'content':result, 'templates':used_templates}}
        return 0, uni(str(d))

    def _get_valid_btn(self):
        """ renvoie la description du bouton de validation de la recherche
            la cible du retour est la balise : user_div_container
        """
        icone = '/image/ok.gif'
        title = 'Lister selon les filtres'
        libelle = 'Lister'
        href = make_form_link(self.server_nb, self.name, True, ['search'], 'user_div_container')
        return dict(icone=icone, title=title, libelle=libelle, href=href)

    def _valid_form(self):
        """ valide les retours de formulaire """
        request = format_form_result(self.form_result['search'])
        keys = request.keys()
        if 'alpha_key' not in keys or 'domains' not in keys or 'user_class' not in keys or 'user_type' not in keys or 'namepart' not in keys or 'size' not in keys:
            return ""
        alpha_key, _type, domain, _class, namepart = request['alpha_key'], request['user_type'], request['domains'], request['user_class'], request['namepart']
        size = int(request['size'])
        return self.filter_users(alpha_key, _type, domain, _class, namepart, size)
    
    def filter_users(self, alpha_key="", _type="", domain="", _class="", namepart='', size=20, page=1):
        """ filtre les utilisateurs selon les parametres  on cree le filtre ldap et on lance une recherche
            :alpha_key: première lettre du nom de l'utilisateur
            :_type: type d'utilisateur prof/eleve (pupil/teacher)
            :domain: domaine mail
            :class: classe de l'utilisateur (pour les élèves à priori, marche aussi pour les profs admin de classe)
            :size: nombre d'utilisateur par page
        """
        log.msg("params de filtrage")
        log.msg(alpha_key,_type,domain,_class, namepart)
        namepart.strip()
        users = scribe_tools.user_filter(_type, domain, _class, namepart, alpha_key)
        if len(users)>size: # on affiche sur plusieurs pages 
            #FIXME: l'ordre reste-t-il le meme dans les resultats de requetes ldap
            max_index = size
            if page is not None:
                max_index = size * page
            min_index = max_index - size
            user_btns=self.get_btns(users, alpha_key, _type, domain, _class, namepart, size, page)
            users = self._get_table_users(users[min_index:max_index])
            return dict(users=users, user_btns=user_btns)
        users = self._get_table_users(users)
        return dict(users=users)

    def _get_table_users(self, users):
        """ renvoie la description du tableau de listing utilisateur
        """
        result = []
        for user in users:
            edit_pwd = {'title':'', 'libelle':"Changer le mot de passe",'icone':'', '_class':'scribe_simple_link',
                        'href': make_js_link(self.server_nb, 'scribe_user_password', balise='mod_user_div_container',
                            user=user)}
            modify = {'title':'', 'libelle':"Editer", 'icone':'','_class':'scribe_simple_link',
                            'href':make_js_link(self.server_nb, 'scribe_user_modify', balise='mod_user_div_container',
                                user=user)}
            delete = {'title':'', 'libelle':"Supprimer", "icone":"", "_class":'scribe_simple_link',
                            'href':make_js_link(self.server_nb, 'scribe_user_delete', balise='user_div_container', user=user)
                     }
            result.append({'name':user, 'pwd_link':edit_pwd, 'mod_link':modify, 'del_link':delete})
        return result

    def get_btns(self, users, alpha_key, _type, domain, _class, namepart, size, page):
        """ renvoit la description des boutons de passage de page en page.
            on passe les params de recherche """
        next = {'libelle': 'suivant', 'icone':'', 'title':'', 
                'href':make_js_link(self.server_nb, self.name, balise='user_div_container',
                                page=page+1, alpha_key=alpha_key, _type=_type, domain=domain, 
                                _class=_class, namepart=namepart, size=size)}
        prec = {'libelle': 'précédent', 'icone':'', 'title':'', 
                'href':make_js_link(self.server_nb, self.name, balise='user_div_container', 
                                page=page-1, alpha_key=alpha_key, _type=_type, domain=domain, 
                                _class=_class, namepart=namepart, size=size)}
        if page == 1 :
            return [next]
        elif ((page) * size)>=len(users):
            return [prec]
        else:
            return [prec, next]
            
class UserModify(Action):
    """ Modifie les utilisateurs """
    user_description = Dict(default={}, doc="description de l'éxécutant", keys=['ip', 'name'])
    name = 'scribe_user_modify'
    libelle = "Modification"
    category = None
    description = 'Modifier un utilisateur'
    request = Dict(default={}, doc="arguments de la requete en cour cote frontend", keys=['server', 'action', 'user'])
    form_result = Dict(default={}, doc="Retour de formulaire en Json", keys=['mod_user', 'current_type', 
                            "equipe_form", 'admin_class_form', 'matiere_form', 'groupe_form', 'spec_group_form'])
    
    def execute(self):
        """ renvoit les données pour l'affichage """
        params, self.server_nb = get_request(self.request)
        result = {}#'titre':"Gestion des utilisateurs"}
        self.user = ''
        self._type = ''
        datas = {}
        if params.has_key('user'):
            self.user = params['user'][0]
        if self.form_result != {}:
            if self.form_result.has_key('current_type'):
                self._type = format_form_result(self.form_result['current_type'])['_type']
                self.form_result.pop('current_type')
            try:                        
                result.update(self._valid_form())
            except (MissingKey, MissingValue), mess:
                result.update({'message':mess.message})
            except:
                result.update({'message':"Erreur inconnue dans la modification"})
            self.form_result ={}
        datas = self._get_user()
        result['_type'] = {'name':'_type', 'default_value':self._type }
        result.update(self._get_form(datas))
        result.update(self._get_valid_btn())
        d = {'template':'scribe_users', 'data':{'content':result, 'templates':used_templates}}
        return 0, uni(str(d))

    def _get_user(self):
        """ recupere les attributs de l'annuaire pour l'utilisateur courant, recupere le type au passage"""
        if self.user:
            datas = backend.get_user_data(self.user)
        else:
            return {}
        if backend.is_admin(self.user): self._type = 'teacher'
        else:self._type='pupil'
        return datas

### fonctions de description de formulaire        
    def _get_form(self, datas):
        """ renvoie la description du formulaire de modification d'utilisateur """
        if datas == {}: return {}
        if self._type == 'teacher':
            return self._get_teacher_form(datas)
        else:
            return self._get_pupil_form(datas)

    def _get_teacher_form(self, datas):
        """ renvoie la description du formulaire de modification de prof """
        #login
        user_login = {'name':'login', 'libelle':"Identifiant", 'disabled':'ok', 
                        'default_value':self.user, 'inline':'ok'}
        #identité
        if datas.has_key('displayName'):name = datas['displayName'][0]
        else:name=''
        user_name = {'name':'displayName', 'libelle':"Identité", 'default_value':name, 'inline':'ok'}
        #civilite 
        civilite_opts = [{'name':'1', 'libelle':'Mr.'}, {'name':'2', 'libelle':'Mme'}, 
                        {'name':'3', 'libelle':'Mlle'}]
        if datas.has_key('codecivilite'):civilite_opts[int(datas['codecivilite'][0])-1]['default'] = 'ok'
        civilite = {'name':'codecivilite', 'options':[{'name':'', 'options':civilite_opts}],
                    'libelle':"Civilité", 'inline':'ok'}
        #edition des mails
        _mail = "%s@%s"%(self.user, backend.get_domains()[1])
        mail = {'libelle':"Adresse e-mail (personalisée <b>ou</b> locale)",
                'radios':[{'name':'mailtype', 'value':'perso','inline':'ok'},
                    {'name':'mailtype', 'value':'local','inline':'ok'}],
                    'radios_libelle':['<b>Adresse personalisée :</b>', '<b>Adresse locale:</b> <br />%s'%_mail],
                    'input':{'name':"mailperso", "libelle":''}
                    }
        if datas.has_key('mailDir'):
            if datas['mailDir']:
                # un maildir est associé et non vide :c'est un compte local
                _mail = datas['mail'][0]
                mail['radios'][1]['checked'] = 'true'
                mail['radios_libelle'][1] = '<b>Adresse locale:</b> <br />%s'%_mail
        else:
            # compte mail perso
            if datas.has_key('mail'):
                _mail = datas['mail'][0]
                log.msg(mail['radios'][1])
                mail['radios'][0]['checked'] = 'true'
                mail['input']['default_value'] = _mail
        # quota
        _quota = backend.get_quota(self.user)
        quota =  {'libelle':"Quotas disque (0 pour inactif)", 'default_value':_quota, 'name':'quota', 'inline':'ok'}
        # profil
        options = [{'name':'1', 'libelle':'local'}, {'name':'2', 'libelle':"obligatoire - profil"},
                    {'name':'3', 'libelle':"obligatoire - profil2"}]
        profil = backend.get_profil(self.user)
        options[profil-1]['default'] = 'ok'
        profile = {'name':'_sambaProfilePath', 'libelle':"Profile utilisateur", 
                'options':[{'name':"Profil disponible", 'options':options}], 'inline':'ok'}
        # administration de classe
        if datas['typeadmin'][0] == '2': classes = backend.get_admin_classes(self.user)
        else: classes = []
        available_classes = backend.get_classes()
        for key, item in available_classes.items(): 
            if item in classes:available_classes[key].remove(item)
        liste = {'title':'Classes à administrer', 'formid':'admin_class_form','id':'admin_class_table',
                 'datas':classes}
        available = [{'name':level, 'options':[{'name':cl, 'libelle':cl}for cl in _class]}for level,_class in available_classes.items()]
        select = {'name':'admin_class_select', 
                'options':available, 'libelle':"classe disponibles", 'id':'admin_class_select'}
        classadmin = {'name':'classadmin', 'btns':self._get_transf_btns('admin_class'), 'select':select,
                        'liste':liste, 'title':"Administration de classes"}
        ## attachement à un groupe
        user_groupes = [gr for gr in backend.get_user_groups(self.user)if gr in backend.list_groups('groupe')]
        groupes_dispo = [gr for gr in backend.list_groups('groupe') if gr not in user_groupes]
        liste = {'title':"Groupes de l'enseignant", 'formid':'groupe_form','id':'groupe_table', 'datas':user_groupes}
        select = {'name':'groupe_select', 'libelle':"Groupes",
                'options':[{'name':'Groupes disponibles','options':[{'name':gr, 'libelle':gr}for gr in groupes_dispo]}],
                'id':'groupe_select' }
        groupe = {'name':"groupe", 'btns':self._get_transf_btns('groupe'), 'select':select,
                'liste':liste, 'title':"Groupes"}
        ## attachement à une matière
        user_matieres = [mat for mat in backend.get_user_groups(self.user)if mat in backend.list_groups('matiere')]
        matieres_dispo = [mat for mat in backend.list_groups('matiere') if mat not in user_matieres]
        liste = {'title':"Matières de l'enseignant", 'formid':'matiere_form','id':'matiere_table', 'datas':user_matieres}
        select = {'name':'matiere_select', 'libelle':"Matières",
                'options':[{'name':'Matières disponibles','options':[{'name':mat, 'libelle':mat}for mat in matieres_dispo]}],
                'id':'matiere_select' }
        matiere = {'name':"matiere", 'btns':self._get_transf_btns('matiere'), 'select':select,
                'liste':liste, 'title':"Matières dispensées"}
        ## attachement à une équipe  pédagogique
        user_equipes = [gr for gr in backend.get_user_groups(self.user)if gr in backend.list_groups('equipe')]
        equipes_dispo = [gr for gr in backend.list_groups('equipe')if gr not in user_equipes]
        liste = {'title':"Equipes pédagogiques de l'enseignant", 'formid':'equipe_form', 'id':'equipe_table', 'datas':user_equipes}
        select = {'name':'equipe_select', 'libelle':"Equipes",
                'options':[{'name':'Equipes disponibles','options':[{'name':gr, 'libelle':gr}for gr in equipes_dispo]}],
                'id':'equipe_select'}
        equipe = {'name':"equipe", 'btns':self._get_transf_btns('equipe'), 'select':select,
                'liste':liste, 'title':"Equipes pédagogiques"}
        ## groupes speciaux
        spec_groupes = backend.list_groups('Administrateurs')
        user_spec_groupes = [gr for gr in backend.get_user_groups(self.user)if gr in spec_groupes]
        spec_groupes = [gr for gr in spec_groupes if gr not in user_spec_groupes]
        liste = {'title':"Groupes spéciaux de l'enseignant", 'formid':'spec_group_form', 'id':'spec_group_table', 'datas':user_spec_groupes}
        select = {'name':'spec_group_select', 'libelle':"Groupes",
                  'options':[{'name':'Groupes disponibles','options':[{'name':gr, 'libelle':gr}for gr in spec_groupes]}],
                  'id':'spec_group_select'}
        spec_group = {'name':"spec_group", 'btns':self._get_transf_btns('spec_group'), 'select':select,
                    'liste':liste, 'title':"Groupes spéciaux"}                
        return dict(mod_user_login=user_login,name=user_name, civilite=civilite, mail=mail, 
                        profile=profile, quota=quota, classadmin=classadmin, matiere=matiere,
                        equipe=equipe, groupe=groupe, spec_group=spec_group)

    def _get_pupil_form(self, datas):
        """ renvoie la description du formulaire de modification d'élève """
        user_login = {'name':'login', 'libelle':"Identifiant", 'disabled':'ok',
                        'default_value':self.user, 'inline':'ok'}
        if datas.has_key('displayName'):name = datas['displayName'][0]
        else:name=''
        user_name = {'name':'displayName', 'libelle':"Identité", 'default_value':name, 'inline':'ok'}
        civilite_opts = [{'name':'1', 'libelle':'Mr.'}, {'name':'2', 'libelle':'Mme'}, 
                        {'name':'3', 'libelle':'Mlle'}] 
        if datas.has_key('codecivilite'):civilite_opts[int(datas['codecivilite'][0])-1]['default'] = 'ok'
        civilite = {'name':'codecivilite', 'options':[{'name':'', 'options':civilite_opts}],
                    'libelle':"Civilité", 'inline':'ok'}
        #adresse 
        adress = "adresse non communiquee"
        if datas.has_key('postalAddress'):
            if datas['postalAddress'] != ['none']:
                adress = datas['postalAddress'][0]
        adresse = {'name':'postalAddress', 'libelle':"Adresse", 'default_value':adress, 'inline':'ok'}
        #mail et domaine
        mail = {'libelle':"Adresse e-mail",'name':"email", 'inline':'ok', 'disabled':'ok'}
        _domains = backend.get_domains()
        dom = ''
        if datas.has_key('mail'):
            mail['default_value'] = datas['mail'][0].split('@')[0]
            dom =  datas['mail'][0].split('@')[1]
            _domains.remove(dom)
        domains = [{'libelle':domain, 'name':domain}for domain in _domains]
        domains.append({'libelle':dom, 'name':dom, 'default':'ok'})
        maildomain = {'name':'maildomain', 'options':[{'name':'', 'options':domains}]}
        #mail parent
        mailparent = [{"name":'mail_parent1', 'libelle':"Mail parent n°1", 'inline':'ok'}]
        if datas.has_key('mailparent'): 
            mailparent[0]['default_value']=datas['mailparent'][0]
            mailparent.append({"name":'mail_parent2', 'libelle':"Mail parent n°2", "inline":"ok"})
            if len(datas['mailparent'])>1:mailparent[1]['default_value'] = datas['mailparent'][1]
        #num ELENOET
        num=''
        if datas.has_key('employeeNumber'): num = datas['employeeNumber'][0]
        user_number = {'name':'employeeNumber', 'libelle':"Numéro de l'élève (ELENOET dans GEP)",'inline':'ok',
                        "default_value":num}
        #date de naissance
        _date=''
        if datas.has_key('dateNaissance'): _date = backend.formate_date(datas['dateNaissance'][0])
        date = {'name':'dateNaissance', 'libelle':"Date de naissance", 'inline':'ok', "default_value":_date}
        #quota
        _quota = backend.get_quota(self.user)
        quota =  {'libelle':"Quotas disque (0 pour inactif)", 'default_value':_quota, 'name':'quota', 'inline':'ok'}
        #profil
        options = [{'name':'1', 'libelle':'local'}, {'name':'2', 'libelle':"obligatoire - profil"},
                    {'name':'3', 'libelle':"obligatoire - profil2"}]
        profil = backend.get_profil(self.user)
        options[profil-1]['default'] = 'ok'
        profile = {'name':'_sambaProfilePath', 'libelle':"Profil utilisateur", 
                'options':[{'name':"Profil disponible", 'options':options}], 'inline':'ok'}
        #nivo
        nivo = datas['Meflcf'][0]
        niveau = {'name':'Meflcf', 'libelle':"Niveau", "inline":"ok", "default_value":nivo, 'disabled':'ok'}
        #classe
        _classe = datas['Divcod'][0]
        opts = backend.get_classes()
        #on parcoure les nivo et les classes à la recherche de celle déjà configurée
        classes = []
        for key, items in opts.items():
            nivo = []
            for item in items:
                opt = {'name':item, 'libelle':item}
                if item == _classe:opt['default'] = 'ok'
                nivo.append(opt)
            classes.append({'name':key, 'options':nivo})
        classe = {'name':'_Divcod', 'libelle':"Classe", "inline":"ok", "options":classes}
        ## attachement à un groupe
        user_groupes = [gr for gr in backend.get_user_groups(self.user)if gr in backend.list_groups('groupe')]
        groupes_dispo = [gr for gr in backend.list_groups('groupe') if gr not in user_groupes]
        liste = {'title':"Groupes de l'élève", 'formid':'groupe_form','id':'groupe_table', 'datas':user_groupes}
        select = {'name':'groupe_select', 'libelle':"Groupes",
                'options':[{'name':'Groupes disponibles','options':[{'name':gr, 'libelle':gr}for gr in groupes_dispo]}],
                'id':'groupe_select' }
        groupe = {'name':"groupe", 'btns':self._get_transf_btns('groupe'), 'select':select,
                'liste':liste, 'title':"Groupes"}
        return dict(mod_user_login=user_login,name=user_name, civilite=civilite, maildomain=maildomain,
                    maileleve=mail, mailparent=mailparent, profile=profile, quota=quota, adresse=adresse,
                    user_number=user_number, divcod=classe, niveau=niveau, date=date, groupe=groupe)

### fonction de gestion du retour de formulaire de modification

    def _valid_form(self):
        if self._type == 'teacher':
            return self._modify_teacher()
        else:
            return self._modify_pupil()

    def _modify_teacher(self):
        """ modifie un prof """
        ## recupération des résultats de formulaire et formatage
        modification = format_form_result(self.form_result['mod_user'])
        self.user = modification['login']
        administration = format_form_result(self.form_result['admin_class_form'],'2' )
        groupe = format_form_result(self.form_result['groupe_form'], '2')
        matiere = format_form_result(self.form_result['matiere_form'], '2')
        equipe = format_form_result(self.form_result['equipe_form'], '2')
        spec_groupes = format_form_result(self.form_result['spec_group_form'], '2')
        ## filtrage pour n'avoir que les données modifiées
        modification = self._filter_teacher_modif(modification)
        administration = self._filter_teacher_admin(administration)
        groupe = self._filter_teacher_groupe(groupe)
        matiere = self._filter_teacher_matiere(matiere)
        equipe = self._filter_teacher_equipe(equipe)
        spec_groupes = self._filter_teacher_spec_groupe(spec_groupes)
        ## application des modifications
        log.msg('Modification de prof :')
        log.msg(modification)
        if modification: self._set_teacher_modification(modification)
        log.msg('Administration de :')
        log.msg(administration)
        self._set_teacher_administration(administration)
        log.msg('Groupes')
        log.msg(groupe)
        if groupe != ([],[]):self._set_teacher_groupe(groupe)
        log.msg('Enseigne : ')
        log.msg(matiere)
        if matiere != ([],[]):self._set_teacher_groupe(matiere)
        log.msg('Equipe péda')
        log.msg(equipe)
        if equipe!= ([],[]):self._set_teacher_groupe(equipe)
        log.msg('Groupe spé')
        log.msg(spec_groupes)
        if spec_groupes != ([],[]):self._set_teacher_groupe(spec_groupes)
        return dict(message="Les modifications ont bien été attribuée à %s"%self.user)
 
    def _modify_pupil(self):
        """ modifie un élève """
        ## recuperation des valeurs
        modification = format_form_result(self.form_result['mod_user'])
        self.user = modification['login']
        groupe = format_form_result(self.form_result['groupe_form'], '2')
        ## filtrage des donnees
        modification = self._filter_pupil_modif(modification)
        groupe = self._filter_teacher_groupe(groupe)
        ## application des changements
        log.msg("Modification d'élève")
        log.msg(modification)
        if modification: self._set_pupil_modification(modification)
        log.msg('Groupes')
        log.msg(groupe)
        if groupe != ([],[]):self._set_teacher_groupe(groupe)
        return dict(message="Les modifications ont bien été attribuée à %s"%self.user)

### fonctions de filtrage de données

    def _filter_pupil_modif(self, modif):
        """ filtre les modifications renvoie le dictionnaire de 
            modification en supprimant les champs inchangés"""
        datas = self._get_user()
        for key, item in datas.items():
            if key in modif.keys():
                if modif[key] == item[0]:
                    modif.pop(key)
        for key in modif.keys():modif[key].strip()
        for key, item in modif.items():
            if key in ['displayName', 'mailperso', 'mail_parent1', 'mail_parent2']:
                if not scribe_tools.test_ldap_attribute(item): raise MissingKey, "Erreur : Format d'entrée incorrect %s."%item
        #quota
        quota = backend.get_quota(self.user)
        if quota == modif['quota']:
            modif.pop('quota')
        else:
            modif['_quota'] = int(modif['quota'])
            modif.pop('quota')
        # profil
        profil = backend.get_profil(self.user)
        if modif.has_key('_sambaProfilePath'):
            modif['_sambaProfilePath'] = int(modif['_sambaProfilePath'])
            if modif['_sambaProfilePath'] == profil:
                modif.pop('_sambaProfilePath')
        #mail parent
        mailparent = [modif['mail_parent1']]
        if modif.has_key('mail_parent2'):mailparent.append(modif['mail_parent2'])
        if datas.has_key('mailparent'):
            if mailparent == datas['mailparent']: 
                modif.pop('mail_parent1')
                modif.pop('mail_parent2')
        # mail et domaine eleve
        domain = modif['maildomain']
        if datas.has_key('mail'):
            if domain == datas['mail'][0].split('@')[1]: modif.pop('maildomain')
        # date de naissance
        date = backend.deformate_date(modif['dateNaissance'])
        if datas['dateNaissance'][0] == date:modif.pop('dateNaissance')
        elif not scribe_tools.test_date(modif['dateNaissance']):
            modif.pop('dateNaissance') 
        else:
            modif['dateNaissance'] = date
        return modif

    def _filter_teacher_modif(self, modif):
        """ filtre les modifications renvoie le dictionnaire de 
            modification en supprimant les champs inchangés"""
        datas = self._get_user()
        for key, item in datas.items():
            if key in modif.keys():
                if modif[key] == item[0]:
                    modif.pop(key)
        for key in modif.keys():modif[key].strip()
        for key, item in modif.items():
            log.msg("ona va tester %s %s"%(key, item))
            if key in ['displayName', 'mailperso']:
                if not scribe_tools.test_ldap_attribute(item): raise MissingKey, "Erreur : Format d'entrée incorrect %s."%item
        ## quotas
        quota = backend.get_quota(self.user)
        if quota == modif['quota']:
            modif.pop('quota')
        else:
            modif['_quota'] = int(modif['quota'])
            modif.pop('quota')
        # profil
        profil = backend.get_profil(self.user)
        if modif.has_key('_sambaProfilePath'):
            modif['_sambaProfilePath'] = int(modif['_sambaProfilePath'])
            if modif['_sambaProfilePath'] == profil:
                modif.pop('_sambaProfilePath')
        ## mail modifies?
        if datas.has_key('mail') and modif.has_key('mailtype'):
            if modif['mailtype'] == 'perso':
                if datas['mail'][0] == modif['mailperso']:
                    modif.pop('mailperso')
                    modif.pop('mailtype')
            else:
                if datas.has_key('mailDir'):
                    modif.pop('mailperso')
                    modif.pop('mailtype')
        return modif

    def _filter_teacher_admin(self, admin):
        """ filtre les classes à administrer par l'enseignant
            renvoie le couple nouvelle classe de l'utilisateur et classes à quitter
        """
        return [cl for cl in admin.keys() if cl in backend.list_groups('classe')]
    
    def _filter_teacher_groupe(self, groupes):
        """ filtre les groupes auxquels l'enseignant appartient """
        user_groupes = [cl for cl in backend.get_user_groups(self.user) if cl in backend.list_groups('groupe')]
        return [cl for cl in groupes.keys() if cl not in user_groupes],[cl for cl in user_groupes if cl not in groupes.keys()]
    
    def _filter_teacher_matiere(self, matieres):
        """ filtre les matieres que l'enseignant dispense"""
        user_matieres = [cl for cl in backend.get_user_groups(self.user) if cl in backend.list_groups('matiere')]
        return [cl for cl in matieres.keys() if cl not in user_matieres],[cl for cl in user_matieres if cl not in matieres.keys()]
    
    def _filter_teacher_equipe(self, equipes):
        """ filtre les équipes auxquels l'enseignant appartient"""
        user_equipes = [cl for cl in backend.get_user_groups(self.user) if cl in backend.list_groups('equipe')]
        return [cl for cl in equipes.keys() if cl not in user_equipes],[cl for cl in user_equipes if cl not in equipes.keys()]
    def _filter_teacher_spec_groupe(self, groupes):
        """ filtre les groupes speciaux auxquels l'enseignant appartient """
        user_groupes = [cl for cl in backend.get_user_groups(self.user) if cl in backend.list_groups('Administrateurs')]
        return [cl for cl in groupes.keys() if cl not in user_groupes],[cl for cl in user_groupes if cl not in groupes.keys()]

### fonctions d'application des modifications

    def _set_pupil_modification(self, modif):
        """ modifie les attributs d'un élève"""
        datas = self._get_user()
        # modification des attributs simples
        for key, value in modif.items():
            if key in datas.keys():
                if not backend.set_attr(self.user, key, value): raise MissingKey, "Erreur : Problème de modification de l'attribut %s"%key
        #gestion des quotas:
        if modif.has_key('_quota'):backend.set_quota(self.user, modif['_quota'])
        # gestion du mail:
        if modif.has_key('maildomain') and modif.has_key('email'):
            mail = "%s@%s"%(modif['email'], modif['maildomain'])
            if datas.has_key('mail'):
                if mail != datas['mail'][0]:
                    #FIXME: attribution du mail à un user à priori c bon???
                    backend.mod_mail(self.user, 'local', mail=mail) 
        ## gestion du mail des parents
        parentmail = []
        if modif.has_key('mail_parent1'):
            parentmail.append(modif['mail_parent1'])
        if modif.has_key('mail_parent2'):
            parentmail.append(modif['mail_parent2'])
        if datas.has_key('mailparent'):configured=datas['mailparent']
        else:configured = None
        if parentmail != configured:
            backend.set_attr(self.user, 'mailparent', parentmail)
        ## changement de classe:
        if modif.has_key('_Divcod'):
            if modif['_Divcod'] != datas['Divcod']:
                backend.change_classe(self.user, modif['_Divcod'])
        ## profile
        if modif.has_key('_sambaProfilePath'):
            print "MODIFICAIOTN du PROFIL%s"%modif['_sambaProfilePath']
            backend.set_profil(self.user, modif['_sambaProfilePath'])
                  
    def _set_teacher_modification(self, modif):
        """ modifie les attributs d'un prof
            modif: dictionnaire {'clé':valeur} revenant du formulaire puis filtré
        """
        #FIXME: gestion des erreurs
        datas = self._get_user()
        for key, value in modif.items():
            if key in datas.keys():
                backend.set_attr(self.user, key, value)
        #gestion des quotas:
        if modif.has_key('_quota'):backend.set_quota(self.user, modif['_quota'])
        ## modification du mail
        #FIXME: attribution du mail à un user???
        if modif.has_key('mailtype'):
            mailtype = modif['mailtype']
            if modif.has_key('mailperso') and mailtype=='perso':mailperso = modif['mailperso']
            else:mailperso = ''
            if mailtype == 'perso' and mailperso == '': raise MissingKey, "Il manque l'adresse mail perso"
            backend.mod_mail(self.user, mailtype, mailperso)
        ## profile
        if modif.has_key('_sambaProfilePath'):
            backend.set_profil(self.user, modif['_sambaProfilePath'])
        ## quota
        if modif.has_key('quota'):
            backend.set_quota(self.user, int(modif['quota']))

    def _set_teacher_administration(self, administration):
        """ enregistre un prof comme administrateur d'une classe 
            :administration: liste de classes
            à priori il suffit d'inscrire les groupes pour supprimer ceux qui manquent
        """ 
        backend.set_admin_classes(self.user, administration)

    def _set_teacher_groupe(self, groupe):
        """ enregistre un prof dans une liste de groupes 
            :groupe: liste de groupes
        """ 
        for gr in groupe[0]:
            backend.inscription(self.user, gr)
        for gr in groupe[1]:
            backend.desinscription(self.user, gr)

### description des boutons pour la modification des utilisateurs
    def _get_valid_btn(self):
        """ renvoie la description du bouton valider """
        validate={}
        validate['href'] = make_form_link(self.server_nb, self.name, True, 
                    ['spec_group_form', 'mod_user', 'current_type', 'admin_class_form', 'matiere_form', 'equipe_form', 'groupe_form'], 'mod_user_div_container')
        validate['icone'] = "/image/ok.gif"
        validate['libelle'] = "Valider"
        validate['title'] = "Modifier cet utilisateur"
        return dict(validate=validate)

    
    def _get_transf_btns(self, racine='admin_class'):
        """ renvoit les differents btns de transfert de donnees d'une select à une liste 
            :racine:racine des noms html des balises concernées
        """
        href2 = "javascript:formRemoveFromList('%s_form');"%racine
        href1 = "javascript:formAddNameToList3('%s_select','%s_table');"%(racine, racine)
        icone2 = "/image/scribe/rightleft_red.png"
        icone1 = "/image/scribe/leftright.png"
        msg2 = "Retirer"
        msg1 = "Ajouter"
        btns = [{'href':href2, 'libelle':msg2, 'icone':icone2, 'title':msg2},
                {'href':href1, 'libelle':msg1, 'icone':icone1, 'title':msg1}]
        return btns

class UserDelete(Action):
    """ Supprime un utilisateur """
    user_description = Dict(default={}, doc="description de l'éxécutant", keys=['ip', 'name'])
    name = 'scribe_user_delete'
    libelle = "Suppression"
    category = None
    description = 'Supprimer un utilisateur'
    request = Dict(default={}, doc="arguments de la requete en cour cote frontend", keys=['server', 'action', 'user'])
    form_result = Dict(default={}, doc="Retour de formulaire en Json", keys=['user'])

    def execute(self):
        params, self.server_nb = get_request(self.request)
        self.user = ''
        result = {}
        if params.has_key('user'):
            self.user = params['user'][0].strip()
            try:
                result['message'] = self.del_user()
            except MissingValue, mess:
                result['message'] = mess.message
            result['toexec'] = make_form_link(self.server_nb, 'scribe_user_list', False, ['search'], 
                                                'user_div_container')
        d = {'template':'scribe_users', 'data':{'content':result, 'templates':used_templates}}
        return 0, uni(str(d))

    def del_user(self):
        """ supprime un utilisateur """
        if self.user == '': raise MissingValue, "Il manque un paramètre, impossible de supprimer"
        a = backend.del_user(self.user)
        return "L'utilisateur a bien été supprimé"

class UserModPwd(Action):
    """ Modifie le mot de passe d'un utilisateur """
    user_description = Dict(default={}, doc="description de l'éxécutant", keys=['ip', 'name'])
    name = 'scribe_user_password'
    libelle = "Mot de passe"
    category = None
    description = "Modification du mot de passe d'un utilisateur"
    request = Dict(default={}, doc="arguments de la requete en cour cote frontend", keys=['server', 'action', 'user'])
    form_result = Dict(default={}, doc="Retour de formulaire en Json", keys=['mod_pwd'])

    def execute(self):
        params, self.server_nb = get_request(self.request)
        self.user = ''
        result = {}
        if params.has_key('user'):
            self.user = params['user'][0].strip()
        if self.form_result != {}:
            try:
                result['message'] = self._valid_form()
            except (MissingValue, MissingKey), mess:
                result['message'] = mess.message
            self.form_result = {}
        result.update(self._get_form())
        d = {'template':'scribe_users', 'data':{'content':result, 'templates':used_templates}}
        return 0, uni(str(d))

    def _get_form(self):
        """ renvoie la description du formulaire d'édition de mot de passe """
        user_name = {'name':'user_name', 'default_value':self.user}
        pwd_input = {'name':'password', 'libelle':"Nouveau mot de passe", "inline":"ok", 
                        "onblur":"testVoid('password', 'error_password');"}
        re_pwd_input = {'name':'repassword', 'libelle':"Réentrez le mot de passe", "inline":"ok",
                            "onblur":"testVoid('password', 'error_password');"}
        return dict(user_name=user_name, pwd_input=pwd_input, title="Modification de mot de passe pour %s"%self.user,
                re_pwd_input=re_pwd_input, validate=self._get_validate_btn())

    def _get_validate_btn(self):
        href = make_form_link(self.server_nb, self.name, True, ['mod_pwd'], balise='mod_user_div_container')
        title = "Modifier le mot de passe"
        libelle = "Valider"
        icone = "/image/ok.gif"
        return dict(href=href, title=title, libelle=libelle, icone=icone)

    def _valid_form(self):
        """ valide la modification de mot de passe """
        if not self.form_result.has_key('mod_pwd'):
            raise MissingKey, "Erreur : Il manque des données pour la modification de mot de passe."
        resultat = format_form_result(self.form_result['mod_pwd'])
        if not resultat.has_key('password') or not resultat.has_key('repassword') or not resultat.has_key('user_name'):
            raise MissingKey, "Erreur : Il manque des données pour la modification de mot de passe."
        pwd1 = resultat['password'].strip()
        pwd2 = resultat['repassword'].strip()
        user_name = resultat['user_name'].strip()
        self.user = user_name
        if pwd1 != pwd2: raise MissingKey, "Erreur : Les mots de passe ne correspondent pas"
        if not backend.mod_password(user_name, pwd1):
            raise MissingKey, "Erreur : erreur lors de la modification du mot de passe de %s"%user_name
        return "Le mot de passe de %s a bien été modifié"%user_name
