# -*- coding: UTF-8 -*-
###########################################################################
# Eole NG - 2007  
# Copyright Pole de Competence Eole  (Ministere Education - Academie Dijon)
# Licence CeCill  cf /root/LicenceEole.txt
# eole@ac-dijon.fr 
#  
# Action ServerRestart et ServerStop 
#  
# Redémarre ou stoppe le serveur
#       
###########################################################################

"""Actions effectuant des manoeuvres lourdes sur le serveur"""
from twisted.python import log
from ead2.backend.lib.action import * # import des actions et des types
from ead2.lib.libead import uni
from ead2.lib.i18n_backend import _
from ead2.backend.actions.tools import get_request, make_js_link, make_confirm_link, record_action
import commands
from twisted.internet import reactor
# fixons un mode debug
debug = False
TIMEOUT = 8

class Serverconfigure(Action):
    """Reconfiguration du serveur
    """
    user_description = Dict(default={}, doc="description de l'éxécutant", keys=['ip', 'name'])
    name = 'server-configure'
    libelle = 'Reconfigurer'
    description = 'Reconfigurer le serveur'
    category = 'Système/Serveur'
    request = Dict(default={}, doc="arguments de la requete en cour cote frontend", keys=['server', 'action', 'immediat'])
    
    def execute(self):
        params, self.server_nb = get_request(self.request)
        result = {'titre':self.description}
        message = ''
        if params.has_key('immediat'):
            self.immediat = params['immediat']
            result['btn'] = ''
            message = "Le serveur va être reconfigurer et va peut être redémarrer \\nd'ici peu (%s secondes), vous allez être déconnecté."%TIMEOUT
            record_action("Reconfiguration du serveur", self.user_description)
            # pb de droits ?
            #reactor.callLater(TIMEOUT, self.reconfigure_server)
            self.reconfigure_server()
        else:
            result['btn'] = self._get_btn()
        result['retour_msg'] = message
        result['retour_btn']  = get_retour_btn()
        d = {'template':"serveur",'data':{'content':result}}
        return 0, uni(str(d))
    
    def reconfigure_server(self):
        """ reconfigurer le serveur """
        log.msg('On va reconfigurer le serveur')
        if debug:
            commands.getstatusoutput("cat /usr/share/ead2/config/config.py")
        else:
            commands.getstatusoutput("/usr/bin/reconfigure -ead2")

    def _get_btn(self):
        """ renvoit la description du bouton reconfigurer """
        libelle = self.description
        icone = "/image/redemarrer.gif"
        h = make_js_link(self.server_nb, self.name, confirm=True, immediat='True')
        href = make_confirm_link("Etes-vous sûr de vouloir reconfigurer le serveur ?", h)
        return {'href':href, 'icone':icone, 'libelle':libelle}

class ServerReboot(Action):
    """Redémarrage du serveur
    :immediat: arret immediat ou non
    """
    user_description = Dict(default={}, doc="description de l'éxécutant", keys=['ip', 'name'])
    name = 'server-reboot'
    libelle = 'Redémarrer'
    description = 'Redémarrer le serveur'
    category = 'Système/Serveur'
    request = Dict(default={}, doc="arguments de la requete en cour cote frontend", keys=['server', 'action', 'immediat'])
    
    def execute(self):
        params, self.server_nb = get_request(self.request)
        result = {'titre':self.description}
        message = ''
        if params.has_key('immediat'):
            self.immediat = params['immediat']
            result['btn'] = ''
            message = "Le serveur va redémarrer d'ici peu (%s secondes), vous allez être déconnecté."%TIMEOUT
            record_action("Redémarrage du serveur", self.user_description)
            reactor.callLater(TIMEOUT, self.reboot_server)
        else:
            result['btn'] = self._get_btn()
        result['retour_msg'] = message
        result['retour_btn']  = get_retour_btn()
        d = {'template':"serveur",'data':{'content':result}}
        return 0, uni(str(d))
    
    def reboot_server(self):
        """ redémarre le serveur """
        if debug:
            commands.getstatusoutput("/etc/init.d/ead-server restart")
        else:
            commands.getstatusoutput("/sbin/reboot")

    def _get_btn(self):
        """ renvoit la description du bouton redemarrer """
        libelle = "Redémarrer le Serveur"
        icone = "/image/redemarrer.gif"
        h = make_js_link(self.server_nb, self.name, confirm=True, immediat='True')
        href = make_confirm_link("Etes-vous sûr de vouloir redémarrer le serveur ?", h)
        return {'href':href, 'icone':icone, 'libelle':libelle}

class ServerStop(Action):
    """arret du serveur
    :immediat: arret immediat ou non
    """
    user_description = Dict(default={}, doc="description de l'éxécutant", keys=['ip', 'name'])
    name = 'server-stop'
    libelle = 'Arrêter'
    category = 'Système/Serveur'
    description = 'Stop the server'
    request = Dict(default={}, doc="arguments de la requete en cour cote frontend", keys=['server', 'action', 'immediat'])
 
    def execute(self):
        params, self.server_nb = get_request(self.request)
        result = {'titre':self.description}
        message = ''
        if params.has_key('immediat'):
            self.immediat = params['immediat']
            result['btn'] = ''
            message = "Le serveur va s'arrêter d'ici peu (%s secondes), vous allez être déconnecter."%TIMEOUT
            record_action("Arrêt du serveur", self.user_description)
            reactor.callLater(TIMEOUT, self.stop_server)
        else:
            return_code = 0
            result['btn'] = self._get_btn()
        result['retour_msg'] = message
        result['retour_btn']  = get_retour_btn()
        d = {'template':"serveur",'data':{'content':result}}
        return 0, uni(str(d))

    def stop_server(self):
        """ arrête le serveur """
        if debug:
            commands.getstatusoutput("/etc/init.d/ead-server stop")
        else:
            commands.getstatusoutput("/sbin/halt")

    def _get_btn(self):
        """ renvoit la description du bouton redemarrer """
        libelle = "Arrêter le Serveur"
        icone = "/image/arreter.gif"
        h = make_js_link(self.server_nb, self.name, confirm=True, immediat='True')
        href = make_confirm_link("Etes-vous sûr de vouloir stopper le serveur ?", h)
        return {'href':href, 'icone':icone, 'libelle':libelle}
       
def get_retour_btn():
    return {'href': "javascript:Refresh()", 'icone': "/image/back.png", 'libelle':"Revenir à la page d'accueil"}
