# -*- coding: UTF-8 -*-
###########################################################################
# Eole NG - 2007  
# Copyright Pole de Competence Eole  (Ministere Education - Academie Dijon)
# Licence CeCill  cf /root/LicenceEole.txt
# eole@ac-dijon.fr 
#  
# Action ServiceRestart et ServiceStop 
#  
# Redémarre et stoppe des services
#       
###########################################################################
"""
Actions spécifiques au services qui tournent sur le serveur
"""
from twisted.python import log
from ead2.backend.lib.action import *
from ead2.lib.libead import uni
from ead2.lib.i18n_backend import _
from ead2.backend.config.config import SCRIPT_DIR
#import utiles pour l'execution
from nevow.url import URL
from twisted.internet.utils import getProcessOutput
from os import listdir
from os import path as osp
import commands
from ead2.lib.libbackend import clean_color_codes    
from ead2.backend.actions.tools import get_request, make_form_link, make_confirm_link, record_action, make_js_link
from creole import fonctionseole

tmp = listdir('/etc/init.d/')
if osp.isfile('/etc/sysconfig/eole/services'):
    tmp = file('/etc/sysconfig/eole/services', 'r')
    tmp = [i.strip() for i in tmp.readlines()]
list_services = [nom_serv for nom_serv in tmp]# if not nom_serv.endswith('~')]

class Daemon(Action):
    """ Arrete ou redemarre un demon """
    user_description = Dict(default={}, doc="description de l'éxécutant", keys=['ip', 'name'])
    name = 'daemon'
    libelle = "Services"
    description = "Stopper ou redémarrer un service"
    category = "Système"
    request = Dict(default={}, doc="arguments de la requete en cour cote frontend", 
                        keys=['server', 'action', 'service_restart', 'service_stop', 'all_restart'])

    def execute(self):
        params, self.server_nb = get_request(self.request)
        message = ''
        result = {'titre':self.description}
        if params.has_key('service_restart'):
            message = self.restart(params['service_restart'][0])
        elif params.has_key('service_stop'):
            message = self.stop(params['service_stop'][0])
        elif params.has_key('all_restart'):
            message = self.restart(list_services)
        result['message'] = message
        result['all_restart'] = self._get_all_restart_btn()
        result['services'] = self._get_services()
        result['retour_btn']  = get_retour_btn()
        d = {'template':'daemon', 'data':{'content':result}}
        return 0, uni(str(d))

    def _get_all_restart_btn(self):
        """ renvoie la description du bouton de redémarrage de tous les services """
        a = make_js_link(self.server_nb, self.name, confirm=True, all_restart='true')
        rest_link = {'href':make_confirm_link("Etes-vous sûr de vouloir redémarrer tous les services (sauf ead2)?",a),
                                    'libelle':'Redémarrer', 'icone': '/image/redemarrer_serv.gif'}
        return {"restart":rest_link, 'libelle':"Redémarrer tous les services (sauf ead2)"}

    def _get_services(self):
        """renvoit la liste des services et la description des boutons pour les redémarrer """
        result = []
        for serv in list_services:
            a = make_js_link(self.server_nb, self.name, confirm=True, service_restart=serv)
            rest_link = {'href':make_confirm_link("Etes-vous sûr de vouloir redémarrer ce service?", a),
                    'libelle':'Redémarrer', 'icone': '/image/redemarrer_serv.gif'}
            a = make_js_link(self.server_nb, self.name, confirm=True, service_stop=serv)
            stop_link = {'href':make_confirm_link("Etes-vous sûr de vouloir stopper ce service?", a),
                    'libelle':'Stopper', 'icone': '/image/arreter_serv.gif'}
            libelle = serv
            result.append({'restart':rest_link, 'stop':stop_link, 'libelle':libelle})
        return result

    def stop(self, service):
        log.msg('On arrete %s'%service)
        if service not in list_services: return "Service inconnu"
        return_code, output = commands.getstatusoutput("/etc/init.d/%s stop" %service)
        if return_code >= 256:
            return_code /= 256
        output = clean_color_codes(output)
        if return_code > 0:
            return "Erreur lors de l'arrêt du service : %s"%(service)
        else:
            return "Le service a été correctement stoppé : %s"%(service)

    def restart(self, service):
        log.msg('On redemmarre %s'%service)
        if type(service) == list:
            fonctionseole.restart_services(ead2=True)
            return "Tous les services (sauf ead-server et ead-web) ont été redémarré"
        if service not in list_services: return "Service inconnu"
        return_code, output = commands.getstatusoutput("/etc/init.d/%s restart" %service)
        if return_code >= 256:
            return_code /= 256
        output = clean_color_codes(output)
        if return_code > 0:
            return "Erreur lors de l'arrêt du service : %s"%(service)
        else:
            return "Le service a été correctement redémarré : %s"%(service)


class ServiceStop(Action):
    """
    arreter un service
    """
    user_description = Dict(default={}, doc="description de l'éxécutant", keys=['ip', 'name'])
    name = 'service-stop'
    libelle = 'Arrêter'
    description = 'Stoppe un service'
    category = None#'Système/Services'
    request = Dict(default={}, doc="arguments de la requete en cour cote frontend", keys=['server', 'action'])
    form_result = Dict(default={}, doc="Retour de formulaire en Json", keys=['service'])
    
    def execute(self):
        params, self.server_nb = get_request(self.request)
        result = {'titre':self.description}
        message = ''
        if self.form_result.has_key('service'):
            self.service = self._get_service(self.form_result['service'])
            result['btn'] = ''
            log.msg(self.service)
        # on vérifie que le service existe
            if not osp.isfile('/etc/init.d/%s' % self.service):
                message = "Error when stopping service %s : "%(self.service)
                return_code = 1
            else:
                record_action("Arrêt de %s"%self.service, self.user_description)
                return_code, output = commands.getstatusoutput("/etc/init.d/%s stop" % self.service)
                if return_code >= 256:
                    return_code /= 256
                output = clean_color_codes(output)
                if return_code > 0:
                    message = "Error when stopping service : %s"%(self.service)
                else:
                    message = "Service correctly stopped : %s"%(self.service)
        else:
            return_code = 0
            result['btn'] = self._get_btn()
            result['list_services'] = {'name':'service', 'formulaire':self._get_form()}
        result['retour_btn']  = get_retour_btn()
        result['retour_msg'] = message
        d = {'template':'services', 'data':{'content':result}}
        return return_code, uni(str(d))

    def _get_form(self):
        """ renvoit la description du formulaire de choix de service """
        tag1 = {'_type':'choice', 'name':'service_name', 'default_value': '',
                    'values':list_services, 'libelle':"Service à stopper"}
        form = [tag1]
        return form

    def _get_btn(self):
        """ renvoit la description du bouton redemarrer """
        libelle = "Stopper le Service"
        icone = "/image/arreter.gif"
        h = make_form_link(self.server_nb, self.name, False, ['service'])
        href = make_confirm_link("Etes-vous sûr de vouloir stopper ce service ?", h)
        return {'href':href, 'icone':icone, 'libelle':libelle}

    def _get_service(self, _list):
        """ renvoit le nom du service a redemarrer depuis le retour formulaire """
        for i in _list:
            if i['name'] == 'service_name':
                service = i['value']
        return service
        
class ServiceRestart(Action):
    """
    redemarrer un service
    """
    user_description = Dict(default={}, doc="description de l'éxécutant", keys=['ip', 'name'])
    name = 'service-restart'
    libelle = 'Redémarrer'
    description = 'Redémarre un service'
    category = None#'Système/Services'
    request = Dict(default={}, doc="arguments de la requete en cour cote frontend", keys=['server', 'action'])
    form_result = Dict(default={}, doc="Retour de formulaire en Json", keys=['service'])
    
    
    def execute(self):
        params, self.server_nb = get_request(self.request)
        result = {'titre':self.description}
        message = ''
        if self.form_result.has_key('service'):
            self.service = self._get_service(self.form_result['service'])
            result['btn'] = ''
            if not osp.isfile('/etc/init.d/%s' % self.service):
                message = "Error when stopping service %s : "%(self.service)
                return_code = 1
            else:    
                record_action("Redémarrage de %s"%self.service, self.user_description)
                return_code, output = commands.getstatusoutput("/etc/init.d/%s restart" % self.service)
                if return_code >= 256:
                    return_code /= 256
                    output = clean_color_codes(output)
                if return_code > 0:
                    message = "Error when restarting service"%(self.service)
                else:
                    message = "Service correctly restarted : %s"%(self.service)
        else:
            return_code = 0
            result['btn'] = self._get_btn()
            result['list_services'] = {'name':'service', 'formulaire':self._get_form()}
        result['retour_btn']  = get_retour_btn()
        result['retour_msg'] = message
        d = {'template':'services', 'data':{'content':result}}
        return return_code, uni(str(d))

    def _get_form(self):
        """ renvoit la description du formulaire de choix de service """
        tag1 = {'_type':'choice', 'name':'service_name', 'default_value': '',
                    'values':list_services, 'libelle':"Service à redémarrer"}
        form = [tag1]
        return form

    def _get_btn(self):
        """ renvoit la description du bouton redemarrer """
        libelle = "Redémarrer le Service"
        icone = "/image/redemarrer.gif"
        h = make_form_link(self.server_nb, self.name, False, ['service'])
        href = make_confirm_link("Etes-vous sûr de vouloir Redémarrer ce service ?", h)
        return {'href':href, 'icone':icone, 'libelle':libelle}

    def _get_service(self, _list):
        """ renvoit le nom du service a redemarrer depuis le retour formulaire """
        for i in _list:
            if i['name'] == 'service_name':
                service = i['value']
        return service

def get_retour_btn():
    return {'href': "javascript:Refresh()", 'libelle': "Revenir à la page d'acceuil", 'icone': "/image/back.png"}
