# -*- coding: UTF-8 -*-
###########################################################################
# Eole NG - 2007  
# Copyright Pole de Competence Eole  (Ministere Education - Academie Dijon)
# Licence CeCill  cf /root/LicenceEole.txt
# eole@ac-dijon.fr 
#  
# Outil de listing des fichiers de logs 
#  
#       
###########################################################################

""" Liste les fichiers de log de dansguardian et les classe par jour """
from ead2.backend.logparser.config import LOG_DIR, log_root_name
from os.path import join, isdir
from os import listdir
import datetime, re

def get_days(nums=1):
    """ renvoie un dico {'precweek': liste des jours de cette semaine, 'precweek': liste des jours de la semaine denière}
            avec les nums derniers jours dans l'ordre décroissant depuis aujourd'hui"""
    day_dict = ['lundi', 'mardi', 'mercredi', 'jeudi', 'vendredi', 'samedi', 'dimanche']
    timer = datetime.date
    today_index = timer.weekday(timer.today())
    if (today_index+1) > nums:
        firstday_index = (today_index+1) - nums
    else:
        firstday_index = 0
    thisweek = day_dict[firstday_index:(today_index+1)]
    thisweek.reverse()
    left_nums = today_index+1 - nums
    if left_nums < 0:
        precweek = day_dict[left_nums:]
        precweek.reverse()
    else:
        precweek = []
    return thisweek, precweek

def get_logs():
    loglist = [_file for _file in listdir(LOG_DIR) if log_root_name in _file and not '~' in _file]
    loglist.sort(compare_func)
    return loglist

def get_logs_for_form():
    """ renvoie l'ensemble des logs classés avec jour et semaine formaté pour la création de formulaire """
    loglist = get_logs()
    thisweek, precweek = get_days(len(loglist))
    first, second = [], []
    for day in thisweek:first.append({'libelle':day, 'name':loglist.pop(0)})
    for day in precweek:second.append({'libelle':day, 'name':loglist.pop(0)})
    return [{'name':'', 'options':[{'name':'vide', 'libelle':'-----'}]},
            {'name':"Cette semaine",'options':first}, {'name':"La semaine dernière", "options":second}]

## utilitaires de comparaison
def compare_func(a, b):
    """ fonction de comparaison """
    un = _get_nb_in_name(a)
    deux = _get_nb_in_name(b)
    return cmp(un, deux)

def _get_nb_in_name(name):
    """nombre present dans un nom (de fichier),
    utilisé pour parser 'access.log.1.gz', 'access.log.2.gz', 'access.log.3.gz',
    """
    regexp = "[0-9]+"
    if re.search(regexp, name):
        name = re.findall(regexp, name)
        return int(name[0])
    else:
        return -1
