# -*- coding: UTF-8 -*-
###########################################################################
# Eole NG - 2007  
# Copyright Pole de Competence Eole  (Ministere Education - Academie Dijon)
# Licence CeCill  cf /root/LicenceEole.txt
# eole@ac-dijon.fr 
#  
# Outil de parsing des fichiers de logs renvoie les lignes de logs par ip, 
# url, login, tranche horaire
#         
###########################################################################
""" Outils de parsing des fichiers de logs """
from ead2.backend.logparser.config import LOG_DIR, log_root_name, TEMP_FILENAME
from ead2.backend.logparser import file_listing
from os.path import join, isdir, splitext
import commands
from os import unlink

## fonction à utiliser de l'extérieur du module
def get_logs(filenames=[], ip=None, url=None, login=None, hours=(None, None)):
    """ renvoie les logs selon les critères fournis  !!!!!!fonction générale!!!!!!"""
    if hours == (None, None):
        return _get_filtered_logs(filenames, ip=ip, url=url, login=login)
    if None in hours:
        raise "Erreur : Il manque l'heure de début ou de fin."
    result = []
    beg, end = hours
    for filename in filenames:
        date = _get_file_date(filename)
        timestamps = _construct_datetime_range(date, beg, end)
        for timestamp in timestamps:
            parsed_logs = _get_filtered_log(filename, ip=ip, url=url, login=login, date=timestamp)
            result.extend(parsed_logs)
    return result


## parse les logs d'un ensemble de fichier
def _get_filtered_logs(filenames=[], url=None, ip=None, login=None):
    """ renvoie les logs filtrés pour les fichiers filenames """
    if filenames == []: filenames = file_listing.get_logs()
    result = []
    for filename in filenames:
        result.extend(_get_filtered_log(filename, url, ip, login))
    return result

## parse les logs de filename selon url, ip, login, date en supprimant les doublons (s'appuie sur parser.sh)
def _get_filtered_log(filename, url=None, ip=None, login=None, date=None):
    """ renvoie les logs filtrés selon les parametres envoyés """
    filepath = join(LOG_DIR,filename)
    cmd=''
    if splitext(filename)[1] == '.gz':   
        act_grep = 'zgrep'
        act_cat = 'zcat'
    else:
        act_grep = 'grep'
        act_cat = 'cat'
    ## on a des " dans les champs du csv, donc on doit les escaper 
    ## pour le passage de la commande au script(une fois pour python, une fois pour grep
    grep = act_grep+' \'%s\' ' + filepath
    othergrep = ' | grep \'%s\' '
    if url is not None:
        cmd += grep%url
        grep = othergrep
    if ip is not None:
        ip = '\\"%s\\"'%ip
        cmd += grep%ip
        grep = othergrep
    if login is not None:
        login = '\\"%s\\"'%login
        cmd += grep%login
        grep = othergrep
    if date is not None:
        cmd += grep%date
        grep = othergrep
    if cmd != '': 
        cmd = "/usr/share/ead2/backend/logparser/parser.sh \"%s\" \"%s\""%(cmd, TEMP_FILENAME)
    else:
        #FIXME: on passe par un fichier temporaire pour éviter de dézipper (utilisation de zcat)
        # on doit pouvoir faire mieux
        cmd = "%s %s >%s"%(act_cat, filepath, TEMP_FILENAME)
    return_code, return_value = commands.getstatusoutput(cmd)
    if return_code: raise Exception, "Erreur : le parsing en ligne de commande a échoué"
    grepped_datas = file(TEMP_FILENAME).read().splitlines()
    unlink(TEMP_FILENAME)
    parsed_datas = _parse_log_lines(grepped_datas)
    return parsed_datas

# cree un dico depuis des lignes de logs
def _parse_log_lines(datas):
    """ renvoie les logs sous forme de liste de dico """
    return [{'login':line.split(',')[1][1:-1], 'date':line.split(',')[0][1:-1], 'url':_get_domain(line.split(',')[3][1:-1]),
             'ip':line.split(',')[2][1:-1]}for line in datas if len(line.split(','))>3]

# recuper le nom de domaine d'une url
def _get_domain(url):
    """ renvoie le nom de domaine d'une url choisie """
    return url.split('/')[2]

### utilitaires pour le parsing par date
def _construct_datetime_range(date, beg, end):
    """ renvoie la liste des chaines "date heure:" qui correspondent à la beach hour (plage horaire) demandée 
    """
    try:
        beg = int(beg)
        end = int(end)
    except:
        raise Exception, "Erreur : Le début ou la fin de la plage horaire demandée ne sont pas des entiers."
    if beg>=end:
        raise Exception, "Erreur : Le début de la plage horaire demandée est supérieure ou égale à la fin."
    day, month, year = date
    plage = ["%s.%s.%s %s:"%(year,month,day, hour) for hour in range(beg,end)]
    return plage    

def _get_file_date(filename):
    """ renvoie la date d'un fichier de log en 
        parsant la première ligne
    """
    filepath = join(LOG_DIR,filename)
    if splitext(filename)[1] == '.gz':cmd = "zgrep -m1 , %s"%filepath
    else:cmd="grep -m1 , %s"%filepath
    return_code, first_file_line = commands.getstatusoutput(cmd)#'grep -m1 , %s'%filepath)
    if return_code:
        raise Exception, "Une erreur s'est produite au parsing de %s : %s"%(filepath, first_file_line) 
    date = first_file_line.split(',')[0].split()[0]
    return reformate_date(date)

def reformate_date(date):
    """ renvoie une date à un nouveau format: (jour,mois,année) depuis une date au format : '"année:mois:jour'  """
    year,month,day = date[1:].split('.')
    return day, month, year

    
#def get_url_logs(url):
#    """ renvoie les logs liés à une url donnée (passe outre le filtre qui supprime les récurences """
    

#### parser généraliste
#def _get_url_lines(filename, str):
#    """ renvoie les lignes du fichier filename comportant la ligne str
#        en évitant les récurences successives d'url (zappe les telechargement d'image depuis une page html)
#    """
#    print "On parse le fichier %s"%filename
#    filepath = join(LOG_DIR,filename)
#    if splitext(filename)[1] == '.gz':cmd = """zgrep '%s' %s| sort -t ',' -k 1,3 -u | awk -F ',' '{print $1","$2","$3","$4}'"""%(str, filepath)
#    else:cmd="""grep '%s' %s | sort -t ',' -k 1,3 -u | awk -F ',' '{print $1","$2","$3","$4}'"""%(str, filepath)
#    return_code, result = commands.getstatusoutput(cmd)
#    if return_code and return_code%255 != 1: 
#        raise Exception, "Erreur : erreur au parsing du fichier %s : %s"%(filename, result)
#    return [line.split(',') for line in result.split('\n')]
#
#def _get_lines(filename, str):
#    """ renvoie les lignes du fichier filename comportant la ligne str
#        en évitant les récurences successives d'url (zappe les telechargement d'image depuis une page html)
#    """
#    print "On parse le fichier %s"%filename
#    filepath = join(LOG_DIR,filename)
#    if splitext(filename)[1] == '.gz':cmd = """zgrep '%s' %s| sort -t ',' -k 1,3 -u | awk -F ',' '{print $1","$2","$3","$4}'"""%(str, filepath)
#    else:cmd="""grep '%s' %s | sort -t ',' -k 1,3 -u | awk -F ',' '{print $1","$2","$3","$4}'"""%(str, filepath)
#    return_code, result = commands.getstatusoutput(cmd)
#    if return_code and return_code%255 != 1: 
#        raise Exception, "Erreur : erreur au parsing du fichier %s : %s"%(filename, result)
#    return [line.split(',') for line in result.split('\n')]
#
#def parse_log_lines(lines):
#    """ renvoie une structure de données pour la mise en forme des logs """
#    result = []
#    current={'date':"##############", 'login':'', 'url':'', 'ip':''}
#    for line in lines:
#        if len(line)>3:
#            date=line[0][1:-1]
#            login=line[1][1:-1]
#            ip=line[2][1:-1]
#            url=line[3][1:-1]
#            if (current['date'] != date or current['login'] != login or current['ip'] == ip)and current['url'] not in url:
#                current = {'date':date, 'login':login, 'ip':ip, 'url':url.split('/')[2]}
#                if not os.path.splitext(line[3][1:-1])[1] in ['.js', '.css', '.png', '.gif', '.jpg','.jpeg', '.ico']:
#                    result.append(current)
#    return result
#
#    
#def _get_all_lines(filename):
#    """ renvoie les lignes du fichier filename
#    """
#    filepath = join(LOG_DIR,filename)
#    if splitext(filename)[1] == '.gz':cmd = "zcat '%s' %s"%(str, filepath)
#    else:cmd="cat '%s' %s"%(str, filepath)
#    return_code, result = commands.getstatusoutput(cmd)
#    if return_code and return_code%255 != 1: 
#        raise Exception, "Erreur : erreur au parsing du fichier %s : %s"%(filename, result)
#    return result.split('\n')
#
#def _get_log(filename):
#    """ renvoie les logs pour le fichier filename """
#    lines = _get_all_lines(filename)
#    log_lines = [line.split(',') for line in lines]
#    return log_lines
#    return parse_log_lines(log_lines)
#
#def _get_logs(filenames=[]):
#    """ renvoie les logs l'ensemble des logs dispo (par def) de la liste de fichier sinon"""
#    if filenames == []:filenames = file_listing.get_logs()
#    result = []
#    for filename in filenames:
#        result.extend(_get_log(filename))
#    return result
#
#### parser d'ips
#def _filter_ip_lines(lines, ip):
#    """ filtre une deuxième fois les lignes pour vérifier que ce sont les logs de l'ip demandée """
#    try:
#        result = []
#        for line in lines:
#            if len(line)>2:
#                if ip in line[2]:
#                    result.append(line)
#        return result
#    except:
#        raise Exception, "Erreur au parsing des logs pour l'ip %s."%ip
#
#def _get_log_by_ip(filename, ip):
#    """ renvoie les logs du fichier filename pour l'ip ip sous forme de dico """
#    lines = _get_lines(filename,ip)
#    ip_lines = _filter_ip_lines(lines, ip)
#    #return parse_log_lines(ip_lines)
#    return ip_lines
#
#def _get_logs_by_ip(ip, filenames=[]):
#    """ renvoie les logs pour l'ip ip dde l'ensemble des logs dispo (par def) de la liste de fichier sinon"""
#    if filenames == []:filenames = file_listing.get_logs()
#    result = []
#    for filename in filenames:
#        result.extend(_get_log_by_ip(filename, ip))
#    return result
#
#### parser de login
#def filter_by_login(lines, login):
#    try:
#        result = []
#        for line in lines:
#            if len(line)>1:
#                if login in line[1]:
#                    result.append(line)
#        return result
#    except:
#        raise Exception, "Erreur au parsing des logs pour %s."%login
#
#def _get_log_by_login(filename, login):
#    """ renvoie les lignes de logs du login login """
#    login = '"%s"'%login
#    lines = _get_lines(filename, login)
#    login_lines = filter_by_login(lines, login)
#    #return parse_log_lines(login_lines)
#    return login_lines
#
#def _get_logs_by_login(login, filenames=[]):
#    """ renvoie les logs pour le login login de l'ensemble des logs dispo (par def) de la liste de fichier sinon"""
#    if filenames == []:filenames = file_listing.get_logs()
#    result = []
#    for filename in filenames:
#        result.extend(_get_log_by_login(filename, login))
#    return result
#
### parser par url
#def filter_by_url(lines, url):
#    """ verifie que l'url url est bien présente dans les lignes de logs"""
#    try:    
#        result = []
#        for line in lines:
#            if len(line)>3:
#                if url in line[3]:
#                    result.append(line)
#        return result
#    except:
#        import traceback
#        traceback.print_exc()
#        raise Exception, "Erreur au parsing des logs pour l'url %s."%url
#
#def _get_log_by_url(filename, url):
#    """ renvoie les lignes de logs contenant l'url url """
#    lines = _get_lines(filename, url)
#    url_lines = filter_by_url(lines, url)
#   # return parse_log_lines(url_lines)
#    return url_lines
#
#def _get_logs_by_url(url, filenames=[]):
#    """ renvoie les logs pour l'url url de l'ensemble des logs dispo (par def) de la liste de fichier sinon"""
#    if filenames == []:filenames = file_listing.get_logs()
#    result = []
#    for filename in filenames:
#        result.extend(_get_log_by_url(filename, url))
#    return result
#
### parser par tranche horaire
#def _filter_hour_lines(lines, hour):
#    """ filtre les lignes et vérifie la présence de hour en tant qu'heure"""
#    try:    
#        result = []
#        for line in lines:
#            if len(line)>0:
#                if hour in line[0]:
#                    result.append(line)
#        return result
#    except:
#        raise Exception, "Erreur au parsing des logs pour l'heure %s."%hour
#
#def _get_log_by_hours(filename, beg='0', end='24'):
#    """ renvoie les logs entre deb et fin du fichier filename """
#    date = _get_file_date(filename)
#    hours = _construct_datetime_range(date, beg, end)
#    hours_lines = []
#    for h in hours:
#        lines = _get_lines(filename, h)
#        hour_lines = _filter_hour_lines(lines,h)
#    #    hour_lines = parse_log_lines(hour_lines)
#        hours_lines.extend(hour_lines)
#    return hours_lines
#
#def _get_logs_by_hours(beg, end, filenames=[]):
#    """ renvoie les logs pour l'url url de l'ensemble des logs dispo (par def) de la liste de fichier sinon"""
#    if filenames == []:filenames = file_listing.get_logs()
#    result = []
#    for filename in filenames:
#        result.extend(_get_log_by_hours(filename, beg, end))
#    return result
#
#
### parser de fichier interdits
#def _get_log_by_denied(filename):
#    """ renvoie les tentatives de connexion refusée """
#    lines = _get_lines(filename, 'DENIED')
#    return lines
#
#def _get_logs_by_denied(filenames=[]):
#    """ renvoie les logs d'accès interditsl'ensemble des logs dispo (par def) de la liste de fichier sinon"""
#    if filenames == []:filenames = file_listing.get_logs()
#    result = []
#    for filename in filenames:
#        result.extend(_get_log_by_denied(filename))
#    return result
#
